@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   array.c -
 
-  $Author: matz $
-  $Date: 2000/06/05 09:00:39 $
+  $Author: eban $
+  $Date: 2000/09/12 07:08:40 $
   created at: Fri Aug  6 09:46:12 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "util.h"
@@ -38,18 +40,16 @@ memfill(mem, size, val)
     }
 }
 
-#define ARY_FREEZE   FL_USER1
-#define ARY_TMPLOCK  FL_USER2
+#define ARY_TMPLOCK  FL_USER1
 
 static void
 rb_ary_modify(ary)
     VALUE ary;
 {
-    if (FL_TEST(ary, ARY_FREEZE))
-	rb_raise(rb_eTypeError, "can't modify frozen array");
+    if (OBJ_FROZEN(ary)) rb_error_frozen("array");
     if (FL_TEST(ary, ARY_TMPLOCK))
 	rb_raise(rb_eTypeError, "can't modify array during sort");
-    if (!FL_TEST(ary, FL_TAINT) && rb_safe_level() >= 4)
+    if (!OBJ_TAINTED(ary) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify array");
 }
 
@@ -57,15 +57,14 @@ VALUE
 rb_ary_freeze(ary)
     VALUE ary;
 {
-    FL_SET(ary, ARY_FREEZE);
-    return ary;
+    return rb_obj_freeze(ary);
 }
 
 static VALUE
 rb_ary_frozen_p(ary)
     VALUE ary;
 {
-    if (FL_TEST(ary, ARY_FREEZE|ARY_TMPLOCK))
+    if (FL_TEST(ary, FL_FREEZE|ARY_TMPLOCK))
 	return Qtrue;
     return Qfalse;
 }
@@ -170,33 +169,42 @@ rb_ary_s_new(argc, argv, klass)
     VALUE *argv;
     VALUE klass;
 {
-    long len = 0;
-    VALUE size, val;
-    NEWOBJ(ary, struct RArray);
+    VALUE ary = rb_ary_new();
     OBJSETUP(ary, klass, T_ARRAY);
+    rb_obj_call_init(ary, argc, argv);
+
+    return ary;
+}
+
+static VALUE
+rb_ary_initialize(argc, argv, ary)
+    int argc;
+    VALUE *argv;
+    VALUE ary;
+{
+    long len;
+    VALUE size, val;
 
-    ary->len = 0;
-    ary->ptr = 0;
     if (rb_scan_args(argc, argv, "02", &size, &val) == 0) {
-	ary->capa = ARY_DEFAULT_SIZE;
+	return ary;
     }
-    else {
-	long capa = NUM2LONG(size);
 
-	if (capa < 0) {
-	    rb_raise(rb_eArgError, "negative array size");
-	}
-	if (capa > 0 && capa*sizeof(VALUE) <= 0) {
-	    rb_raise(rb_eArgError, "array size too big");
-	}
-	ary->capa = capa;
-	len = capa;
+    rb_ary_modify(ary);
+    len = NUM2LONG(size);
+    if (len < 0) {
+	rb_raise(rb_eArgError, "negative array size");
     }
-    ary->ptr = ALLOC_N(VALUE, ary->capa);
-    memfill(ary->ptr, len, val);
-    ary->len = len;
+    if (len > 0 && len*sizeof(VALUE) <= 0) {
+	rb_raise(rb_eArgError, "array size too big");
+    }
+    if (len > RARRAY(ary)->capa) {
+	RARRAY(ary)->capa = len;
+	REALLOC_N(RARRAY(ary)->ptr, VALUE, RARRAY(ary)->capa);
+    }
+    memfill(RARRAY(ary)->ptr, len, val);
+    RARRAY(ary)->len = len;
 
-    return (VALUE)ary;
+    return ary;
 }
 
 static VALUE
@@ -265,13 +273,20 @@ rb_ary_push(ary, item)
 }
 
 static VALUE
-rb_ary_push_method(argc, argv, ary)
+rb_ary_push_m(argc, argv, ary)
     int argc;
     VALUE *argv;
     VALUE ary;
 {
-    while (argc--) {
-	rb_ary_store(ary, RARRAY(ary)->len, *argv++);
+    if (argc > 0) {
+	long len = RARRAY(ary)->len;
+
+	--argc;
+	/* make rooms by copying the last item */
+	rb_ary_store(ary, len + argc, argv[argc]);
+
+	if (argc)		/* if any rest */
+	    MEMCPY(RARRAY(ary)->ptr + len, argv, VALUE, argc);
     }
     return ary;
 }
@@ -334,6 +349,28 @@ rb_ary_unshift(ary, item)
     return ary;
 }
 
+static VALUE
+rb_ary_unshift_m(argc, argv, ary)
+    int argc;
+    VALUE *argv;
+    VALUE ary;
+{
+    if (argc == 0) {
+	rb_raise(rb_eArgError, "wrong # of arguments(at least 1)");
+    }
+    if (argc > 0) {
+	long len = RARRAY(ary)->len;
+
+	/* make rooms by setting the last item */
+	rb_ary_store(ary, len + argc - 1, Qnil);
+
+	/* sliding items */
+	MEMMOVE(RARRAY(ary)->ptr + argc, RARRAY(ary)->ptr, VALUE, len);
+	MEMCPY(RARRAY(ary)->ptr, argv, VALUE, argc);
+    }
+    return ary;
+}
+
 VALUE
 rb_ary_entry(ary, offset)
     VALUE ary;
@@ -352,7 +389,7 @@ rb_ary_entry(ary, offset)
 }
 
 static VALUE
-rb_ary_subary(ary, beg, len)
+rb_ary_subseq(ary, beg, len)
     VALUE ary;
     long beg, len;
 {
@@ -394,7 +431,7 @@ rb_ary_aref(argc, argv, ary)
 	if (beg < 0) {
 	    beg = RARRAY(ary)->len + beg;
 	}
-	return rb_ary_subary(ary, beg, len);
+	return rb_ary_subseq(ary, beg, len);
     }
 
     /* special case - speeding up */
@@ -412,12 +449,35 @@ rb_ary_aref(argc, argv, ary)
 	  case Qnil:
 	    return Qnil;
 	  default:
-	    return rb_ary_subary(ary, beg, len);
+	    return rb_ary_subseq(ary, beg, len);
 	}
     }
     return rb_ary_entry(ary, NUM2LONG(arg1));
 }
 
+static VALUE
+rb_ary_at(ary, pos)
+    VALUE ary, pos;
+{
+    return rb_ary_entry(ary, NUM2LONG(pos));
+}
+
+static VALUE
+rb_ary_first(ary)
+    VALUE ary;
+{
+    if (RARRAY(ary)->len == 0) return Qnil;
+    return RARRAY(ary)->ptr[0];
+}
+
+static VALUE
+rb_ary_last(ary)
+    VALUE ary;
+{
+    if (RARRAY(ary)->len == 0) return Qnil;
+    return RARRAY(ary)->ptr[RARRAY(ary)->len-1];
+}
+
 static VALUE
 rb_ary_index(ary, val)
     VALUE ary;
@@ -480,8 +540,11 @@ rb_ary_replace(ary, beg, len, rpl)
 	len = RARRAY(ary)->len - beg;
     }
 
-    if (TYPE(rpl) != T_ARRAY) {
-	rpl = rb_Array(rpl);
+    if (NIL_P(rpl)) {
+	rpl = rb_ary_new2(0);
+    }
+    else if (TYPE(rpl) != T_ARRAY) {
+	rpl = rb_ary_new3(1, rpl);
     }
 
     rb_ary_modify(ary);
@@ -523,30 +586,32 @@ rb_ary_aset(argc, argv, ary)
     VALUE *argv;
     VALUE ary;
 {
-    VALUE arg1, arg2, arg3;
     long offset, beg, len;
 
-    if (rb_scan_args(argc, argv, "21", &arg1, &arg2, &arg3) == 3) {
-	rb_ary_replace(ary, NUM2LONG(arg1), NUM2LONG(arg2), arg3);
-	return arg3;
+    if (argc == 3) {
+	rb_ary_replace(ary, NUM2LONG(argv[0]), NUM2LONG(argv[1]), argv[2]);
+	return argv[2];
     }
-    else if (FIXNUM_P(arg1)) {
-	offset = FIX2LONG(arg1);
+    if (argc != 2) {
+	rb_raise(rb_eArgError, "wrong # of arguments(%d for 2)", argc);
+    }
+    if (FIXNUM_P(argv[0])) {
+	offset = FIX2LONG(argv[0]);
 	goto fixnum;
     }
-    else if (rb_range_beg_len(arg1, &beg, &len, RARRAY(ary)->len, 1)) {
+    else if (rb_range_beg_len(argv[0], &beg, &len, RARRAY(ary)->len, 1)) {
 	/* check if idx is Range */
-	rb_ary_replace(ary, beg, len, arg2);
-	return arg2;
+	rb_ary_replace(ary, beg, len, argv[1]);
+	return argv[1];
     }
-    if (TYPE(arg1) == T_BIGNUM) {
+    if (TYPE(argv[0]) == T_BIGNUM) {
 	rb_raise(rb_eIndexError, "index too big");
     }
 
-    offset = NUM2LONG(arg1);
+    offset = NUM2LONG(argv[0]);
   fixnum:
-    rb_ary_store(ary, offset, arg2);
-    return arg2;
+    rb_ary_store(ary, offset, argv[1]);
+    return argv[1];
 }
 
 VALUE
@@ -605,19 +670,12 @@ static VALUE
 rb_ary_clone(ary)
     VALUE ary;
 {
-    VALUE ary2 = rb_ary_new2(RARRAY(ary)->len);
+    VALUE clone = rb_ary_new2(RARRAY(ary)->len);
 
-    CLONESETUP(ary2, ary);
-    MEMCPY(RARRAY(ary2)->ptr, RARRAY(ary)->ptr, VALUE, RARRAY(ary)->len);
-    RARRAY(ary2)->len = RARRAY(ary)->len;
-    return ary2;
-}
-
-static VALUE
-rb_ary_dup(ary)
-    VALUE ary;
-{
-    return rb_ary_s_create(RARRAY(ary)->len, RARRAY(ary)->ptr, CLASS_OF(ary));
+    CLONESETUP(clone, ary);
+    MEMCPY(RARRAY(clone)->ptr, RARRAY(ary)->ptr, VALUE, RARRAY(ary)->len);
+    RARRAY(clone)->len = RARRAY(ary)->len;
+    return clone;
 }
 
 static VALUE
@@ -642,10 +700,15 @@ rb_ary_join(ary, sep)
     VALUE ary, sep;
 {
     long i;
+    int taint = 0;
     VALUE result, tmp;
+
     if (RARRAY(ary)->len == 0) return rb_str_new(0, 0);
+    if (OBJ_TAINTED(ary)) taint = 1;
+    if (OBJ_TAINTED(sep)) taint = 1;
 
     tmp = RARRAY(ary)->ptr[0];
+    if (OBJ_TAINTED(tmp)) taint = 1;
     switch (TYPE(tmp)) {
       case T_STRING:
 	result = rb_str_dup(tmp);
@@ -687,16 +750,17 @@ rb_ary_join(ary, sep)
 	  default:
 	    tmp = rb_obj_as_string(tmp);
 	}
-	if (!NIL_P(sep)) rb_str_concat(result, sep);
-	rb_str_cat(result, RSTRING(tmp)->ptr, RSTRING(tmp)->len);
-	if (OBJ_TAINTED(tmp)) OBJ_TAINT(result);
+	if (!NIL_P(sep)) rb_str_append(result, sep);
+	rb_str_append(result, tmp);
+	if (OBJ_TAINTED(tmp)) taint = 1;
     }
 
+    if (taint) OBJ_TAINT(result);
     return result;
 }
 
 static VALUE
-rb_ary_join_method(argc, argv, ary)
+rb_ary_join_m(argc, argv, ary)
     int argc;
     VALUE *argv;
     VALUE ary;
@@ -751,8 +815,8 @@ rb_protect_inspect(func, obj, arg)
     VALUE obj, arg;
 {
     struct inspect_arg iarg;
-
     VALUE inspect_tbl;
+    VALUE id;
 
     if (!inspect_key) {
 	inspect_key = rb_intern("__inspect_key__");
@@ -762,10 +826,15 @@ rb_protect_inspect(func, obj, arg)
 	inspect_tbl = rb_ary_new();
 	rb_thread_local_aset(rb_thread_current(), inspect_key, inspect_tbl);
     }
-    rb_ary_push(inspect_tbl, obj);
+    id = rb_obj_id(obj);
+    if (rb_ary_includes(inspect_tbl, id)) {
+	return (*func)(obj, arg);
+    }
+    rb_ary_push(inspect_tbl, id);
     iarg.func = func;
     iarg.arg1 = obj;
     iarg.arg2 = arg;
+
     return rb_ensure(inspect_call, (VALUE)&iarg, inspect_ensure, obj);
 }
 
@@ -778,13 +847,14 @@ rb_inspecting_p(obj)
     if (!inspect_key) return Qfalse;
     inspect_tbl = rb_thread_local_aref(rb_thread_current(), inspect_key);
     if (NIL_P(inspect_tbl)) return Qfalse;
-    return rb_ary_includes(inspect_tbl, obj);
+    return rb_ary_includes(inspect_tbl, rb_obj_id(obj));
 }
 
 static VALUE
 inspect_ary(ary)
     VALUE ary;
 {
+    int tainted = OBJ_TAINTED(ary);
     long i = 0;
     VALUE s, str;
 
@@ -792,11 +862,13 @@ inspect_ary(ary)
 
     for (i=0; i<RARRAY(ary)->len; i++) {
 	s = rb_inspect(RARRAY(ary)->ptr[i]);
-	if (i > 0) rb_str_cat(str, ", ", 2);
-	rb_str_cat(str, RSTRING(s)->ptr, RSTRING(s)->len);
+	tainted = OBJ_TAINTED(s);
+	if (i > 0) rb_str_cat2(str, ", ");
+	rb_str_append(str, s);
     }
     rb_str_cat(str, "]", 1);
 
+    if (tainted) OBJ_TAINT(str);
     return str;
 }
 
@@ -823,8 +895,8 @@ rb_ary_reverse(ary)
     VALUE *p1, *p2;
     VALUE tmp;
 
+    if (RARRAY(ary)->len <= 1) return ary;
     rb_ary_modify(ary);
-    if (RARRAY(ary)->len == 0) return ary;
 
     p1 = RARRAY(ary)->ptr;
     p2 = p1 + RARRAY(ary)->len - 1;	/* points last item */
@@ -840,10 +912,18 @@ rb_ary_reverse(ary)
 }
 
 static VALUE
-rb_ary_reverse_method(ary)
+rb_ary_reverse_bang(ary)
     VALUE ary;
 {
-    return rb_ary_reverse(rb_ary_dup(ary));
+    if (RARRAY(ary)->len <= 1) return Qnil;
+    return rb_ary_reverse(ary);
+}
+
+static VALUE
+rb_ary_reverse_m(ary)
+    VALUE ary;
+{
+    return rb_ary_reverse(rb_obj_dup(ary));
 }
 
 static ID cmp;
@@ -878,7 +958,7 @@ sort_internal(ary)
     VALUE ary;
 {
     qsort(RARRAY(ary)->ptr, RARRAY(ary)->len, sizeof(VALUE),
-	  rb_iterator_p()?sort_1:sort_2);
+	  rb_block_given_p()?sort_1:sort_2);
     return ary;
 }
 
@@ -894,9 +974,9 @@ VALUE
 rb_ary_sort_bang(ary)
     VALUE ary;
 {
-    if (RARRAY(ary)->len <= 1) return ary;
-
     rb_ary_modify(ary);
+    if (RARRAY(ary)->len <= 1) return Qnil;
+
     FL_SET(ary, ARY_TMPLOCK);	/* prohibit modification during sort */
     rb_ensure(sort_internal, ary, sort_unlock, ary);
     return ary;
@@ -906,8 +986,49 @@ VALUE
 rb_ary_sort(ary)
     VALUE ary;
 {
-    if (RARRAY(ary)->len == 0) return ary;
-    return rb_ary_sort_bang(rb_ary_dup(ary));
+    ary = rb_obj_dup(ary);
+    rb_ary_sort_bang(ary);
+    return ary;
+}
+
+static VALUE
+rb_ary_collect(ary)
+    VALUE ary;
+{
+    long len, i;
+    VALUE collect;
+
+    if (!rb_block_given_p()) {
+	return rb_obj_dup(ary);
+    }
+
+    len = RARRAY(ary)->len;
+    collect = rb_ary_new2(len);
+    for (i=0; i<len; i++) {
+	rb_ary_push(collect, rb_yield(RARRAY(ary)->ptr[i]));
+    }
+    return collect;
+}
+
+static VALUE
+rb_ary_collect_bang(ary)
+    VALUE ary;
+{
+    long i;
+
+    rb_ary_modify(ary);
+    for (i = 0; i < RARRAY(ary)->len; i++) {
+	RARRAY(ary)->ptr[i] = rb_yield(RARRAY(ary)->ptr[i]);
+    }
+    return ary;
+}
+
+static VALUE
+rb_ary_filter(ary)
+    VALUE ary;
+{
+    rb_warn("Array#filter is deprecated; use Array#collect!");
+    return rb_ary_collect_bang(ary);
 }
 
 VALUE
@@ -926,7 +1047,7 @@ rb_ary_delete(ary, item)
 	i2++;
     }
     if (RARRAY(ary)->len == i2) {
-	if (rb_iterator_p()) {
+	if (rb_block_given_p()) {
 	    return rb_yield(item);
 	}
 	return Qnil;
@@ -939,11 +1060,11 @@ rb_ary_delete(ary, item)
 }
 
 VALUE
-rb_ary_delete_at(ary, at)
+rb_ary_delete_at(ary, pos)
     VALUE ary;
-    VALUE at;
+    long pos;
 {
-    long i, pos = NUM2LONG(at), len = RARRAY(ary)->len;
+    long i, len = RARRAY(ary)->len;
     VALUE del = Qnil;
 
     rb_ary_modify(ary);
@@ -960,8 +1081,57 @@ rb_ary_delete_at(ary, at)
     return del;
 }
 
+VALUE
+rb_ary_delete_at_m(ary, pos)
+    VALUE ary, pos;
+{
+    return rb_ary_delete_at(ary, NUM2LONG(pos));
+}
+
 static VALUE
-rb_ary_delete_if(ary)
+rb_ary_slice_bang(argc, argv, ary)
+    int argc;
+    VALUE *argv;
+    VALUE ary;
+{
+    VALUE arg1, arg2;
+    long pos, len, i;
+
+    rb_ary_modify(ary);
+    if (rb_scan_args(argc, argv, "11", &arg1, &arg2) == 2) {
+	pos = NUM2LONG(arg1);
+	len = NUM2LONG(arg2);
+      delete_pos_len:
+	if (pos < 0) {
+	    pos = RARRAY(ary)->len + pos;
+	}
+	arg2 = rb_ary_subseq(ary, pos, len);
+	rb_ary_replace(ary, pos, len, Qnil);	/* Qnil/rb_ary_new2(0) */
+	return arg2;
+    }
+
+    if (!FIXNUM_P(arg1) && rb_range_beg_len(arg1, &pos, &len, RARRAY(ary)->len, 1)) {
+	goto delete_pos_len;
+    }
+
+    pos = NUM2LONG(arg1);
+    len = RARRAY(ary)->len;
+
+    if (pos >= len) return Qnil;
+    if (pos < 0) pos += len;
+    if (pos < 0) return Qnil;
+
+    arg2 = RARRAY(ary)->ptr[pos];
+    for (i = pos + 1; i < len; i++, pos++) {
+	RARRAY(ary)->ptr[pos] = RARRAY(ary)->ptr[i];
+    }
+    RARRAY(ary)->len = pos;
+
+    return arg2;
+}
+
+static VALUE
+rb_ary_reject_bang(ary)
     VALUE ary;
 {
     long i1, i2;
@@ -974,26 +1144,22 @@ rb_ary_delete_if(ary)
 	}
 	i2++;
     }
+    if (RARRAY(ary)->len == i2) return Qnil;
     RARRAY(ary)->len = i2;
 
     return ary;
 }
 
 static VALUE
-rb_ary_filter(ary)
+rb_ary_delete_if(ary)
     VALUE ary;
 {
-    long i;
-
-    rb_ary_modify(ary);
-    for (i = 0; i < RARRAY(ary)->len; i++) {
-	RARRAY(ary)->ptr[i] = rb_yield(RARRAY(ary)->ptr[i]);
-    }
+    rb_ary_reject_bang(ary);
     return ary;
 }
 
 static VALUE
-rb_ary_replace_method(ary, ary2)
+rb_ary_replace_m(ary, ary2)
     VALUE ary, ary2;
 {
     ary2 = to_ary(ary2);
@@ -1070,9 +1236,7 @@ rb_ary_plus(x, y)
 {
     VALUE z;
 
-    if (TYPE(y) != T_ARRAY) {
-	return rb_ary_plus(x, rb_Array(y));
-    }
+    Check_Type(y, T_ARRAY);
 
     z = rb_ary_new2(RARRAY(x)->len + RARRAY(y)->len);
     MEMCPY(RARRAY(z)->ptr, RARRAY(x)->ptr, VALUE, RARRAY(x)->len);
@@ -1085,18 +1249,9 @@ VALUE
 rb_ary_concat(x, y)
     VALUE x, y;
 {
-    VALUE *p, *pend;
+    Check_Type(y, T_ARRAY);
 
-    if (TYPE(y) != T_ARRAY) {
-	return rb_ary_concat(x, rb_Array(y));
-    }
-
-    p = RARRAY(y)->ptr;
-    pend = p + RARRAY(y)->len;
-    while (p < pend) {
-	rb_ary_store(x, RARRAY(x)->len, *p);
-	p++;
-    }
+    rb_ary_push_m(RARRAY(y)->len, RARRAY(y)->ptr, x);
     return x;
 }
 
@@ -1138,7 +1293,7 @@ rb_ary_assoc(ary, key)
     p = RARRAY(ary)->ptr; pend = p + RARRAY(ary)->len;
     while (p < pend) {
 	if (TYPE(*p) == T_ARRAY
-	    && RARRAY(*p)->len > 1
+	    && RARRAY(*p)->len > 0
 	    && rb_equal(RARRAY(*p)->ptr[0], key))
 	    return *p;
 	p++;
@@ -1200,11 +1355,12 @@ rb_ary_hash(ary)
     VALUE ary;
 {
     long i;
-    int h;
+    int n, h;
 
     h = RARRAY(ary)->len;
     for (i=0; i<RARRAY(ary)->len; i++) {
-	int n = rb_hash(RARRAY(ary)->ptr[i]);
+	h = (h<<1) | (h<0 ? 1 : 0);
+	n = rb_hash(RARRAY(ary)->ptr[i]);
 	h ^= NUM2LONG(n);
     }
     return INT2FIX(h);
@@ -1292,7 +1448,7 @@ rb_ary_or(ary1, ary2)
 
     if (TYPE(ary2) != T_ARRAY) {
 	if (rb_ary_includes(ary1, ary2)) return ary1;
-	else return rb_ary_plus(ary1, ary2);
+	else return rb_ary_push(ary1, ary2);
     }
 
     ary3 = rb_ary_new();
@@ -1340,10 +1496,9 @@ static VALUE
 rb_ary_uniq(ary)
     VALUE ary;
 {
-    VALUE v = rb_ary_uniq_bang(rb_ary_dup(ary));
-
-    if (NIL_P(v)) return ary;
-    return v;
+    ary = rb_obj_dup(ary);
+    rb_ary_uniq_bang(ary);
+    return ary;
 }
 
 static VALUE
@@ -1372,10 +1527,9 @@ static VALUE
 rb_ary_compact(ary)
     VALUE ary;
 {
-    VALUE v = rb_ary_compact_bang(rb_ary_dup(ary));
-
-    if (NIL_P(v)) return ary;
-    return v;
+    ary = rb_obj_dup(ary);
+    rb_ary_compact_bang(ary);
+    return ary;
 }
 
 static VALUE
@@ -1400,11 +1554,26 @@ rb_ary_flatten_bang(ary)
 {
     long i;
     int mod = 0;
+    VALUE flattening = Qnil;
 
     rb_ary_modify(ary);
     for (i=0; i<RARRAY(ary)->len; i++) {
 	VALUE ary2 = RARRAY(ary)->ptr[i];
 	if (TYPE(ary2) == T_ARRAY) {
+	    if (ary == ary2) {
+		ary2 = Qnil;
+	    } else {
+		VALUE id;
+
+		if (NIL_P(flattening)) {
+		    flattening = rb_ary_new();
+		}
+		id = rb_obj_id(ary2);
+		if (rb_ary_includes(flattening, id)) {
+		    rb_raise(rb_eArgError, "tryed to flatten recursive array");
+		}
+		rb_ary_push(flattening, id);
+	    }
 	    rb_ary_replace(ary, i--, 1, ary2);
 	    mod = 1;
 	}
@@ -1417,10 +1586,9 @@ static VALUE
 rb_ary_flatten(ary)
     VALUE ary;
 {
-    VALUE v = rb_ary_flatten_bang(rb_ary_dup(ary));
-
-    if (NIL_P(v)) return ary;
-    return v;
+    ary = rb_obj_dup(ary);
+    rb_ary_flatten_bang(ary);
+    return ary;
 }
 
 void
@@ -1431,12 +1599,11 @@ Init_Array()
 
     rb_define_singleton_method(rb_cArray, "new", rb_ary_s_new, -1);
     rb_define_singleton_method(rb_cArray, "[]", rb_ary_s_create, -1);
+    rb_define_method(rb_cArray, "initialize", rb_ary_initialize, -1);
     rb_define_method(rb_cArray, "to_s", rb_ary_to_s, 0);
     rb_define_method(rb_cArray, "inspect", rb_ary_inspect, 0);
     rb_define_method(rb_cArray, "to_a", rb_ary_to_a, 0);
     rb_define_method(rb_cArray, "to_ary", rb_ary_to_a, 0);
-
-    rb_define_method(rb_cArray, "freeze",  rb_ary_freeze, 0);
     rb_define_method(rb_cArray, "frozen?",  rb_ary_frozen_p, 0);
 
     rb_define_method(rb_cArray, "==", rb_ary_equal, 1);
@@ -1446,12 +1613,15 @@ Init_Array()
 
     rb_define_method(rb_cArray, "[]", rb_ary_aref, -1);
     rb_define_method(rb_cArray, "[]=", rb_ary_aset, -1);
+    rb_define_method(rb_cArray, "at", rb_ary_at, 1);
+    rb_define_method(rb_cArray, "first", rb_ary_first, 0);
+    rb_define_method(rb_cArray, "last", rb_ary_last, 0);
     rb_define_method(rb_cArray, "concat", rb_ary_concat, 1);
     rb_define_method(rb_cArray, "<<", rb_ary_push, 1);
-    rb_define_method(rb_cArray, "push", rb_ary_push_method, -1);
+    rb_define_method(rb_cArray, "push", rb_ary_push_m, -1);
     rb_define_method(rb_cArray, "pop", rb_ary_pop, 0);
     rb_define_method(rb_cArray, "shift", rb_ary_shift, 0);
-    rb_define_method(rb_cArray, "unshift", rb_ary_unshift, 1);
+    rb_define_method(rb_cArray, "unshift", rb_ary_unshift_m, -1);
     rb_define_method(rb_cArray, "each", rb_ary_each, 0);
     rb_define_method(rb_cArray, "each_index", rb_ary_each_index, 0);
     rb_define_method(rb_cArray, "reverse_each", rb_ary_reverse_each, 0);
@@ -1463,24 +1633,28 @@ Init_Array()
     rb_define_method(rb_cArray, "indexes", rb_ary_indexes, -1);
     rb_define_method(rb_cArray, "indices", rb_ary_indexes, -1);
     rb_define_method(rb_cArray, "clone", rb_ary_clone, 0);
-    rb_define_method(rb_cArray, "dup", rb_ary_dup, 0);
-    rb_define_method(rb_cArray, "join", rb_ary_join_method, -1);
-    rb_define_method(rb_cArray, "reverse", rb_ary_reverse_method, 0);
-    rb_define_method(rb_cArray, "reverse!", rb_ary_reverse, 0);
+    rb_define_method(rb_cArray, "join", rb_ary_join_m, -1);
+    rb_define_method(rb_cArray, "reverse", rb_ary_reverse_m, 0);
+    rb_define_method(rb_cArray, "reverse!", rb_ary_reverse_bang, 0);
     rb_define_method(rb_cArray, "sort", rb_ary_sort, 0);
     rb_define_method(rb_cArray, "sort!", rb_ary_sort_bang, 0);
-    rb_define_method(rb_cArray, "delete", rb_ary_delete, 1);
-    rb_define_method(rb_cArray, "delete_at", rb_ary_delete_at, 1);
-    rb_define_method(rb_cArray, "delete_if", rb_ary_delete_if, 0);
-    rb_define_method(rb_cArray, "reject!", rb_ary_delete_if, 0);
+    rb_define_method(rb_cArray, "collect", rb_ary_collect, 0);
+    rb_define_method(rb_cArray, "collect!", rb_ary_collect_bang, 0);
+    rb_define_method(rb_cArray, "map!", rb_ary_collect_bang, 0);
     rb_define_method(rb_cArray, "filter", rb_ary_filter, 0);
-    rb_define_method(rb_cArray, "collect!", rb_ary_filter, 0);
-    rb_define_method(rb_cArray, "replace", rb_ary_replace_method, 1);
+    rb_define_method(rb_cArray, "delete", rb_ary_delete, 1);
+    rb_define_method(rb_cArray, "delete_at", rb_ary_delete_at_m, 1);
+    rb_define_method(rb_cArray, "delete_if", rb_ary_delete_if, 0);
+    rb_define_method(rb_cArray, "reject!", rb_ary_reject_bang, 0);
+    rb_define_method(rb_cArray, "replace", rb_ary_replace_m, 1);
     rb_define_method(rb_cArray, "clear", rb_ary_clear, 0);
     rb_define_method(rb_cArray, "fill", rb_ary_fill, -1);
     rb_define_method(rb_cArray, "include?", rb_ary_includes, 1);
     rb_define_method(rb_cArray, "<=>", rb_ary_cmp, 1);
 
+    rb_define_method(rb_cArray, "slice", rb_ary_aref, -1);
+    rb_define_method(rb_cArray, "slice!", rb_ary_slice_bang, -1);
+
     rb_define_method(rb_cArray, "assoc", rb_ary_assoc, 1);
     rb_define_method(rb_cArray, "rassoc", rb_ary_rassoc, 1);
 
