@@ -1,12 +1,14 @@
-/************************************************
+/**********************************************************************
 
   bignum.c -
 
   $Author: matz $
-  $Date: 2000/06/23 16:06:50 $
+  $Date: 2000/09/15 06:00:22 $
   created at: Fri Jun 10 00:48:55 JST 1994
 
-************************************************/
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
+
+**********************************************************************/
 
 #include "ruby.h"
 #include <math.h>
@@ -84,22 +86,24 @@ static VALUE
 bignorm(x)
     VALUE x;
 {
-    long len = RBIGNUM(x)->len;
-    USHORT *ds = BDIGITS(x);
+    if (!FIXNUM_P(x)) {
+	long len = RBIGNUM(x)->len;
+	USHORT *ds = BDIGITS(x);
 
-    while (len-- && !ds[len]) ;
-    RBIGNUM(x)->len = ++len;
+	while (len-- && !ds[len]) ;
+	RBIGNUM(x)->len = ++len;
 
-    if (len*sizeof(USHORT) <= sizeof(VALUE)) {
-	long num = 0;
-	while (len--) {
-	    num = BIGUP(num) + ds[len];
-	}
-	if (num >= 0) {
-	    if (RBIGNUM(x)->sign) {
-		if (POSFIXABLE(num)) return INT2FIX(num);
+	if (len*sizeof(USHORT) <= sizeof(VALUE)) {
+	    long num = 0;
+	    while (len--) {
+		num = BIGUP(num) + ds[len];
+	    }
+	    if (num >= 0) {
+		if (RBIGNUM(x)->sign) {
+		    if (POSFIXABLE(num)) return INT2FIX(num);
+		}
+		else if (NEGFIXABLE(-(long)num)) return INT2FIX(-(long)num);
 	    }
-	    else if (NEGFIXABLE(-(long)num)) return INT2FIX(-(long)num);
 	}
     }
     return x;
@@ -169,7 +173,7 @@ rb_int2inum(n)
 }
 
 VALUE
-rb_str2inum(str, base)
+rb_cstr2inum(str, base)
     const char *str;
     int base;
 {
@@ -307,6 +311,21 @@ rb_str2inum(str, base)
     return bignorm(z);
 }
 
+VALUE
+rb_str2inum(str, base)
+    VALUE str;
+    int base;
+{
+    char *s;
+    int len;
+
+    s = rb_str2cstr(str, &len);
+    if (len != strlen(s)) {
+	rb_raise(rb_eArgError, "string for Integer contains null byte");
+    }
+    return rb_cstr2inum(s, base); 
+}
+
 static char hexmap[] = "0123456789abcdef";
 VALUE
 rb_big2str(x, base)
@@ -394,7 +413,7 @@ big2ulong(x, type)
     USHORT *ds;
 
     if (len > sizeof(long)/sizeof(USHORT))
-	rb_raise(rb_eArgError, "bignum too big to convert into `%s'", type);
+	rb_raise(rb_eRangeError, "bignum too big to convert into `%s'", type);
     ds = BDIGITS(x);
     num = 0;
     while (len--) {
@@ -421,19 +440,12 @@ rb_big2long(x)
     unsigned long num = big2ulong(x, "int");
 
     if ((long)num < 0) {
-	rb_raise(rb_eArgError, "bignum too big to convert into `int'");
+	rb_raise(rb_eRangeError, "bignum too big to convert into `int'");
     }
     if (!RBIGNUM(x)->sign) return -(long)num;
     return num;
 }
 
-static VALUE
-rb_big_to_i(x)
-    VALUE x;
-{
-    return bignorm(x);
-}
-
 static VALUE
 dbl2big(d)
     double d;
@@ -507,11 +519,11 @@ rb_big_cmp(x, y)
 	y = rb_int2big(FIX2LONG(y));
 	break;
 
-      case T_FLOAT:
-	y = dbl2big(RFLOAT(y)->value);
+      case T_BIGNUM:
 	break;
 
-      case T_BIGNUM:
+      case T_FLOAT:
+	y = dbl2big(RFLOAT(y)->value);
 	break;
 
       default:
@@ -550,7 +562,7 @@ rb_big_eq(x, y)
     }
     if (RBIGNUM(x)->sign != RBIGNUM(y)->sign) return Qfalse;
     if (RBIGNUM(x)->len != RBIGNUM(y)->len) return Qfalse;
-    if (memcmp(BDIGITS(x),BDIGITS(y),RBIGNUM(y)->len) != 0) return Qfalse;
+    if (MEMCMP(BDIGITS(x),BDIGITS(y),USHORT,RBIGNUM(y)->len) != 0) return Qfalse;
     return Qtrue;
 }
 
@@ -626,7 +638,7 @@ bigsub(x, y)
 	i++;
     }
     
-    return bignorm(z);
+    return z;
 }
 
 static VALUE
@@ -671,7 +683,7 @@ bigadd(x, y, sign)
     }
     BDIGITS(z)[i] = (USHORT)num;
 
-    return bignorm(z);
+    return z;
 }
 
 VALUE
@@ -683,7 +695,7 @@ rb_big_plus(x, y)
 	y = rb_int2big(FIX2LONG(y));
 	/* fall through */
       case T_BIGNUM:
-	return bigadd(x, y, 1);
+	return bignorm(bigadd(x, y, 1));
 
       case T_FLOAT:
 	return rb_float_new(rb_big2dbl(x) + RFLOAT(y)->value);
@@ -702,7 +714,7 @@ rb_big_minus(x, y)
 	y = rb_int2big(FIX2LONG(y));
 	/* fall through */
       case T_BIGNUM:
-	return bigadd(x, y, 0);
+	return bignorm(bigadd(x, y, 0));
 
       case T_FLOAT:
 	return rb_float_new(rb_big2dbl(x) - RFLOAT(y)->value);
@@ -760,10 +772,9 @@ rb_big_mul(x, y)
 }
 
 static void
-bigdivmod(x, y, div, mod, modulo)
+bigdivrem(x, y, divp, modp)
     VALUE x, y;
-    VALUE *div, *mod;
-    int modulo;
+    VALUE *divp, *modp;
 {
     long nx = RBIGNUM(x)->len, ny = RBIGNUM(y)->len;
     long i, j;
@@ -776,8 +787,8 @@ bigdivmod(x, y, div, mod, modulo)
     yds = BDIGITS(y);
     if (ny == 0 && yds[0] == 0) rb_num_zerodiv();
     if (nx < ny	|| nx == ny && BDIGITS(x)[nx - 1] < BDIGITS(y)[ny - 1]) {
-	if (div) *div = INT2FIX(0);
-	if (mod) *mod = bignorm(x);
+	if (divp) *divp = rb_int2big(0);
+	if (modp) *modp = x;
 	return;
     }
     xds = BDIGITS(x);
@@ -792,14 +803,9 @@ bigdivmod(x, y, div, mod, modulo)
 	    t2 %= dd;
 	}
 	RBIGNUM(z)->sign = RBIGNUM(x)->sign==RBIGNUM(y)->sign;
-	if (div) *div = bignorm(z);
-	if (mod) {
-	    if (!RBIGNUM(x)->sign) t2 = -(long)t2;
-	    if (modulo && RBIGNUM(x)->sign != RBIGNUM(y)->sign) {
-		t2 = t2 + yds[0] * (RBIGNUM(y)->sign ? 1 : -1);
-	    }
-	    *mod = INT2NUM(t2);
-	}
+	if (!RBIGNUM(x)->sign) t2 = -(long)t2;
+	if (modp) *modp = rb_int2big((long)t2);
+	if (divp) *divp = z;
 	return;
     }
     z = bignew(nx==ny?nx+2:nx+1, RBIGNUM(x)->sign==RBIGNUM(y)->sign);
@@ -860,18 +866,17 @@ bigdivmod(x, y, div, mod, modulo)
 	}
 	zds[j] = q;
     } while (--j >= ny);
-    if (div) {			/* move quotient down in z */
-	*div = rb_big_clone(z);
-	zds = BDIGITS(*div);
+    if (divp) {			/* move quotient down in z */
+	*divp = rb_big_clone(z);
+	zds = BDIGITS(*divp);
 	j = (nx==ny ? nx+2 : nx+1) - ny;
 	for (i = 0;i < j;i++) zds[i] = zds[i+ny];
-	RBIGNUM(*div)->len = i;
-	*div = bignorm(*div);
+	RBIGNUM(*divp)->len = i;
     }
-    if (mod) {			/* just normalize remainder */
-	*mod = rb_big_clone(z);
+    if (modp) {			/* just normalize remainder */
+	*modp = rb_big_clone(z);
 	if (dd) {
-	    zds = BDIGITS(*mod);
+	    zds = BDIGITS(*modp);
 	    t2 = 0; i = ny;
 	    while(i--) {
 		t2 = BIGUP(t2) + zds[i];
@@ -879,13 +884,26 @@ bigdivmod(x, y, div, mod, modulo)
 		t2 %= dd;
 	    }
 	}
-	RBIGNUM(*mod)->len = ny;
-	RBIGNUM(*mod)->sign = RBIGNUM(x)->sign;
-	if (modulo && RBIGNUM(x)->sign != RBIGNUM(y)->sign) {
-	    *mod = bigadd(*mod, y, 1);
-	    return;
-	}
-	*mod = bignorm(*mod);
+	RBIGNUM(*modp)->len = ny;
+	RBIGNUM(*modp)->sign = RBIGNUM(x)->sign;
+    }
+}
+
+static void
+bigdivmod(x, y, divp, modp)
+    VALUE x, y;
+    VALUE *divp, *modp;
+{
+    VALUE mod;
+
+    bigdivrem(x, y, divp, &mod);
+    if (RBIGNUM(x)->sign != RBIGNUM(y)->sign && RBIGNUM(mod)->len > 0) {
+	if (divp) *divp = bigadd(*divp, rb_int2big(1), 0);
+	if (modp) *modp = bigadd(mod, y, 1);
+    }
+    else {
+	if (divp) *divp = *divp;
+	if (modp) *modp = mod;
     }
 }
 
@@ -909,16 +927,15 @@ rb_big_div(x, y)
       default:
 	return rb_num_coerce_bin(x, y);
     }
-    bigdivmod(x, y, &z, 0, 0);
+    bigdivmod(x, y, &z, 0);
 
-    return z;
+    return bignorm(z);
 }
 
 
 static VALUE
-rb_big_modulo(x, y, modulo)
+rb_big_modulo(x, y)
     VALUE x, y;
-    int modulo;
 {
     VALUE z;
 
@@ -930,33 +947,37 @@ rb_big_modulo(x, y, modulo)
       case T_BIGNUM:
 	break;
 
-      case T_FLOAT:
-	y = dbl2big(RFLOAT(y)->value);
-	break;
-
       default:
 	return rb_num_coerce_bin(x, y);
     }
-    bigdivmod(x, y, 0, &z, modulo);
+    bigdivmod(x, y, 0, &z);
 
-    return z;
-}
-
-static VALUE
-rb_big_mod(x, y)
-    VALUE x, y;
-{
-    return rb_big_modulo(x, y, 1);
+    return bignorm(z);
 }
 
 static VALUE
 rb_big_remainder(x, y)
     VALUE x, y;
 {
-    return rb_big_modulo(x, y, 0);
+    VALUE z;
+
+    switch (TYPE(y)) {
+      case T_FIXNUM:
+	y = rb_int2big(FIX2LONG(y));
+	break;
+
+      case T_BIGNUM:
+	break;
+
+      default:
+	return rb_num_coerce_bin(x, y);
+    }
+    bigdivrem(x, y, 0, &z);
+
+    return bignorm(z);
 }
 
-static VALUE
+VALUE
 rb_big_divmod(x, y)
     VALUE x, y;
 {
@@ -967,19 +988,15 @@ rb_big_divmod(x, y)
 	y = rb_int2big(FIX2LONG(y));
 	break;
 
-      case T_FLOAT:
-	y = dbl2big(RFLOAT(y)->value);
-	break;
-
       case T_BIGNUM:
 	break;
 
       default:
 	return rb_num_coerce_bin(x, y);
     }
-    bigdivmod(x, y, &div, &mod, 1);
+    bigdivmod(x, y, &div, &mod);
 
-    return rb_assoc_new(div, mod);
+    return rb_assoc_new(bignorm(div), bignorm(mod));
 }
 
 VALUE
@@ -1332,7 +1349,7 @@ rb_big_rand(max, rand)
 	BDIGITS(v)[len] = ((USHORT)~0) * rand;
     }
 
-    return rb_big_mod((VALUE)v, max);
+    return rb_big_modulo((VALUE)v, max);
 }
 
 static VALUE
@@ -1363,8 +1380,9 @@ Init_Bignum()
     rb_define_method(rb_cBignum, "-", rb_big_minus, 1);
     rb_define_method(rb_cBignum, "*", rb_big_mul, 1);
     rb_define_method(rb_cBignum, "/", rb_big_div, 1);
-    rb_define_method(rb_cBignum, "%", rb_big_mod, 1);
+    rb_define_method(rb_cBignum, "%", rb_big_modulo, 1);
     rb_define_method(rb_cBignum, "divmod", rb_big_divmod, 1);
+    rb_define_method(rb_cBignum, "modulo", rb_big_modulo, 1);
     rb_define_method(rb_cBignum, "remainder", rb_big_remainder, 1);
     rb_define_method(rb_cBignum, "**", rb_big_pow, 1);
     rb_define_method(rb_cBignum, "&", rb_big_and, 1);
@@ -1380,7 +1398,6 @@ Init_Bignum()
     rb_define_method(rb_cBignum, "===", rb_big_eq, 1);
     rb_define_method(rb_cBignum, "eql?", rb_big_eq, 1);
     rb_define_method(rb_cBignum, "hash", rb_big_hash, 0);
-    rb_define_method(rb_cBignum, "to_i", rb_big_to_i, 0);
     rb_define_method(rb_cBignum, "to_f", rb_big_to_f, 0);
     rb_define_method(rb_cBignum, "abs", rb_big_abs, 0);
     rb_define_method(rb_cBignum, "size", rb_big_size, 0);
