@@ -3,7 +3,7 @@
   defines.h -
 
   $Author: matz $
-  $Date: 2000/07/31 09:20:55 $
+  $Date: 2000/07/27 09:49:14 $
   created at: Wed May 18 00:21:44 JST 1994
 
 ************************************************/
@@ -14,7 +14,7 @@
 
 /* define RUBY_USE_EUC/SJIS for default kanji-code */
 #ifndef DEFAULT_KCODE
-#if defined(MSDOS) || defined(__CYGWIN32__) || defined(__human68k__) || defined(__MACOS__) || defined(__EMX__) || defined(OS2) || defined(NT)
+#if defined(MSDOS) || defined(__CYGWIN__) || defined(__human68k__) || defined(__MACOS__) || defined(__EMX__) || defined(OS2) || defined(NT)
 #define DEFAULT_KCODE KCODE_SJIS
 #else
 #define DEFAULT_KCODE KCODE_EUC
@@ -61,16 +61,22 @@
 
 #if defined(MSDOS) || defined(NT) || defined(__human68k__) || defined(OS2)
 #define PATH_SEP ";"
+#elif defined(riscos)
+#define PATH_SEP ","
 #else
 #define PATH_SEP ":"
 #endif
 #define PATH_SEP_CHAR PATH_SEP[0]
 
-#if defined(__human68k__) || defined(__CYGWIN32__)
+#if defined(__human68k__)
 #undef HAVE_RANDOM
 #undef HAVE_SETITIMER
 #endif
 
+#if defined(DJGPP) || defined(__BOW__)
+#undef HAVE_SETITIMER
+#endif
+
 #ifndef RUBY_PLATFORM
 #define RUBY_PLATFORM "unknown-unknown"
 #endif
