@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   dir.c -
 
   $Author: matz $
-  $Date: 2000/07/31 09:20:55 $
+  $Date: 2000/08/07 05:01:46 $
   created at: Wed Jan  5 09:51:01 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 
@@ -48,9 +50,6 @@
 #endif
 
 #include <errno.h>
-#ifdef USE_CWGUSI
-# include <sys/errno.h>
-#endif
 
 #ifndef HAVE_STDLIB_H
 char *getenv();
@@ -233,14 +232,27 @@ free_dir(dir)
 static VALUE dir_close _((VALUE));
 
 static VALUE
-dir_s_open(dir_class, dirname)
-    VALUE dir_class, dirname;
+dir_s_new(argc, argv, klass)
+    int argc;
+    VALUE *argv;
+    VALUE klass;
+{
+    VALUE obj = Data_Wrap_Struct(klass, 0, free_dir, 0);
+
+    rb_obj_call_init(obj, argc, argv);
+
+    return obj;
+}
+
+static VALUE
+dir_initialize(dir, dirname)
+    VALUE dir, dirname;
 {
-    VALUE obj;
     DIR *dirp;
 
     Check_SafeStr(dirname);
-
+    if (DATA_PTR(dir)) closedir(DATA_PTR(dir));
+    DATA_PTR(dir) = NULL;
     dirp = opendir(RSTRING(dirname)->ptr);
     if (dirp == NULL) {
 	if (errno == EMFILE || errno == ENFILE) {
@@ -251,14 +263,24 @@ dir_s_open(dir_class, dirname)
 	    rb_sys_fail(RSTRING(dirname)->ptr);
 	}
     }
+    DATA_PTR(dir) = dirp;
 
-    obj = Data_Wrap_Struct(dir_class, 0, free_dir, dirp);
+    return dir;
+}
 
-    if (rb_iterator_p()) {
-	return rb_ensure(rb_yield, obj, dir_close, obj);
+static VALUE
+dir_s_open(klass, dirname)
+    VALUE klass, dirname;
+{
+    VALUE dir = Data_Wrap_Struct(klass, 0, free_dir, 0);
+
+    dir_initialize(dir, dirname);
+    if (rb_block_given_p()) {
+	rb_ensure(rb_yield, dir, dir_close, dir);
+	return Qnil;
     }
 
-    return obj;
+    return dir;
 }
 
 static void
@@ -299,12 +321,10 @@ dir_each(dir)
 {
     DIR *dirp;
     struct dirent *dp;
-    VALUE file;
 
     GetDIR(dir, dirp);
     for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
-	file = rb_tainted_str_new(dp->d_name, NAMLEN(dp));
-	rb_yield(file);
+	rb_yield(rb_tainted_str_new(dp->d_name, NAMLEN(dp)));
 	if (DATA_PTR(dir) == NULL) dir_closed();
     }
     return dir;
@@ -413,7 +433,7 @@ static VALUE
 dir_s_chroot(dir, path)
     VALUE dir, path;
 {
-#if !defined(DJGPP) && !defined(NT) && !defined(__human68k__) && !defined(USE_CWGUSI) && !defined(__BEOS__) && !defined(__EMX__)
+#if defined(HAVE_CHROOT) && !defined(__CHECKER__)
     rb_secure(2);
     Check_SafeStr(path);
 
@@ -436,7 +456,6 @@ dir_s_mkdir(argc, argv, obj)
     VALUE path, vmode;
     int mode;
 
-    rb_secure(2);
     if (rb_scan_args(argc, argv, "11", &path, &vmode) == 2) {
 	mode = NUM2INT(vmode);
     }
@@ -445,7 +464,8 @@ dir_s_mkdir(argc, argv, obj)
     }
 
     Check_SafeStr(path);
-#if !defined(NT) && !defined(USE_CWGUSI)
+    rb_secure(2);
+#if !defined(NT)
     if (mkdir(RSTRING(path)->ptr, mode) == -1)
 	rb_sys_fail(RSTRING(path)->ptr);
 #else
@@ -460,12 +480,12 @@ static VALUE
 dir_s_rmdir(obj, dir)
     VALUE obj, dir;
 {
-    rb_secure(2);
     Check_SafeStr(dir);
+    rb_secure(2);
     if (rmdir(RSTRING(dir)->ptr) < 0)
 	rb_sys_fail(RSTRING(dir)->ptr);
 
-    return Qtrue;
+    return INT2FIX(0);
 }
 
 /* Return nonzero if S has any special globbing chars in it.  */
@@ -537,8 +557,8 @@ extract_elem(path)
 #   define S_ISDIR(m) ((m & S_IFMT) == S_IFDIR)
 #endif
 
-static void
-glob(path, func, arg)
+void
+rb_glob(path, func, arg)
     char *path;
     void (*func)();
     VALUE arg;
@@ -558,9 +578,10 @@ glob(path, func, arg)
 	if (*p == '/') p++;
 	m = strchr(p, '/');
 	if (has_magic(p, m)) {
-	    char *dir, *base, *magic;
+	    char *dir, *base, *magic, *buf;
 	    DIR *dirp;
 	    struct dirent *dp;
+	    int recursive = 0;
 
 	    struct d_link {
 		char *path;
@@ -571,25 +592,41 @@ glob(path, func, arg)
 	    if (path == p) dir = ".";
 	    else dir = base;
 
+	    magic = extract_elem(p);
+	    if (m && strcmp(magic, "**") == 0) {
+		recursive = 1;
+		buf = ALLOC_N(char, strlen(base)+strlen(m)+3);
+		sprintf(buf, "%s%s%s", base, (*base)?"":".", m);
+		rb_glob(buf, func, arg);
+		free(buf);
+	    }
 	    dirp = opendir(dir);
 	    if (dirp == NULL) {
 		free(base);
 		break;
 	    }
-	    magic = extract_elem(p);
 #define BASE (*base && !(*base == '/' && !base[1]))
 
 	    for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
+		if (recursive) {
+		    if (strcmp(".", dp->d_name) == 0 || strcmp("..", dp->d_name) == 0)
+			continue;
+		    buf = ALLOC_N(char, strlen(base)+NAMLEN(dp)+strlen(m)+6);
+		    sprintf(buf, "%s%s%s/**%s", base, (BASE)?"/":"", dp->d_name, m);
+		    rb_glob(buf, func, arg);
+		    free(buf);
+		    continue;
+		}
 		if (fnmatch(magic, dp->d_name, FNM_PERIOD|FNM_PATHNAME) == 0) {
-		    char *fix = ALLOC_N(char, strlen(base)+NAMLEN(dp)+2);
-		    sprintf(fix, "%s%s%s", base, (BASE)?"/":"", dp->d_name);
+		    buf = ALLOC_N(char, strlen(base)+NAMLEN(dp)+2);
+		    sprintf(buf, "%s%s%s", base, (BASE)?"/":"", dp->d_name);
 		    if (!m) {
-			(*func)(fix, arg);
-			free(fix);
+			(*func)(buf, arg);
+			free(buf);
 			continue;
 		    }
 		    tmp = ALLOC(struct d_link);
-		    tmp->path = fix;
+		    tmp->path = buf;
 		    tmp->next = link;
 		    link = tmp;
 		}
@@ -605,7 +642,7 @@ glob(path, func, arg)
 		    char *t = ALLOC_N(char, len+mlen+1);
 
 		    sprintf(t, "%s%s", link->path, m);
-		    glob(t, func, arg);
+		    rb_glob(t, func, arg);
 		    free(t);
 		}
 		tmp = link;
@@ -631,7 +668,7 @@ push_globs(ary, s)
     VALUE ary;
     char *s;
 {
-    glob(s, push_pattern, ary);
+    rb_glob(s, push_pattern, ary);
 }
 
 static void
@@ -727,6 +764,14 @@ dir_s_glob(dir, str)
     }
     if (buf != buffer)
 	free(buf);
+    if (rb_block_given_p()) {
+	long len = RARRAY(ary)->len;
+	VALUE *ptr = RARRAY(ary)->ptr;
+
+	while (len--) {
+	    rb_yield(*ptr++);
+	}
+    }
     return ary;
 }
 
@@ -737,7 +782,8 @@ dir_foreach(io, dirname)
     VALUE dir;
 
     dir = rb_funcall(rb_cDir, rb_intern("open"), 1, dirname);
-    return rb_ensure(dir_each, dir, dir_close, dir);
+    rb_ensure(dir_each, dir, dir_close, dir);
+    return Qnil;
 }
 
 static VALUE
@@ -757,16 +803,19 @@ Init_Dir()
 
     rb_include_module(rb_cDir, rb_mEnumerable);
 
-    rb_define_singleton_method(rb_cDir, "new", dir_s_open, 1);
+    rb_define_singleton_method(rb_cDir, "new", dir_s_new, -1);
     rb_define_singleton_method(rb_cDir, "open", dir_s_open, 1);
     rb_define_singleton_method(rb_cDir, "foreach", dir_foreach, 1);
     rb_define_singleton_method(rb_cDir, "entries", dir_entries, 1);
 
+    rb_define_method(rb_cDir,"initialize", dir_initialize, 1);
     rb_define_method(rb_cDir,"read", dir_read, 0);
     rb_define_method(rb_cDir,"each", dir_each, 0);
     rb_define_method(rb_cDir,"rewind", dir_rewind, 0);
     rb_define_method(rb_cDir,"tell", dir_tell, 0);
     rb_define_method(rb_cDir,"seek", dir_seek, 1);
+    rb_define_method(rb_cDir,"pos", dir_tell, 0);
+    rb_define_method(rb_cDir,"pos=", dir_seek, 1);
     rb_define_method(rb_cDir,"close", dir_close, 0);
 
     rb_define_singleton_method(rb_cDir,"chdir", dir_s_chdir, -1);
