@@ -1,6 +1,6 @@
 #
 #		tk.rb - Tk interface modue using tcltklib
-#			$Date: 2000/06/13 09:57:39 $
+#			$Date: 2000/08/02 04:54:20 $
 #			by Yukihiro Matsumoto <matz@netlab.co.jp>
 
 # use Shigehiro's tcltklib
@@ -219,6 +219,17 @@ module TkComm
   end
   private :_get_eval_string
 
+  def ruby2tcl(v)
+    if v.kind_of?(Hash)
+      v = hash_kv(v)
+      v.flatten!
+      v.collect{|e|ruby2tcl(e)}
+    else
+      _get_eval_string(v)
+    end
+  end
+  private :ruby2tcl
+
   Tk_IDs = [0, 0]		# [0]-cmdid, [1]-winid
   def _curr_cmd_id
     id = format("c%.4d", Tk_IDs[0])
@@ -300,7 +311,7 @@ module TkComm
 
   def install_bind(cmd, args=nil)
     if args
-      id = install_cmd(proc{|arg|
+      id = install_cmd(proc{|*arg|
 	TkUtil.eval_cmd cmd, *arg
       })
       id + " " + args
@@ -485,7 +496,7 @@ module TkCore
   end
 
   def rb_appsend(interp, async, *args)
-    args = args.filter{|c| _get_eval_string(c).gsub(/[][$"]/, '\\\\\&')}
+    args = args.collect!{|c| _get_eval_string(c).gsub(/[][$"]/, '\\\\\&')}
     args.push(').to_s"')
     appsend(interp, async, 'ruby "(', *args)
   end
@@ -500,7 +511,7 @@ module TkCore
   end
 
   def rb_appsend_displayof(interp, win, async, *args)
-    args = args.filter{|c| _get_eval_string(c).gsub(/[][$"]/, '\\\\\&')}
+    args = args.collect!{|c| _get_eval_string(c).gsub(/[][$"]/, '\\\\\&')}
     args.push(').to_s"')
     appsend_displayof(interp, win, async, 'ruby "(', *args)
   end
@@ -527,21 +538,21 @@ module TkCore
     tk_call 'tk_messageBox', *hash_kv(keys)
   end
 
-  def getOpenFile(keys)
+  def getOpenFile(keys = nil)
     tk_call 'tk_getOpenFile', *hash_kv(keys)
   end
 
-  def getSaveFile(keys)
+  def getSaveFile(keys = nil)
     tk_call 'tk_getSaveFile', *hash_kv(keys)
   end
 
-  def chooseColor(keys)
+  def chooseColor(keys = nil)
     tk_call 'tk_chooseColor', *hash_kv(keys)
   end
 
   def tk_call(*args)
     print args.join(" "), "\n" if $DEBUG
-    args.filter{|x|_get_eval_string(x)}
+    args.collect! {|x|ruby2tcl(x)}
     args.compact!
     args.flatten!
     begin
@@ -773,6 +784,96 @@ module Tk
   end
 end
 
+###########################################
+#  convert kanji string to/from utf-8
+###########################################
+if /^8\.[1-9]/ =~ Tk::TCL_VERSION && !Tk::JAPANIZED_TK
+  class TclTkIp
+    # from tkencoding.rb by ttate@jaist.ac.jp
+    alias __eval _eval
+    alias __invoke _invoke
+    private :__eval
+    private :__invoke
+    
+    attr_accessor :encoding
+    
+    def _eval(cmd)
+      if @encoding
+	_fromUTF8(__eval(_toUTF8(cmd, @encoding)), @encoding)
+      else
+	__eval(cmd)
+      end
+    end
+    
+    def _invoke(*cmds)
+      if @encoding
+	cmds = cmds.collect{|cmd| _toUTF8(cmd, @encoding)}
+	_fromUTF8(__invoke(*cmds), @encoding)
+      else
+	__invoke(*cmds)
+	end
+    end
+  end
+
+  module Tk
+    def encoding=(name)
+      INTERP.encoding = name
+    end
+
+    def encoding
+      INTERP.encoding
+    end
+
+    def encoding_names
+      tk_split_simplelist(tk_call('encoding', 'names'))
+    end
+
+    def encoding_system
+      tk_call('encoding', 'system')
+    end
+
+    def encoding_system=(enc)
+      tk_call('encoding', 'system', enc)
+    end
+  end
+
+  # estimate encoding
+  case $KCODE
+  when /^e/i  # EUC
+    Tk.encoding = 'euc-jp'
+  when /^s/i  # SJIS
+    Tk.encoding = 'shiftjis'
+  when /^u/i  # UTF8
+    Tk.encoding = 'utf-8'
+  else        # NONE
+    begin
+      Tk.encoding = Tk.encoding_system
+    rescue StandardError, NameError
+      Tk.encoding = 'utf-8'
+    end
+  end
+
+else
+  # dummy methods
+  module Tk
+    def encoding=(name)
+      nil
+    end
+    def encoding
+      nil
+    end
+    def encoding_names
+      nil
+    end
+    def encoding_system
+      nil
+    end
+    def encoding_system=(enc)
+      nil
+    end
+  end
+end
+
 module TkBindCore
   def bind(context, cmd=Proc.new, args=nil)
     Tk.bind(to_eval, context, cmd, args)
