@@ -1,8 +1,8 @@
 #
 #		tkcanvas.rb - Tk canvas classes
-#			$Date: 2000/06/23 08:38:45 $
+#			$Date: 2000/06/12 07:48:26 $
 #			by Yukihiro Matsumoto <matz@caelum.co.jp>
-#			$Date: 2000/06/23 08:38:45 $
+#			$Date: 2000/06/12 07:48:26 $
 #			by Hidetoshi Nagai <nagai@ai.kyutech.ac.jp>
 
 require "tk"
@@ -216,7 +216,7 @@ class TkCanvas<TkWindow
   end
 
   def find(mode, *args)
-    list(tk_send 'find', mode, *args).filter{|id| 
+    list(tk_send 'find', mode, *args).collect!{|id| 
       TkcItem.id2obj(self, id)
     }
   end
@@ -789,7 +789,7 @@ class TkImage<TkObject
   end
 
   def TkImage.names
-    Tk.tk_call('image', 'names').split.filter{|id|
+    Tk.tk_call('image', 'names').split.collect!{|id|
       (Tk_IMGTBL[id])? Tk_IMGTBL[id] : id
     }
   end
