@@ -1,6 +1,6 @@
 #
 #		tkentry.rb - Tk entry classes
-#			$Date: 2000/02/01 03:11:28 $
+#			$Date: 2000/06/12 07:48:28 $
 #			by Yukihiro Matsumoto <matz@caelum.co.jp>
 
 require 'tk.rb'
@@ -59,6 +59,14 @@ class TkEntry<TkLabel
     tk_send 'selection', 'to', index
   end
 
+  def validate
+    if tk_send('validate') == '0'
+      false
+    else 
+      true
+    end
+  end
+
   def value
     tk_send 'get'
   end
