@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   file.c -
 
   $Author: matz $
-  $Date: 2000/07/24 07:16:04 $
+  $Date: 2000/09/18 08:47:06 $
   created at: Mon Nov 15 12:24:34 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #ifdef NT
 #include "missing/file.h"
@@ -17,11 +19,18 @@
 #include "ruby.h"
 #include "rubyio.h"
 #include "rubysig.h"
+#include "dln.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_SYS_FILE_H
+# include <sys/file.h>
+#else
+int flock _((int, int));
+#endif
+
 #ifdef HAVE_SYS_PARAM_H
 # include <sys/param.h>
 #else
@@ -57,23 +66,13 @@ char *strrchr _((const char*,const char));
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifdef USE_CWGUSI
-#include "macruby_missing.h"
-extern int fileno(FILE *stream);
-extern int utimes();
-#endif
-
-#ifndef strdup
-char *strdup();
-#endif
-
 #ifdef __EMX__
 #define lstat stat
 #endif
  
 VALUE rb_cFile;
 VALUE rb_mFileTest;
-static VALUE sStat;
+static VALUE rb_cStat;
 
 static int
 apply2files(func, vargs, arg)
@@ -105,15 +104,11 @@ rb_file_path(obj)
     OpenFile *fptr;
 
     GetOpenFile(obj, fptr);
-    if (fptr->path == NULL) return Qnil;
+    if (!fptr->path) return Qnil;
     return rb_str_new2(fptr->path);
 }
 
-#ifndef NT
-# ifndef USE_CWGUSI
-#  include <sys/file.h>
-# endif
-#else
+#ifdef NT
 #include "missing/file.h"
 #endif
 
@@ -121,33 +116,186 @@ static VALUE
 stat_new(st)
     struct stat *st;
 {
+    struct stat *nst;
     if (!st) rb_bug("stat_new() called with bad value");
-    return rb_struct_new(sStat,
-			 INT2FIX((int)st->st_dev),
-			 INT2FIX((int)st->st_ino),
-			 INT2FIX((int)st->st_mode),
-			 INT2FIX((int)st->st_nlink),
-			 INT2FIX((int)st->st_uid),
-			 INT2FIX((int)st->st_gid),
+
+    nst = ALLOC(struct stat);
+    *nst = *st;
+    return Data_Wrap_Struct(rb_cStat, NULL, free, nst);
+}
+
+static struct stat*
+get_stat(self)
+    VALUE self;
+{
+    struct stat* st;
+    Data_Get_Struct(self, struct stat, st);
+    if (!st) rb_bug("collapsed File::Stat");
+    return st;
+}
+
+static VALUE
+rb_stat_cmp(self, other)
+    VALUE self, other;
+{
+    time_t t1 = get_stat(self)->st_mtime;
+    time_t t2 = get_stat(other)->st_mtime;
+    if (t1 == t2)
+	return INT2FIX(0);
+    else if (t1 < t2)
+	return INT2FIX(-1);
+    else
+	return INT2FIX(1);
+}
+
+static VALUE
+rb_stat_dev(self)
+    VALUE self;
+{
+    return INT2FIX((int)get_stat(self)->st_dev);
+}
+
+static VALUE
+rb_stat_ino(self)
+    VALUE self;
+{
+    return INT2FIX((int)get_stat(self)->st_ino);
+}
+
+static VALUE
+rb_stat_mode(self)
+    VALUE self;
+{
+    return INT2FIX((int)get_stat(self)->st_mode);
+}
+
+static VALUE
+rb_stat_nlink(self)
+    VALUE self;
+{
+    return INT2FIX((int)get_stat(self)->st_nlink);
+}
+
+static VALUE
+rb_stat_uid(self)
+    VALUE self;
+{
+    return INT2FIX((int)get_stat(self)->st_uid);
+}
+
+static VALUE
+rb_stat_gid(self)
+    VALUE self;
+{
+    return INT2FIX((int)get_stat(self)->st_gid);
+}
+
+static VALUE
+rb_stat_rdev(self)
+    VALUE self;
+{
 #ifdef HAVE_ST_RDEV
-			 INT2FIX((int)st->st_rdev),
+    return INT2FIX((int)get_stat(self)->st_rdev);
 #else
-			 INT2FIX(0),
+    return INT2FIX(0);
 #endif
-			 INT2FIX((int)st->st_size),
+}
+
+static VALUE
+rb_stat_size(self)
+    VALUE self;
+{
+    return INT2FIX((int)get_stat(self)->st_size);
+}
+
+static VALUE
+rb_stat_blksize(self)
+    VALUE self;
+{
 #ifdef HAVE_ST_BLKSIZE
-			 INT2FIX((int)st->st_blksize),
+    return INT2FIX((int)get_stat(self)->st_blksize);
 #else
-			 INT2FIX(0),
+    return INT2FIX(0);
 #endif
+}
+
+static VALUE
+rb_stat_blocks(self)
+    VALUE self;
+{
 #ifdef HAVE_ST_BLOCKS
-			 INT2FIX((int)st->st_blocks),
+    return INT2FIX((int)get_stat(self)->st_blocks);
 #else
-			 INT2FIX(0),
+    return INT2FIX(0);
 #endif
-			 rb_time_new(st->st_atime, 0),
-			 rb_time_new(st->st_mtime, 0),
-			 rb_time_new(st->st_ctime, 0));
+}
+
+static VALUE
+rb_stat_atime(self)
+    VALUE self;
+{
+    return rb_time_new(get_stat(self)->st_atime, 0);
+}
+
+static VALUE
+rb_stat_mtime(self)
+    VALUE self;
+{
+    return rb_time_new(get_stat(self)->st_mtime, 0);
+}
+
+static VALUE
+rb_stat_ctime(self)
+    VALUE self;
+{
+    return rb_time_new(get_stat(self)->st_ctime, 0);
+}
+
+static VALUE
+rb_stat_inspect(self)
+    VALUE self;
+{
+    VALUE str;
+    int i;
+    struct {
+        char *name;
+        VALUE (*func)();
+    } member[] = {
+        {"dev",     rb_stat_dev},
+        {"ino",     rb_stat_ino},
+        {"mode",    rb_stat_mode},
+        {"nlink",   rb_stat_nlink},
+        {"uid",     rb_stat_uid},
+        {"gid",     rb_stat_gid},
+        {"rdev",    rb_stat_rdev},
+        {"size",    rb_stat_size},
+        {"blksize", rb_stat_blksize},
+        {"blocks",  rb_stat_blocks},
+        {"atime",   rb_stat_atime},
+        {"mtime",   rb_stat_mtime},
+        {"ctime",   rb_stat_ctime},
+    };
+
+    str = rb_str_new2("#<");
+    rb_str_cat2(str, rb_class2name(CLASS_OF(self)));
+    rb_str_cat2(str, " ");
+
+    for (i = 0; i < sizeof(member)/sizeof(member[0]); i++) {
+	VALUE str2;
+	char *p;
+
+	if (i > 0) {
+	    rb_str_cat2(str, ", ");
+	}
+	rb_str_cat2(str, member[i].name);
+	rb_str_cat2(str, "=");
+	str2 = rb_inspect((*member[i].func)(self));
+	rb_str_append(str, str2);
+    }
+    rb_str_cat2(str, ">");
+    OBJ_INFECT(str, self);
+
+    return str;
 }
 
 static int
@@ -158,7 +306,7 @@ rb_stat(file, st)
     if (TYPE(file) == T_FILE) {
 	OpenFile *fptr;
 
-	rb_secure(4);
+	rb_secure(2);
 	GetOpenFile(file, fptr);
 	return fstat(fileno(fptr->f), st);
     }
@@ -222,8 +370,9 @@ rb_file_lstat(obj)
     OpenFile *fptr;
     struct stat st;
 
-    rb_secure(4);
+    rb_secure(2);
     GetOpenFile(obj, fptr);
+    if (!fptr->path) return Qnil;
     if (lstat(fptr->path, &st) == -1) {
 	rb_sys_fail(fptr->path);
     }
@@ -238,7 +387,7 @@ static int
 group_member(gid)
     GETGROUPS_T gid;
 {
-#if !defined(NT) && !defined(USE_CWGUSI)
+#if !defined(NT)
     if (getgid() ==  gid || getegid() == gid)
 	return Qtrue;
 
@@ -269,7 +418,7 @@ eaccess(path, mode)
      const char *path;
      int mode;
 {
-#ifndef NT
+#ifdef S_IXGRP
   struct stat st;
   static int euid = -1;
 
@@ -360,8 +509,8 @@ test_l(obj, fname)
     Check_SafeStr(fname);
     if (lstat(RSTRING(fname)->ptr, &st) < 0) return Qfalse;
     if (S_ISLNK(st.st_mode)) return Qtrue;
-
 #endif
+
     return Qfalse;
 }
 
@@ -597,7 +746,7 @@ static VALUE
 test_sgid(obj, fname)
     VALUE obj, fname;
 {
-#ifndef NT
+#ifdef S_ISGID
     Check_SafeStr(fname);
     return check3rdbyte(RSTRING(fname)->ptr, S_ISGID);
 #else
@@ -627,12 +776,51 @@ rb_file_s_size(obj, fname)
     return rb_int2inum(st.st_size);
 }
 
+static VALUE
+rb_file_ftype(st)
+    struct stat *st;
+{
+    char *t;
+
+    if (S_ISREG(st->st_mode)) {
+	t = "file";
+    } else if (S_ISDIR(st->st_mode)) {
+	t = "directory";
+    } else if (S_ISCHR(st->st_mode)) {
+	t = "characterSpecial";
+    }
+#ifdef S_ISBLK
+    else if (S_ISBLK(st->st_mode)) {
+	t = "blockSpecial";
+    }
+#endif
+#ifdef S_ISFIFO
+    else if (S_ISFIFO(st->st_mode)) {
+	t = "fifo";
+    }
+#endif
+#ifdef S_ISLNK
+    else if (S_ISLNK(st->st_mode)) {
+	t = "link";
+    }
+#endif
+#ifdef S_ISSOCK
+    else if (S_ISSOCK(st->st_mode)) {
+	t = "socket";
+    }
+#endif
+    else {
+	t = "unknown";
+    }
+
+    return rb_str_new2(t);
+}
+
 static VALUE
 rb_file_s_ftype(obj, fname)
     VALUE obj, fname;
 {
     struct stat st;
-    char *t;
 
 #if defined(MSDOS) || defined(NT)
     if (rb_stat(fname, &st) < 0)
@@ -644,38 +832,7 @@ rb_file_s_ftype(obj, fname)
     }
 #endif
 
-    if (S_ISREG(st.st_mode)) {
-	t = "file";
-    } else if (S_ISDIR(st.st_mode)) {
-	t = "directory";
-    } else if (S_ISCHR(st.st_mode)) {
-	t = "characterSpecial";
-    }
-#ifdef S_ISBLK
-    else if (S_ISBLK(st.st_mode)) {
-	t = "blockSpecial";
-    }
-#endif
-#ifdef S_ISFIFO
-    else if (S_ISFIFO(st.st_mode)) {
-	t = "fifo";
-    }
-#endif
-#ifdef S_ISLNK
-    else if (S_ISLNK(st.st_mode)) {
-	t = "link";
-    }
-#endif
-#ifdef S_ISSOCK
-    else if (S_ISSOCK(st.st_mode)) {
-	t = "socket";
-    }
-#endif
-    else {
-	t = "unknown";
-    }
-
-    return rb_str_new2(t);
+    return rb_file_ftype(&st);
 }
 
 static VALUE
@@ -771,6 +928,7 @@ rb_file_s_chmod(argc, argv)
     VALUE rest;
     int mode, n;
 
+    rb_secure(2);
     rb_scan_args(argc, argv, "1*", &vmode, &rest);
     mode = NUM2INT(vmode);
 
@@ -785,11 +943,12 @@ rb_file_chmod(obj, vmode)
     OpenFile *fptr;
     int mode;
 
-    rb_secure(4);
+    rb_secure(2);
     mode = NUM2INT(vmode);
 
     GetOpenFile(obj, fptr);
-#if defined(DJGPP) || defined(NT) || defined(USE_CWGUSI) || defined(__BEOS__) || defined(__EMX__)
+#if defined(DJGPP) || defined(NT) || defined(__BEOS__) || defined(__EMX__)
+    if (!fptr->path) return Qnil;
     if (chmod(fptr->path, mode) == -1)
 	rb_sys_fail(fptr->path);
 #else
@@ -822,6 +981,7 @@ rb_file_s_chown(argc, argv)
     struct chown_args arg;
     int n;
 
+    rb_secure(2);
     rb_scan_args(argc, argv, "2*", &o, &g, &rest);
     if (NIL_P(o)) {
 	arg.owner = -1;
@@ -846,9 +1006,10 @@ rb_file_chown(obj, owner, group)
 {
     OpenFile *fptr;
 
-    rb_secure(4);
+    rb_secure(2);
     GetOpenFile(obj, fptr);
-#if defined(DJGPP) || defined(__CYGWIN32__) || defined(NT) || defined(USE_CWGUSI) || defined(__EMX__)
+#if defined(DJGPP) || defined(__CYGWIN32__) || defined(NT) || defined(__EMX__)
+    if (!fptr->path) return Qnil;
     if (chown(fptr->path, NUM2INT(owner), NUM2INT(group)) == -1)
 	rb_sys_fail(fptr->path);
 #else
@@ -861,7 +1022,7 @@ rb_file_chown(obj, owner, group)
 
 struct timeval rb_time_timeval();
 
-#ifdef HAVE_UTIMES
+#if defined(HAVE_UTIMES) && !defined(__CHECKER__)
 
 static void
 utime_internal(path, tvp)
@@ -899,7 +1060,7 @@ rb_file_s_utime(argc, argv)
 #   else
 #  include <sys/utime.h>
 #   endif
-#   if defined(_MSC_VER)
+#   if defined(_MSC_VER) || defined __MINGW32__
 #  define utimbuf _utimbuf
 #   endif
 # else
@@ -946,23 +1107,19 @@ static VALUE
 rb_file_s_link(obj, from, to)
     VALUE obj, from, to;
 {
-#if defined(USE_CWGUSI)
-        rb_notimplement();
-#else
     Check_SafeStr(from);
     Check_SafeStr(to);
 
     if (link(RSTRING(from)->ptr, RSTRING(to)->ptr) < 0)
 	rb_sys_fail(RSTRING(from)->ptr);
     return INT2FIX(0);
-#endif /* USE_CWGUSI */
 }
 
 static VALUE
 rb_file_s_symlink(obj, from, to)
     VALUE obj, from, to;
 {
-#if !defined(MSDOS) && !defined(NT) && !defined(__EMX__)
+#if !defined(MSDOS) && !defined(NT) && !defined(__EMX__) && !defined(riscos)
     Check_SafeStr(from);
     Check_SafeStr(to);
 
@@ -979,7 +1136,7 @@ static VALUE
 rb_file_s_readlink(obj, path)
     VALUE obj, path;
 {
-#if !defined(MSDOS) && !defined(NT) && !defined(__EMX__)
+#if !defined(MSDOS) && !defined(NT) && !defined(__EMX__) && !defined(riscos)
     char buf[MAXPATHLEN];
     int cc;
 
@@ -1031,12 +1188,9 @@ rb_file_s_umask(argc, argv)
     int argc;
     VALUE *argv;
 {
-#ifdef USE_CWGUSI
-    rb_notimplement();
-#else
     int omask = 0;
 
-    rb_secure(4);
+    rb_secure(2);
     if (argc == 0) {
 	omask = umask(0);
 	umask(omask);
@@ -1048,7 +1202,6 @@ rb_file_s_umask(argc, argv)
 	rb_raise(rb_eArgError, "wrong # of argument");
     }
     return INT2FIX(omask);
-#endif /* USE_CWGUSI */
 }
 
 #if defined DOSISH
@@ -1065,14 +1218,14 @@ rb_file_s_expand_path(argc, argv)
     VALUE fname, dname;
     char *s, *p;
     char buf[MAXPATHLEN+2];
-    int tainted = 0;
+    int tainted;
 
     rb_scan_args(argc, argv, "11", &fname, &dname);
 
+    tainted = OBJ_TAINTED(fname);
     s = STR2CSTR(fname);
     p = buf;
     if (s[0] == '~') {
-	tainted = 1;
 	if (isdirsep(s[1]) || s[1] == '\0') {
 	    char *dir = getenv("HOME");
 
@@ -1082,6 +1235,7 @@ rb_file_s_expand_path(argc, argv)
 	    strcpy(buf, dir);
 	    p = &buf[strlen(buf)];
 	    s++;
+	    tainted = 1;
 	}
 	else {
 #ifdef HAVE_PWD_H
@@ -1223,19 +1377,20 @@ rb_file_s_basename(argc, argv)
     name = STR2CSTR(fname);
     p = strrchr(name, '/');
     if (!p) {
-	if (!NIL_P(fext)) {
-	    f = rmext(name, ext);
-	    if (f) return rb_str_new(name, f);
+	if (NIL_P(fext) || !(f = rmext(name, ext)))
+	    return fname;
+	basename = rb_str_new(name, f);
+    }
+    else {
+	p++;			/* skip last `/' */
+	if (NIL_P(fext) || !(f = rmext(p, ext))) {
+	    basename = rb_str_new2(p);
+	}
+	else {
+	    basename = rb_str_new(p, f);
 	}
-	return fname;
     }
-    p++;			/* skip last `/' */
-    if (!NIL_P(fext)) {
-	f = rmext(p, ext);
-	if (f) return rb_str_new(p, f);
-    }
-    basename = rb_str_new2(p);
-    if (OBJ_TAINTED(fname)) OBJ_TAINT(basename);
+    OBJ_INFECT(basename, fname);
     return basename;
 }
 
@@ -1278,6 +1433,7 @@ static VALUE
 rb_file_s_truncate(obj, path, len)
     VALUE obj, path, len;
 {
+    rb_secure(2);
     Check_SafeStr(path);
 
 #ifdef HAVE_TRUNCATE
@@ -1316,7 +1472,7 @@ rb_file_truncate(obj, len)
 {
     OpenFile *fptr;
 
-    rb_secure(4);
+    rb_secure(2);
     GetOpenFile(obj, fptr);
     if (!(fptr->mode & FMODE_WRITABLE)) {
 	rb_raise(rb_eIOError, "not opened for writing");
@@ -1348,7 +1504,7 @@ rb_file_truncate(obj, len)
 #  define LOCK_UN 8
 # endif
 
-#if defined(EWOULDBLOCK)
+#if defined(EWOULDBLOCK) && 0
 static int
 rb_thread_flock(fd, op, fptr)
     int fd, op;
@@ -1360,9 +1516,9 @@ rb_thread_flock(fd, op, fptr)
     op |= LOCK_NB;
     while (flock(fd, op) < 0) {
 	switch (errno) {
-	  case EINTR:		/* can be happen? */
+	  case EINTR:			/* can be happen? */
 	  case EWOULDBLOCK:
-	    rb_thread_schedule();	/* busy wait */
+	    rb_thread_polling();	/* busy wait */
 	    rb_io_check_closed(fptr);
 	    break;
 	  default:
@@ -1379,14 +1535,15 @@ rb_file_flock(obj, operation)
     VALUE obj;
     VALUE operation;
 {
-#ifdef USE_CWGUSI
-    rb_notimplement();
-#else
+#ifndef __CHECKER__
     OpenFile *fptr;
 
-    rb_secure(4);
+    rb_secure(2);
     GetOpenFile(obj, fptr);
 
+    if (fptr->mode & FMODE_WRITABLE) {
+	fflush(GetWriteFile(fptr));
+    }
     if (flock(fileno(fptr->f), NUM2INT(operation)) < 0) {
 #ifdef EWOULDBLOCK
 	if (errno == EWOULDBLOCK) {
@@ -1395,8 +1552,8 @@ rb_file_flock(obj, operation)
 #endif
 	rb_sys_fail(fptr->path);
     }
+#endif
     return INT2FIX(0);
-#endif /* USE_CWGUSI */
 }
 #undef flock
 
@@ -1433,6 +1590,11 @@ rb_f_test(argc, argv)
     int cmd;
 
     if (argc == 0) rb_raise(rb_eArgError, "wrong # of arguments");
+#if 0 /* 1.7 behavior? */
+    if (argc == 1) {
+	return RTEST(argv[0]) ? Qtrue : Qfalse;
+    }
+#endif
     cmd = NUM2CHR(argv[0]);
     if (cmd == 0) return Qfalse;
     if (strchr("bcdefgGkloOprRsSuwWxXz", cmd)) {
@@ -1556,6 +1718,275 @@ rb_f_test(argc, argv)
     return Qnil;		/* not reached */
 }
 
+static VALUE
+rb_stat_ftype(obj)
+    VALUE obj;
+{
+    return rb_file_ftype(get_stat(obj));
+}
+
+static VALUE
+rb_stat_d(obj)
+    VALUE obj;
+{
+    if (S_ISDIR(get_stat(obj)->st_mode)) return Qtrue;
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_p(obj)
+    VALUE obj;
+{
+#ifdef S_IFIFO
+    if (S_ISFIFO(get_stat(obj)->st_mode)) return Qtrue;
+
+#endif
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_l(obj)
+    VALUE obj;
+{
+#ifdef S_ISLNK
+    if (S_ISLNK(get_stat(obj)->st_mode)) return Qtrue;
+#endif
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_S(obj)
+    VALUE obj;
+{
+#ifdef S_ISSOCK
+    if (S_ISSOCK(get_stat(obj)->st_mode)) return Qtrue;
+
+#endif
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_b(obj)
+    VALUE obj;
+{
+#ifdef S_ISBLK
+    if (S_ISBLK(get_stat(obj)->st_mode)) return Qtrue;
+
+#endif
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_c(obj)
+    VALUE obj;
+{
+    if (S_ISCHR(get_stat(obj)->st_mode)) return Qtrue;
+
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_owned(obj)
+    VALUE obj;
+{
+    if (get_stat(obj)->st_uid == geteuid()) return Qtrue;
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_rowned(obj)
+    VALUE obj;
+{
+    if (get_stat(obj)->st_uid == getuid()) return Qtrue;
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_grpowned(obj)
+    VALUE obj;
+{
+#ifndef NT
+    if (get_stat(obj)->st_gid == getegid()) return Qtrue;
+#endif
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_r(obj)
+    VALUE obj;
+{
+    struct stat *st = get_stat(obj);
+
+#ifdef S_IRUSR
+    if (rb_stat_owned(obj))
+	return st->st_mode & S_IRUSR ? Qtrue : Qfalse;
+#endif
+#ifdef S_IRGRP
+    if (rb_stat_grpowned(obj))
+	return st->st_mode & S_IRGRP ? Qtrue : Qfalse;
+#endif
+#ifdef S_IROTH
+    if (!(st->st_mode & S_IROTH)) return Qfalse;
+#endif
+    return Qtrue;
+}
+
+static VALUE
+rb_stat_R(obj)
+    VALUE obj;
+{
+    struct stat *st = get_stat(obj);
+
+#ifdef S_IRUSR
+    if (rb_stat_rowned(obj))
+	return st->st_mode & S_IRUSR ? Qtrue : Qfalse;
+#endif
+#ifdef S_IRGRP
+    if (group_member(get_stat(obj)->st_gid))
+	return st->st_mode & S_IRGRP ? Qtrue : Qfalse;
+#endif
+#ifdef S_IROTH
+    if (!(st->st_mode & S_IROTH)) return Qfalse;
+#endif
+    return Qtrue;
+}
+
+static VALUE
+rb_stat_w(obj)
+    VALUE obj;
+{
+    struct stat *st = get_stat(obj);
+
+#ifdef S_IWUSR
+    if (rb_stat_owned(obj))
+	return st->st_mode & S_IWUSR ? Qtrue : Qfalse;
+#endif
+#ifdef S_IWGRP
+    if (rb_stat_grpowned(obj))
+	return st->st_mode & S_IWGRP ? Qtrue : Qfalse;
+#endif
+#ifdef S_IWOTH
+    if (!(st->st_mode & S_IWOTH)) return Qfalse;
+#endif
+    return Qtrue;
+}
+
+static VALUE
+rb_stat_W(obj)
+    VALUE obj;
+{
+    struct stat *st = get_stat(obj);
+
+#ifdef S_IWUSR
+    if (rb_stat_rowned(obj))
+	return st->st_mode & S_IWUSR ? Qtrue : Qfalse;
+#endif
+#ifdef S_IWGRP
+    if (group_member(get_stat(obj)->st_gid))
+	return st->st_mode & S_IWGRP ? Qtrue : Qfalse;
+#endif
+#ifdef S_IWOTH
+    if (!(st->st_mode & S_IWOTH)) return Qfalse;
+#endif
+    return Qtrue;
+}
+
+static VALUE
+rb_stat_x(obj)
+    VALUE obj;
+{
+    struct stat *st = get_stat(obj);
+
+#ifdef S_IXUSR
+    if (rb_stat_owned(obj))
+	return st->st_mode & S_IXUSR ? Qtrue : Qfalse;
+#endif
+#ifdef S_IXGRP
+    if (rb_stat_grpowned(obj))
+	return st->st_mode & S_IXGRP ? Qtrue : Qfalse;
+#endif
+#ifdef S_IXOTH
+    if (!(st->st_mode & S_IXOTH)) return Qfalse;
+#endif
+    return Qtrue;
+}
+
+static VALUE
+rb_stat_X(obj)
+    VALUE obj;
+{
+    struct stat *st = get_stat(obj);
+
+#ifdef S_IXUSR
+    if (rb_stat_rowned(obj))
+	return st->st_mode & S_IXUSR ? Qtrue : Qfalse;
+#endif
+#ifdef S_IXGRP
+    if (group_member(get_stat(obj)->st_gid))
+	return st->st_mode & S_IXGRP ? Qtrue : Qfalse;
+#endif
+#ifdef S_IXOTH
+    if (!(st->st_mode & S_IXOTH)) return Qfalse;
+#endif
+    return Qtrue;
+}
+
+static VALUE
+rb_stat_f(obj)
+    VALUE obj;
+{
+    if (S_ISREG(get_stat(obj)->st_mode)) return Qtrue;
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_z(obj)
+    VALUE obj;
+{
+    if (get_stat(obj)->st_size == 0) return Qtrue;
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_s(obj)
+    VALUE obj;
+{
+    int size = get_stat(obj)->st_size;
+
+    if (size == 0) return Qnil;
+    return INT2FIX(size);
+}
+
+static VALUE
+rb_stat_suid(obj)
+    VALUE obj;
+{
+#ifdef S_ISUID
+    if (get_stat(obj)->st_mode & S_ISUID) return Qtrue;
+#endif
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_sgid(obj)
+    VALUE obj;
+{
+#ifdef S_ISGID
+    if (get_stat(obj)->st_mode & S_ISGID) return Qtrue;
+#endif
+    return Qfalse;
+}
+
+static VALUE
+rb_stat_sticky(obj)
+    VALUE obj;
+{
+#ifdef S_ISVTX
+    if (get_stat(obj)->st_mode & S_ISVTX) return Qtrue;
+#endif
+    return Qfalse;
+}
+
 static VALUE rb_mConst;
 
 void
@@ -1567,40 +1998,202 @@ rb_file_const(name, value)
     rb_define_const(rb_mConst, name, value);
 }
 
-void
+static int
+is_absolute_path(path)
+    const char *path;
+{
+    if (path[0] == '/') return 1;
+# if defined DOSISH
+    if (path[0] == '\\') return 1;
+    if (strlen(path) > 2 && path[1] == ':') return 1;
+# endif
+    return 0;
+}
+
+static int
+path_check_1(path)
+    char *path;
+{
+    struct stat st;
+    char *p = 0;
+    char *s;
+
+    if (!is_absolute_path(path)) {
+	char buf[MAXPATHLEN+1];
+
+#ifdef HAVE_GETCWD
+	if (getcwd(buf, MAXPATHLEN) == 0) return 0;
+#else
+	if (getwd(buf) == 0) return 0;
+#endif
+	strncat(buf, "/", MAXPATHLEN);
+	strncat(buf, path, MAXPATHLEN);
+	buf[MAXPATHLEN] = '\0';
+	return path_check_1(buf);
+    }
+    for (;;) {
+	if (stat(path, &st) == 0 && (st.st_mode & 002)) {
+	    return 0;
+	}
+	s = strrchr(path, '/');
+	if (p) *p = '/';
+	if (!s || s == path) return 1;
+	p = s;
+	*p = '\0';
+    }
+}
+
+int
+rb_path_check(path)
+    char *path;
+{
+    char *p, *pend;
+    const char sep = PATH_SEP_CHAR;
+
+    if (!path) return 1;
+
+    p = path;
+    pend = strchr(path, sep);
+    
+    for (;;) {
+	int safe;
+
+	if (pend) *pend = '\0';
+	safe = path_check_1(p);
+	if (!safe) return 0;
+	if (!pend) break;
+	*pend = sep;
+	p = pend + 1;
+	pend = strchr(p, sep);
+    }
+    return 1;
+}
+
+#if defined(__MACOS__) || defined(riscos)
+static int
+is_macos_native_path(path)
+    const char *path;
+{
+    if (strchr(path, ':')) return 1;
+    return 0;
+}
+#endif
+
+char*
+rb_find_file(file)
+    char *file;
+{
+    extern VALUE rb_load_path;
+    volatile VALUE vpath;
+    VALUE fname;
+    char *path;
+    struct stat st;
+
+#if defined(__MACOS__) || defined(riscos)
+    if (is_macos_native_path(file)) {
+	FILE *f;
+
+	if (safe_level >= 2 && !rb_path_check(file)) {
+	    rb_raise(rb_eSecurityError, "loading from unsafe file %s", file);
+	}
+	f= fopen(file, "r");
+	if (f == NULL) return 0;
+	fclose(f);
+	return file;
+    }
+#endif
+
+    if (is_absolute_path(file)) {
+	FILE *f;
+
+	if (rb_safe_level() >= 2 && !rb_path_check(file)) {
+	    rb_raise(rb_eSecurityError, "loading from unsafe file %s", file);
+	}
+	f = fopen(file, "r");
+	if (f == NULL) return 0;
+	fclose(f);
+	return file;
+    }
+
+    if (file[0] == '~') {
+	fname = rb_str_new2(file);
+	fname = rb_file_s_expand_path(1, &fname);
+	if (rb_safe_level() >= 2 && OBJ_TAINTED(fname)) {
+	    rb_raise(rb_eSecurityError, "loading from unsafe file %s", file);
+	}
+	file = STR2CSTR(fname);
+    }
+
+    if (rb_load_path) {
+	int i;
+
+	Check_Type(rb_load_path, T_ARRAY);
+	vpath = rb_ary_new();
+	for (i=0;i<RARRAY(rb_load_path)->len;i++) {
+	    VALUE str = RARRAY(rb_load_path)->ptr[i];
+	    Check_SafeStr(str);
+	    if (RSTRING(str)->len > 0) {
+		rb_ary_push(vpath, str);
+	    }
+	}
+	vpath = rb_ary_join(vpath, rb_str_new2(PATH_SEP));
+	path = STR2CSTR(vpath);
+	if (rb_safe_level() >= 2 && !rb_path_check(path)) {
+	    rb_raise(rb_eSecurityError, "loading from unsafe path %s", path);
+	}
+    }
+    else {
+	path = 0;
+    }
+
+    path = dln_find_file(file, path);
+    if (path && stat(path, &st) == 0) {
+	return path;
+    }
+    return 0;
+}
+
+static void
+define_filetest_function(name, func, argc)
+    const char *name;
+    VALUE (*func)();
+    int argc;
+{
+    rb_define_module_function(rb_mFileTest, name, func, argc);
+    rb_define_singleton_method(rb_cFile, name, func, argc);
+}
+
 Init_File()
 {
     rb_mFileTest = rb_define_module("FileTest");
-
-    rb_define_module_function(rb_mFileTest, "directory?",  test_d, 1);
-    rb_define_module_function(rb_mFileTest, "exist?",  test_e, 1);
-    rb_define_module_function(rb_mFileTest, "exists?",  test_e, 1); /* temporary */
-    rb_define_module_function(rb_mFileTest, "readable?",  test_r, 1);
-    rb_define_module_function(rb_mFileTest, "readable_real?",  test_R, 1);
-    rb_define_module_function(rb_mFileTest, "writable?",  test_w, 1);
-    rb_define_module_function(rb_mFileTest, "writable_real?",  test_W, 1);
-    rb_define_module_function(rb_mFileTest, "executable?",  test_x, 1);
-    rb_define_module_function(rb_mFileTest, "executable_real?",  test_X, 1);
-    rb_define_module_function(rb_mFileTest, "file?",  test_f, 1);
-    rb_define_module_function(rb_mFileTest, "zero?",  test_z, 1);
-    rb_define_module_function(rb_mFileTest, "size?",  test_s, 1);
-    rb_define_module_function(rb_mFileTest, "size",   test_s, 1);
-    rb_define_module_function(rb_mFileTest, "owned?",  test_owned, 1);
-    rb_define_module_function(rb_mFileTest, "grpowned?",  test_grpowned, 1);
-
-    rb_define_module_function(rb_mFileTest, "pipe?",  test_p, 1);
-    rb_define_module_function(rb_mFileTest, "symlink?",  test_l, 1);
-    rb_define_module_function(rb_mFileTest, "socket?",  test_S, 1);
-
-    rb_define_module_function(rb_mFileTest, "blockdev?",  test_b, 1);
-    rb_define_module_function(rb_mFileTest, "chardev?",  test_c, 1);
-
-    rb_define_module_function(rb_mFileTest, "setuid?",  test_suid, 1);
-    rb_define_module_function(rb_mFileTest, "setgid?",  test_sgid, 1);
-    rb_define_module_function(rb_mFileTest, "sticky?",  test_sticky, 1);
-
     rb_cFile = rb_define_class("File", rb_cIO);
-    rb_extend_object(rb_cFile, CLASS_OF(rb_mFileTest));
+
+    define_filetest_function("directory?", test_d, 1);
+    define_filetest_function("exist?", test_e, 1);
+    define_filetest_function("exists?", test_e, 1); /* temporary */
+    define_filetest_function("readable?", test_r, 1);
+    define_filetest_function("readable_real?", test_R, 1);
+    define_filetest_function("writable?", test_w, 1);
+    define_filetest_function("writable_real?", test_W, 1);
+    define_filetest_function("executable?", test_x, 1);
+    define_filetest_function("executable_real?", test_X, 1);
+    define_filetest_function("file?", test_f, 1);
+    define_filetest_function("zero?", test_z, 1);
+    define_filetest_function("size?", test_s, 1);
+    define_filetest_function("size", test_s, 1);
+    define_filetest_function("owned?", test_owned, 1);
+    define_filetest_function("grpowned?", test_grpowned, 1);
+
+    define_filetest_function("pipe?", test_p, 1);
+    define_filetest_function("symlink?", test_l, 1);
+    define_filetest_function("socket?", test_S, 1);
+
+    define_filetest_function("blockdev?", test_b, 1);
+    define_filetest_function("chardev?", test_c, 1);
+
+    define_filetest_function("setuid?", test_suid, 1);
+    define_filetest_function("setgid?", test_sgid, 1);
+    define_filetest_function("sticky?", test_sticky, 1);
 
     rb_define_singleton_method(rb_cFile, "stat",  rb_file_s_stat, 1);
     rb_define_singleton_method(rb_cFile, "lstat", rb_file_s_lstat, 1);
@@ -1634,7 +2227,7 @@ Init_File()
     rb_define_singleton_method(rb_cFile, "split",  rb_file_s_split, 1);
     rb_define_singleton_method(rb_cFile, "join",   rb_file_s_join, -2);
 
-#ifdef DOSISH
+#if defined DOSISH && !defined __CYGWIN__
     rb_define_const(rb_cFile, "ALT_SEPARATOR", rb_str_new2("\\"));
 #else
     rb_define_const(rb_cFile, "ALT_SEPARATOR", Qnil);
@@ -1664,8 +2257,51 @@ Init_File()
 
     rb_define_global_function("test", rb_f_test, -1);
 
-    sStat = rb_struct_define("Stat", "dev", "ino", "mode",
-			     "nlink", "uid", "gid", "rdev",
-			     "size", "blksize", "blocks", 
-			     "atime", "mtime", "ctime", 0);
+    rb_cStat = rb_define_class_under(rb_cFile, "Stat", rb_cObject);
+
+    rb_include_module(rb_cStat, rb_mComparable);
+
+    rb_define_method(rb_cStat, "<=>", rb_stat_cmp, 1);
+
+    rb_define_method(rb_cStat, "dev", rb_stat_dev, 0);
+    rb_define_method(rb_cStat, "ino", rb_stat_ino, 0);
+    rb_define_method(rb_cStat, "mode", rb_stat_mode, 0);
+    rb_define_method(rb_cStat, "nlink", rb_stat_nlink, 0);
+    rb_define_method(rb_cStat, "uid", rb_stat_uid, 0);
+    rb_define_method(rb_cStat, "gid", rb_stat_gid, 0);
+    rb_define_method(rb_cStat, "rdev", rb_stat_rdev, 0);
+    rb_define_method(rb_cStat, "size", rb_stat_size, 0);
+    rb_define_method(rb_cStat, "blksize", rb_stat_blksize, 0);
+    rb_define_method(rb_cStat, "blocks", rb_stat_blocks, 0);
+    rb_define_method(rb_cStat, "atime", rb_stat_atime, 0);
+    rb_define_method(rb_cStat, "mtime", rb_stat_mtime, 0);
+    rb_define_method(rb_cStat, "ctime", rb_stat_ctime, 0);
+
+    rb_define_method(rb_cStat, "inspect", rb_stat_inspect, 0);
+
+    rb_define_method(rb_cStat, "ftype", rb_stat_ftype, 0);
+
+    rb_define_method(rb_cStat, "directory?",  rb_stat_d, 0);
+    rb_define_method(rb_cStat, "readable?",  rb_stat_r, 0);
+    rb_define_method(rb_cStat, "readable_real?",  rb_stat_R, 0);
+    rb_define_method(rb_cStat, "writable?",  rb_stat_w, 0);
+    rb_define_method(rb_cStat, "writable_real?",  rb_stat_W, 0);
+    rb_define_method(rb_cStat, "executable?",  rb_stat_x, 0);
+    rb_define_method(rb_cStat, "executable_real?",  rb_stat_X, 0);
+    rb_define_method(rb_cStat, "file?",  rb_stat_f, 0);
+    rb_define_method(rb_cStat, "zero?",  rb_stat_z, 0);
+    rb_define_method(rb_cStat, "size?",  rb_stat_s, 0);
+    rb_define_method(rb_cStat, "owned?",  rb_stat_owned, 0);
+    rb_define_method(rb_cStat, "grpowned?",  rb_stat_grpowned, 0);
+
+    rb_define_method(rb_cStat, "pipe?",  rb_stat_p, 0);
+    rb_define_method(rb_cStat, "symlink?",  rb_stat_l, 0);
+    rb_define_method(rb_cStat, "socket?",  rb_stat_S, 0);
+
+    rb_define_method(rb_cStat, "blockdev?",  rb_stat_b, 0);
+    rb_define_method(rb_cStat, "chardev?",  rb_stat_c, 0);
+
+    rb_define_method(rb_cStat, "setuid?",  rb_stat_suid, 0);
+    rb_define_method(rb_cStat, "setgid?",  rb_stat_sgid, 0);
+    rb_define_method(rb_cStat, "sticky?",  rb_stat_sticky, 0);
 }
