@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   gc.c -
 
   $Author: matz $
-  $Date: 2000/06/05 09:00:53 $
+  $Date: 2000/09/12 05:37:21 $
   created at: Tue Oct  5 09:44:46 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "rubysig.h"
@@ -41,15 +43,21 @@ static void run_final();
 #if defined(MSDOS) || defined(__human68k__)
 #define GC_MALLOC_LIMIT 100000
 #else
-#define GC_MALLOC_LIMIT 400000
+#define GC_MALLOC_LIMIT 4000000
 #endif
 #endif
-#define GC_NEWOBJ_LIMIT 10000
 
 static unsigned long malloc_memories = 0;
-static unsigned long alloc_objects = 0;
 
-static int malloc_called = 0;
+static void
+mem_error(mesg)
+    char *mesg;
+{
+    if (rb_safe_level() >= 4) {
+	rb_raise(rb_eNoMemError, mesg);
+    }
+    rb_fatal(mesg);
+}
 
 void *
 ruby_xmalloc(size)
@@ -58,20 +66,23 @@ ruby_xmalloc(size)
     void *mem;
 
     if (size < 0) {
-	rb_raise(rb_eArgError, "negative allocation size (or too big)");
+	rb_raise(rb_eNoMemError, "negative allocation size (or too big)");
     }
     if (size == 0) size = 1;
     malloc_memories += size;
-    if (malloc_memories > GC_MALLOC_LIMIT && alloc_objects > GC_NEWOBJ_LIMIT) {
+    if (malloc_memories > GC_MALLOC_LIMIT) {
 	rb_gc();
     }
-    malloc_called++;
     mem = malloc(size);
     if (!mem) {
 	rb_gc();
 	mem = malloc(size);
-	if (!mem)
-	    rb_fatal("failed to allocate memory");
+	if (!mem) {
+	    if (size >= 10 * 1024 * 1024) {
+		rb_raise(rb_eNoMemError, "tryed to allocate too big memory");
+	    }
+	    mem_error("failed to allocate memory");
+	}
     }
 
     return mem;
@@ -107,7 +118,10 @@ ruby_xrealloc(ptr, size)
 	rb_gc();
 	mem = realloc(ptr, size);
 	if (!mem)
-	    rb_fatal("failed to allocate memory(realloc)");
+	    if (size >= 10 * 1024 * 1024) {
+		rb_raise(rb_eNoMemError, "tryed to re-allocate too big memory");
+	    }
+	    mem_error("failed to allocate memory(realloc)");
     }
 
     return mem;
@@ -120,40 +134,11 @@ ruby_xfree(x)
     if (x) free(x);
 }
 
-/* The way of garbage collecting which allows use of the cstack is due to */
-/* Scheme In One Defun, but in C this time.
-
- *			  COPYRIGHT (c) 1989 BY				    *
- *	  PARADIGM ASSOCIATES INCORPORATED, CAMBRIDGE, MASSACHUSETTS.	    *
- *			   ALL RIGHTS RESERVED				    *
-
-Permission to use, copy, modify, distribute and sell this software
-and its documentation for any purpose and without fee is hereby
-granted, provided that the above copyright notice appear in all copies
-and that both that copyright notice and this permission notice appear
-in supporting documentation, and that the name of Paradigm Associates
-Inc not be used in advertising or publicity pertaining to distribution
-of the software without specific, written prior permission.
-
-PARADIGM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
-ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
-PARADIGM BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
-ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
-ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
-SOFTWARE.
-
-gjc@paradigm.com
-
-Paradigm Associates Inc		 Phone: 617-492-6079
-29 Putnam Ave, Suite 6
-Cambridge, MA 02138
-*/
-
 extern int ruby_in_compile;
 static int dont_gc;
 static int during_gc;
 static int need_call_final = 0;
+static st_table *finalizer_table = 0;
 
 static VALUE
 gc_enable()
@@ -181,17 +166,47 @@ static struct gc_list {
 } *Global_List = 0;
 
 void
-rb_global_variable(var)
-    VALUE *var;
+rb_gc_register_address(addr)
+    VALUE *addr;
 {
     struct gc_list *tmp;
 
     tmp = ALLOC(struct gc_list);
     tmp->next = Global_List;
-    tmp->varptr = var;
+    tmp->varptr = addr;
     Global_List = tmp;
 }
 
+void
+rb_gc_unregister_address(addr)
+    VALUE *addr;
+{
+    struct gc_list *tmp = Global_List;
+
+    if (tmp->varptr == addr) {
+	Global_List = tmp->next;
+	free(tmp);
+	return;
+    }
+    while (tmp->next) {
+	if (tmp->next->varptr == addr) {
+	    struct gc_list *t = tmp->next;
+
+	    tmp->next = tmp->next->next;
+	    free(t);
+	    break;
+	}
+	tmp = tmp->next;
+    }
+}
+
+void
+rb_global_variable(var)
+    VALUE *var;
+{
+    rb_gc_register_address(var);
+}
+
 typedef struct RVALUE {
     union {
 	struct {
@@ -225,7 +240,7 @@ static int heaps_length = 0;
 static int heaps_used   = 0;
 
 #define HEAP_SLOTS 10000
-#define FREE_MIN  512
+#define FREE_MIN  4096
 
 static RVALUE *himem, *lomem;
 
@@ -240,11 +255,11 @@ add_heap()
 	heaps = (heaps_used>0)?
 	    (RVALUE**)realloc(heaps, heaps_length*sizeof(RVALUE*)):
 	    (RVALUE**)malloc(heaps_length*sizeof(RVALUE*));
-	if (heaps == 0) rb_fatal("can't alloc memory");
+	if (heaps == 0) mem_error("heaps: can't alloc memory");
     }
 
     p = heaps[heaps_used++] = (RVALUE*)malloc(sizeof(RVALUE)*HEAP_SLOTS);
-    if (p == 0) rb_fatal("add_heap: can't alloc memory");
+    if (p == 0) mem_error("add_heap: can't alloc memory");
     pend = p + HEAP_SLOTS;
     if (lomem == 0 || lomem > p) lomem = p;
     if (himem < pend) himem = pend;
@@ -263,25 +278,20 @@ rb_newobj()
 {
     VALUE obj;
 
-    if (freelist) {
-      retry:
-	obj = (VALUE)freelist;
-	freelist = freelist->as.free.next;
-	alloc_objects++;
-	return obj;
-    }
-    if (dont_gc || during_gc || rb_prohibit_interrupt) add_heap();
-    else rb_gc();
+    if (!freelist) rb_gc();
 
-    goto retry;
+    obj = (VALUE)freelist;
+    freelist = freelist->as.free.next;
+    MEMZERO((void*)obj, RVALUE, 1);
+    return obj;
 }
 
 VALUE
 rb_data_object_alloc(klass, datap, dmark, dfree)
     VALUE klass;
     void *datap;
-    void (*dfree)();
-    void (*dmark)();
+    RUBY_DATA_FUNC dmark;
+    RUBY_DATA_FUNC dfree;
 {
     NEWOBJ(data, struct RData);
     OBJSETUP(data, klass, T_DATA);
@@ -299,7 +309,7 @@ VALUE *rb_gc_stack_start = 0;
 __inline__
 #endif
 static int
-looks_pointerp(ptr)
+is_pointer_to_heap(ptr)
     void *ptr;
 {
     register RVALUE *p = RANY(ptr);
@@ -324,7 +334,7 @@ mark_locations_array(x, n)
     register long n;
 {
     while (n--) {
-	if (looks_pointerp(*x)) {
+	if (is_pointer_to_heap(*x)) {
 	    rb_gc_mark(*x);
 	}
 	x++;
@@ -386,7 +396,7 @@ void
 rb_gc_mark_maybe(obj)
     void *obj;
 {
-    if (looks_pointerp(obj)) {
+    if (is_pointer_to_heap(obj)) {
 	rb_gc_mark(obj);
     }
 }
@@ -469,9 +479,11 @@ rb_gc_mark(ptr)
 	  case NODE_GASGN:
 	  case NODE_LASGN:
 	  case NODE_DASGN:
-	  case NODE_DASGN_PUSH:
+	  case NODE_DASGN_CURR:
 	  case NODE_IASGN:
-	  case NODE_CASGN:
+	  case NODE_CDECL:
+	  case NODE_CVDECL:
+	  case NODE_CVASGN2:
 	  case NODE_MODULE:
 	  case NODE_COLON3:
 	  case NODE_OPT_N:
@@ -507,6 +519,7 @@ rb_gc_mark(ptr)
 	  case NODE_DVAR:
 	  case NODE_IVAR:
 	  case NODE_CVAR:
+	  case NODE_CVAR2:
 	  case NODE_NTH_REF:
 	  case NODE_BACK_REF:
 	  case NODE_ALIAS:
@@ -533,13 +546,13 @@ rb_gc_mark(ptr)
 #endif
 
 	  default:
-	    if (looks_pointerp(obj->as.node.u1.node)) {
+	    if (is_pointer_to_heap(obj->as.node.u1.node)) {
 		rb_gc_mark(obj->as.node.u1.node);
 	    }
-	    if (looks_pointerp(obj->as.node.u2.node)) {
+	    if (is_pointer_to_heap(obj->as.node.u2.node)) {
 		rb_gc_mark(obj->as.node.u2.node);
 	    }
-	    if (looks_pointerp(obj->as.node.u3.node)) {
+	    if (is_pointer_to_heap(obj->as.node.u3.node)) {
 		obj = RANY(obj->as.node.u3.node);
 		goto Top;
 	    }
@@ -607,8 +620,7 @@ rb_gc_mark(ptr)
 	break;
 
       case T_SCOPE:
-	if (obj->as.scope.local_vars &&
-            obj->as.scope.flag != SCOPE_ALLOCA) {
+	if (obj->as.scope.local_vars && (obj->as.scope.flag & SCOPE_MALLOC)) {
 	    int n = obj->as.scope.local_tbl[0]+1;
 	    VALUE *vars = &obj->as.scope.local_vars[-1];
 
@@ -632,12 +644,10 @@ rb_gc_mark(ptr)
       default:
 	rb_bug("rb_gc_mark(): unknown data type 0x%x(0x%x) %s",
 	       obj->as.basic.flags & T_MASK, obj,
-	       looks_pointerp(obj)?"corrupted object":"non object");
+	       is_pointer_to_heap(obj)?"corrupted object":"non object");
     }
 }
 
-#define MIN_FREE_OBJ 512
-
 static void obj_free _((VALUE));
 
 static void
@@ -647,18 +657,6 @@ gc_sweep()
     int freed = 0;
     int i, used = heaps_used;
 
-    if (ruby_in_compile) {
-	/* sould not reclaim nodes during compilation */
-	for (i = 0; i < used; i++) {
-	    p = heaps[i]; pend = p + HEAP_SLOTS;
-	    while (p < pend) {
-		if (!(p->as.basic.flags&FL_MARK) && BUILTIN_TYPE(p) == T_NODE)
-		    rb_gc_mark(p);
-		p++;
-	    }
-	}
-    }
-
     freelist = 0;
     final_list = 0;
     for (i = 0; i < used; i++) {
@@ -670,7 +668,7 @@ gc_sweep()
 		if (p->as.basic.flags) {
 		    obj_free((VALUE)p);
 		}
-	if (need_call_final && FL_TEST(p, FL_FINALIZE)) {
+		if (need_call_final && FL_TEST(p, FL_FINALIZE)) {
 		    p->as.free.flag = FL_MARK; /* remain marked */
 		    p->as.free.next = final_list;
 		    final_list = p;
@@ -753,7 +751,7 @@ obj_free(obj)
 	}
 	break;
       case T_STRING:
-#define STR_NO_ORIG FL_USER3	/* copied from string.c */
+#define STR_NO_ORIG FL_USER2	/* copied from string.c */
 	if (!RANY(obj)->as.string.orig || FL_TEST(obj, STR_NO_ORIG))
 	    free(RANY(obj)->as.string.ptr);
 	break;
@@ -899,7 +897,14 @@ rb_gc()
 # define STACK_END (stack_end)
 #endif
 
-    alloc_objects = 0;
+    if (dont_gc || during_gc || rb_prohibit_interrupt || ruby_in_compile) {
+	if (!freelist || malloc_memories > GC_MALLOC_LIMIT) {
+	    malloc_memories = 0;
+	    add_heap();
+	}
+	return;
+    }
+
     malloc_memories = 0;
 
     if (during_gc) return;
@@ -919,17 +924,16 @@ rb_gc()
     rb_gc_mark(ruby_class);
     rb_gc_mark(ruby_scope);
     rb_gc_mark(ruby_dyna_vars);
+    if (finalizer_table) {
+	rb_mark_tbl(finalizer_table);
+    }
 
     FLUSH_REGISTER_WINDOWS;
     /* This assumes that all registers are saved into the jmp_buf */
     setjmp(save_regs_gc_mark);
     mark_locations_array((VALUE*)save_regs_gc_mark, sizeof(save_regs_gc_mark) / sizeof(VALUE *));
     rb_gc_mark_locations(rb_gc_stack_start, (VALUE*)STACK_END);
-#if defined(THINK_C) || defined(__human68k__)
-#ifndef __human68k__
-    mark_locations_array((VALUE*)((char*)save_regs_gc_mark+2),
-			 sizeof(save_regs_gc_mark) / sizeof(VALUE *));
-#endif
+#if defined(__human68k__)
     rb_gc_mark_locations((VALUE*)((char*)rb_gc_stack_start + 2),
 			 (VALUE*)((char*)STACK_END + 2));
 #endif
@@ -1068,6 +1072,7 @@ static VALUE
 add_final(os, proc)
     VALUE os, proc;
 {
+    rb_warn("ObjectSpace::add_finalizer is deprecated; use define_finalizer");
     if (!rb_obj_is_kind_of(proc, rb_cProc)) {
 	rb_raise(rb_eArgError, "wrong type argument %s (Proc required)",
 		 rb_class2name(CLASS_OF(proc)));
@@ -1080,6 +1085,7 @@ static VALUE
 rm_final(os, proc)
     VALUE os, proc;
 {
+    rb_warn("ObjectSpace::remove_finalizer is deprecated; use undefine_finalizer");
     rb_ary_delete(finalizers, proc);
     return proc;
 }
@@ -1087,6 +1093,7 @@ rm_final(os, proc)
 static VALUE
 finals()
 {
+    rb_warn("ObjectSpace::finals is deprecated");
     return finalizers;
 }
 
@@ -1094,11 +1101,55 @@ static VALUE
 call_final(os, obj)
     VALUE os, obj;
 {
+    rb_warn("ObjectSpace::call_final is deprecated; use define_finalizer");
     need_call_final = 1;
     FL_SET(obj, FL_FINALIZE);
     return obj;
 }
 
+static VALUE
+undefine_final(os, obj)
+    VALUE os, obj;
+{
+    VALUE table;
+
+    if (finalizer_table) {
+	st_delete(finalizer_table, &obj, 0);
+    }
+    return obj;
+}
+
+static VALUE
+define_final(argc, argv, os)
+    int argc;
+    VALUE *argv;
+    VALUE os;
+{
+    VALUE obj, proc, table;
+
+    rb_scan_args(argc, argv, "11", &obj, &proc);
+    if (argc == 1) {
+	proc = rb_f_lambda();
+    }
+    else if (!rb_obj_is_kind_of(proc, rb_cProc)) {
+	rb_raise(rb_eArgError, "wrong type argument %s (Proc required)",
+		 rb_class2name(CLASS_OF(proc)));
+    }
+    need_call_final = 1;
+    FL_SET(obj, FL_FINALIZE);
+
+    if (!finalizer_table) {
+	finalizer_table = st_init_numtable();
+    }
+    if (st_lookup(finalizer_table, obj, &table)) {
+	rb_ary_push(table, proc);
+    }
+    else {
+	st_add_direct(finalizer_table, obj, rb_ary_new3(1, proc));
+    }
+    return proc;
+}
+
 static VALUE
 run_single_final(args)
     VALUE *args;
@@ -1112,14 +1163,20 @@ run_final(obj)
     VALUE obj;
 {
     int i, status;
-    VALUE args[2];
+    VALUE args[2], table;
 
-    obj = rb_obj_id(obj);	/* make obj into id */
-    args[1] = rb_ary_new3(1, obj);
+    args[1] = rb_ary_new3(1, rb_obj_id(obj)); /* make obj into id */
     for (i=0; i<RARRAY(finalizers)->len; i++) {
 	args[0] = RARRAY(finalizers)->ptr[i];
 	rb_protect(run_single_final, (VALUE)args, &status);
     }
+    if (finalizer_table && st_lookup(finalizer_table, obj, &table)) {
+	st_delete(finalizer_table, &obj, 0);
+	for (i=0; i<RARRAY(table)->len; i++) {
+	    args[0] = RARRAY(table)->ptr[i];
+	    rb_protect(run_single_final, (VALUE)args, &status);
+	}
+    }
 }
 
 void
@@ -1168,12 +1225,12 @@ id2ref(obj, id)
     if (ptr == Qfalse) return Qfalse;
     if (ptr == Qnil) return Qnil;
 
-    ptr = id ^ FIXNUM_FLAG;
-    if (!looks_pointerp(ptr)) {
-	rb_raise(rb_eIndexError, "0x%x is not id value", ptr);
+    ptr = id ^ FIXNUM_FLAG;	/* unset FIXNUM_FLAG */
+    if (!is_pointer_to_heap(ptr)) {
+	rb_raise(rb_eRangeError, "0x%x is not id value", ptr);
     }
     if (BUILTIN_TYPE(ptr) == 0) {
-	rb_raise(rb_eIndexError, "0x%x is recycled object", ptr);
+	rb_raise(rb_eRangeError, "0x%x is recycled object", ptr);
     }
     return (VALUE)ptr;
 }
@@ -1196,9 +1253,15 @@ Init_GC()
     rb_define_module_function(rb_mObSpace, "remove_finalizer", rm_final, 1);
     rb_define_module_function(rb_mObSpace, "finalizers", finals, 0);
     rb_define_module_function(rb_mObSpace, "call_finalizer", call_final, 1);
+
+    rb_define_module_function(rb_mObSpace, "define_finalizer", define_final, -1);
+    rb_define_module_function(rb_mObSpace, "undefine_finalizer", undefine_final, 1);
+
     rb_define_module_function(rb_mObSpace, "_id2ref", id2ref, 1);
 
+    rb_gc_register_address(&rb_mObSpace);
     rb_global_variable(&finalizers);
+    rb_gc_unregister_address(&rb_mObSpace);
     finalizers = rb_ary_new();
 }
 
@@ -1235,3 +1298,4 @@ xfree(ptr)
 {
     ruby_xfree(ptr);
 }
+
