@@ -1,5 +1,20 @@
-/* Functions and variables that are used by more than one source file of
- * the kernel.  Not available to extensions and applications.
+/**********************************************************************
+
+  intern.h -
+
+  $Author: matz $
+  $Date: 2000/09/15 06:00:24 $
+  created at: Thu Jun 10 14:22:17 JST 1993
+
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
+
+**********************************************************************/
+
+/* 
+ * Functions and variables that are used by more than one source file of
+ * the kernel.
  */
 
 /* array.c */
@@ -25,7 +40,7 @@ VALUE rb_ary_reverse _((VALUE));
 VALUE rb_ary_sort _((VALUE));
 VALUE rb_ary_sort_bang _((VALUE));
 VALUE rb_ary_delete _((VALUE, VALUE));
-VALUE rb_ary_delete_at _((VALUE, VALUE));
+VALUE rb_ary_delete_at _((VALUE, long));
 VALUE rb_ary_plus _((VALUE, VALUE));
 VALUE rb_ary_concat _((VALUE, VALUE));
 VALUE rb_ary_assoc _((VALUE, VALUE));
@@ -41,7 +56,8 @@ VALUE rb_uint2big _((unsigned long));
 VALUE rb_int2big _((long));
 VALUE rb_uint2inum _((unsigned long));
 VALUE rb_int2inum _((long));
-VALUE rb_str2inum _((const char*, int));
+VALUE rb_cstr2inum _((const char*, int));
+VALUE rb_str2inum _((VALUE, int));
 VALUE rb_big2str _((VALUE, int));
 long rb_big2long _((VALUE));
 #define rb_big2int(x) rb_big2long(x)
@@ -52,6 +68,7 @@ double rb_big2dbl _((VALUE));
 VALUE rb_big_plus _((VALUE, VALUE));
 VALUE rb_big_minus _((VALUE, VALUE));
 VALUE rb_big_mul _((VALUE, VALUE));
+VALUE rb_big_divmod _((VALUE, VALUE));
 VALUE rb_big_pow _((VALUE, VALUE));
 VALUE rb_big_and _((VALUE, VALUE));
 VALUE rb_big_or _((VALUE, VALUE));
@@ -89,6 +106,7 @@ VALUE rb_exc_new3 _((VALUE, VALUE));
 void rb_loaderror __((const char*, ...)) NORETURN;
 void rb_compile_error __((const char*, ...));
 void rb_compile_error_append __((const char*, ...));
+void rb_error_frozen _((char*));
 /* eval.c */
 void rb_exc_raise _((VALUE)) NORETURN;
 void rb_exc_fatal _((VALUE)) NORETURN;
@@ -100,6 +118,7 @@ void rb_alias _((VALUE, ID, ID));
 void rb_attr _((VALUE,ID,int,int,int));
 int rb_method_boundp _((VALUE, ID, int));
 VALUE rb_dvar_defined _((ID));
+VALUE rb_dvar_curr _((ID));
 VALUE rb_dvar_ref _((ID));
 void rb_dvar_asgn _((ID, VALUE));
 void rb_dvar_push _((ID, VALUE));
@@ -130,6 +149,7 @@ void rb_thread_wait_fd _((int));
 int rb_thread_fd_writable _((int));
 void rb_thread_fd_close _((int));
 int rb_thread_alone _((void));
+void rb_thread_polling _((void));
 void rb_thread_sleep _((int));
 void rb_thread_sleep_forever _((void));
 VALUE rb_thread_stop _((void));
@@ -150,8 +170,8 @@ VALUE rb_thread_local_aset _((VALUE, ID, VALUE));
 int eaccess _((const char*, int));
 VALUE rb_file_s_expand_path _((int, VALUE *));
 void rb_file_const _((const char*, VALUE));
+char *rb_find_file _((char*));
 /* gc.c */
-void rb_global_variable _((VALUE*));
 void rb_gc_mark_locations _((VALUE*, VALUE*));
 void rb_mark_tbl _((struct st_table*));
 void rb_mark_hash _((struct st_table*));
@@ -166,6 +186,7 @@ VALUE rb_hash_new _((void));
 VALUE rb_hash_freeze _((VALUE));
 VALUE rb_hash_aref _((VALUE, VALUE));
 VALUE rb_hash_aset _((VALUE, VALUE, VALUE));
+VALUE rb_hash_delete_if _((VALUE));
 int rb_path_check _((char *));
 int rb_env_path_tainted _((void));
 /* io.c */
@@ -183,7 +204,6 @@ VALUE rb_io_eof _((VALUE));
 VALUE rb_io_binmode _((VALUE));
 VALUE rb_file_open _((const char*, const char*));
 VALUE rb_gets _((void));
-void rb_str_setter _((VALUE, ID, VALUE*));
 /* numeric.c */
 void rb_num_zerodiv _((void));
 VALUE rb_num_coerce_bin _((VALUE, VALUE));
@@ -199,9 +219,11 @@ VALUE rb_obj_is_instance_of _((VALUE, VALUE));
 VALUE rb_obj_is_kind_of _((VALUE, VALUE));
 VALUE rb_obj_alloc _((VALUE));
 VALUE rb_obj_clone _((VALUE));
+VALUE rb_obj_dup _((VALUE));
 VALUE rb_obj_taint _((VALUE));
 VALUE rb_obj_tainted _((VALUE));
 VALUE rb_obj_untaint _((VALUE));
+VALUE rb_obj_freeze _((VALUE));
 VALUE rb_obj_id _((VALUE));
 VALUE rb_convert_type _((VALUE,int,const char*,const char*));
 VALUE rb_Integer _((VALUE));
@@ -223,6 +245,7 @@ void rb_parser_append_print _((void));
 void rb_parser_while_loop _((int, int));
 int rb_is_const_id _((ID));
 int rb_is_instance_id _((ID));
+int rb_is_class_id _((ID));
 VALUE rb_backref_get _((void));
 void rb_backref_set _((VALUE));
 VALUE rb_lastline_get _((void));
@@ -233,7 +256,9 @@ void rb_syswait _((int));
 /* range.c */
 VALUE rb_range_new _((VALUE, VALUE, int));
 VALUE rb_range_beg_len _((VALUE, long*, long*, long, int));
+VALUE rb_length_by_each _((VALUE));
 /* re.c */
+int rb_memcmp _((char*,char*,size_t));
 VALUE rb_reg_nth_defined _((int, VALUE));
 VALUE rb_reg_nth_match _((int, VALUE));
 VALUE rb_reg_last_match _((VALUE));
@@ -244,10 +269,8 @@ VALUE rb_reg_new _((const char*, long, int));
 VALUE rb_reg_match _((VALUE, VALUE));
 VALUE rb_reg_match2 _((VALUE));
 int rb_reg_options _((VALUE));
-const char* rb_get_kcode _((void));
 void rb_set_kcode _((const char*));
-int rb_ignorecase_p _((void));
-void rb_match_busy _((VALUE, int));
+const char* rb_get_kcode _((void));
 /* ruby.c */
 EXTERN VALUE rb_argv;
 EXTERN VALUE rb_argv0;
@@ -256,8 +279,9 @@ void ruby_script _((char*));
 void ruby_prog_init _((void));
 void ruby_set_argv _((int, char**));
 void ruby_process_options _((int, char**));
-void ruby_require_libraries _((void));
 void ruby_load_script _((void));
+void ruby_init_loadpath _((void));
+void ruby_incpush _((const char*));
 /* signal.c */
 VALUE rb_f_kill _((int, VALUE*));
 void rb_gc_mark_trap_list _((void));
@@ -286,12 +310,16 @@ void rb_str_modify _((VALUE));
 VALUE rb_str_freeze _((VALUE));
 VALUE rb_str_resize _((VALUE, long));
 VALUE rb_str_cat _((VALUE, const char*, long));
+VALUE rb_str_cat2 _((VALUE, const char*));
+VALUE rb_str_append _((VALUE, VALUE));
 VALUE rb_str_concat _((VALUE, VALUE));
 int rb_str_hash _((VALUE));
 int rb_str_cmp _((VALUE, VALUE));
 VALUE rb_str_upto _((VALUE, VALUE, int));
 VALUE rb_str_inspect _((VALUE));
 VALUE rb_str_split _((VALUE, const char*));
+void rb_str_associate _((VALUE, VALUE));
+void rb_str_setter _((VALUE, ID, VALUE*));
 /* struct.c */
 VALUE rb_struct_new __((VALUE, ...));
 VALUE rb_struct_define __((const char*, ...));
@@ -312,7 +340,6 @@ VALUE rb_f_autoload _((VALUE, VALUE, VALUE));
 void rb_gc_mark_global_tbl _((void));
 VALUE rb_f_trace_var _((int, VALUE*));
 VALUE rb_f_untrace_var _((int, VALUE*));
-VALUE rb_gvar_set2 _((const char*, VALUE));
 VALUE rb_f_global_variables _((void));
 void rb_alias_variable _((ID, ID));
 struct st_table* rb_generic_ivar_table _((VALUE));
@@ -332,6 +359,20 @@ VALUE rb_mod_remove_const _((VALUE, VALUE));
 int rb_const_defined_at _((VALUE, ID));
 int rb_autoload_defined _((ID));
 int rb_const_defined _((VALUE, ID));
+VALUE rb_const_get _((VALUE, ID));
+VALUE rb_const_get_at _((VALUE, ID));
+void rb_const_set _((VALUE, ID, VALUE));
+void rb_const_assign _((VALUE, ID, VALUE));
+VALUE rb_mod_constants _((VALUE));
+void rb_autoload_load _((ID));
+void rb_cvar_declare _((VALUE, ID, VALUE));
+int rb_cvar_defined _((VALUE, ID));
+void rb_cvar_set _((VALUE, ID, VALUE));
+VALUE rb_cvar_get _((VALUE, ID));
+int rb_cvar_defined_singleton _((VALUE, ID));
+void rb_cvar_set_singleton _((VALUE, ID, VALUE));
+VALUE rb_cvar_get_singleton _((VALUE, ID));
+VALUE rb_mod_class_variables _((VALUE));
 /* version.c */
 void ruby_show_version _((void));
 void ruby_show_copyright _((void));
