@@ -60,7 +60,7 @@ module Exception2MessageMapper
     cl.bind(self) unless cl == E2MM
   end
   
-  # ȤθߴΤ˻ĤƤ.
+  # backward compatibility
   def E2MM.extend_to(b)
     c = eval("self", b)
     c.extend(self)
@@ -81,15 +81,15 @@ module Exception2MessageMapper
   end
 
   # Fail(err, *rest)
-  #	err:	㳰
-  #	rest:	åϤѥ᡼
+  #	err:	exception
+  #	rest:	message arguments
   #
   def Raise(err = nil, *rest)
     E2MM.Raise(self, err, *rest)
   end
   alias Fail Raise
 
-  # θߴΤ
+  # backward compatibility
   alias fail! fail
   def fail(err = nil, *rest)
     begin 
@@ -106,17 +106,17 @@ module Exception2MessageMapper
   # def_e2message(c, m)
   #	    c:  exception
   #	    m:  message_form
-  #	㳰cΥåmȤ.
+  #	define exception c with message m.
   #
   def def_e2message(c, m)
     E2MM.def_e2message(self, c, m)
   end
   
-  # def_exception(c, m)
+  # def_exception(c, m, s)
   #	    n:  exception_name
   #	    m:  message_form
-  #	    s:	㳰ѡ饹(ǥե: StandardError)
-  #	㳰̾``c''㳰, ΥåmȤ.
+  #	    s:	superclass(default: StandardError)
+  #	define exception named ``c'' with message m.
   #
   def def_exception(n, m, s = StandardError)
     E2MM.def_exception(self, n, m, s)
@@ -129,22 +129,22 @@ module Exception2MessageMapper
   @MessageMap = {}
 
   # E2MM.def_exception(k, e, m)
-  #	    k:  㳰륯饹
+  #	    k:  class to define exception under.
   #	    e:  exception
   #	    m:  message_form
-  #	㳰cΥåmȤ.
+  #	define exception c with message m.
   #
   def E2MM.def_e2message(k, c, m)
     E2MM.instance_eval{@MessageMap[[k, c]] = m}
     c
   end
   
-  # E2MM.def_exception(k, c, m)
-  #	    k:  㳰륯饹
+  # E2MM.def_exception(k, c, m, s)
+  #	    k:  class to define exception under.
   #	    n:  exception_name
   #	    m:  message_form
-  #	    s:	㳰ѡ饹(ǥե: StandardError)
-  #	㳰̾``c''㳰, ΥåmȤ.
+  #	    s:	superclass(default: StandardError)
+  #	define exception named ``c'' with message m.
   #
   def E2MM.def_exception(k, n, m, s = StandardError)
     n = n.id2name if n.kind_of?(Fixnum)
@@ -154,9 +154,9 @@ module Exception2MessageMapper
   end
 
   # Fail(klass, err, *rest)
-  #     klass:  㳰Ƥ륯饹
-  #	err:	㳰
-  #	rest:	åϤѥ᡼
+  #	klass:  class to define exception under.
+  #	err:	exception
+  #	rest:	message arguments
   #
   def E2MM.Raise(klass = E2MM, err = nil, *rest)
     if form = e2mm_message(klass, err)
@@ -177,7 +177,7 @@ module Exception2MessageMapper
   def E2MM.e2mm_message(klass, exp)
     for c in klass.ancestors
       if mes = @MessageMap[[c,exp]]
-	p mes
+	#p mes
 	m = klass.instance_eval('"' + mes + '"')
 	return m
       end
