@@ -1,8 +1,8 @@
 #
 #   mathn.rb - 
 #   	$Release Version: 0.5 $
-#   	$Revision: 1.1 $
-#   	$Date: 1996/11/11 04:25:24 $
+#   	$Revision: 1.1.1.1.4.1 $
+#   	$Date: 1998/01/16 12:36:05 $
 #   	by Keiju ISHITSUKA(SHL Japan Inc.)
 #
 # --
@@ -12,6 +12,7 @@
 
 require "rational.rb"
 require "complex.rb"
+require "matrix.rb"
 
 class Integer
 
@@ -95,6 +96,7 @@ class Prime
     @counts.push @seed + @seed
     return @seed
   end
+  alias next succ
 
   def each
     loop do
@@ -119,7 +121,7 @@ class Bignum
 end
 
 class Rational
-  Unify = TRUE
+  Unify = true
   
   alias power! **
 
@@ -302,6 +304,6 @@ module Math
 end
 
 class Complex
-  Unify = TRUE
+  Unify = true
 end
 
