@@ -4,7 +4,7 @@
 require 'rbconfig'
 require 'find'
 
-include Config
+CONFIG = Config::MAKEFILE_CONFIG
 
 SRC_EXT = ["c", "cc", "m", "cxx", "cpp", "C"]
 
@@ -30,17 +30,32 @@ $hdrdir.gsub!('/', '\\') if RUBY_PLATFORM =~ /mswin32/
 CFLAGS = CONFIG["CFLAGS"]
 if RUBY_PLATFORM == "m68k-human"
   CFLAGS.gsub!(/-c..-stack=[0-9]+ */, '')
-elsif RUBY_PLATFORM =~ /-nextstep|-rhapsody/
+elsif RUBY_PLATFORM =~ /-nextstep|-rhapsody|-darwin/
   CFLAGS.gsub!( /-arch\s\w*/, '' )
 end
-if /win32|djgpp|mingw32|m68k-human|i386-os2_emx/i =~ RUBY_PLATFORM
-  $null = open("nul", "w")
+
+if FileTest.readable? 'nul'
+  $null = open('nul', 'w')
+elsif FileTest.readable? '/dev/null'
+  $null = open('/dev/null', 'w')
 else
-  $null = open("/dev/null", "w")
+  $null = open('test.log', 'w')
 end
+
 LINK = "#{CONFIG['CC']} -o conftest -I#{$hdrdir} #{CFLAGS} -I#{CONFIG['includedir']} %s #{CONFIG['LDFLAGS']} %s conftest.c %s %s #{CONFIG['LIBS']}"
 CPP = "#{CONFIG['CPP']} -E -I#{$hdrdir} #{CFLAGS} -I#{CONFIG['includedir']} %s %s conftest.c"
 
+def rm_f(*files)
+  targets = []
+  for file in files
+    targets.concat Dir[file]
+  end
+  if not targets.empty?
+    File::chmod 0777, *targets
+    File::unlink *targets
+  end
+end
+
 $orgerr = $stderr.dup
 $orgout = $stdout.dup
 def xsystem command
@@ -67,7 +82,7 @@ def try_link(src, opt="")
   begin
     try_link0(src, opt)
   ensure
-    system "rm -f conftest*"
+    rm_f "conftest*"
   end
 end
 
@@ -78,7 +93,7 @@ def try_cpp(src, opt="")
   begin
     xsystem(format(CPP, $CFLAGS, opt))
   ensure
-    system "rm -f conftest*"
+    rm_f "conftest*"
   end
 end
 
@@ -89,7 +104,7 @@ def egrep_cpp(pat, src, opt="")
   begin
     xsystem(format(CPP+"|egrep #{pat}", $CFLAGS, opt))
   ensure
-    system "rm -f conftest*"
+    rm_f "conftest*"
   end
 end
 
@@ -105,7 +120,7 @@ def try_run(src, opt="")
       nil
     end
   ensure
-    system "rm -f conftest*"
+    rm_f "conftest*"
   end
 end
 
@@ -143,7 +158,7 @@ def have_library(lib, func="main")
 
   if func && func != ""
     libs = append_library($libs, lib)
-    if /mswin32/ =~ RUBY_PLATFORM
+    if /mswin32|mingw/ =~ RUBY_PLATFORM
       r = try_link(<<"SRC", libs)
 #include <windows.h>
 #include <winsock.h>
@@ -199,31 +214,33 @@ SRC
   return true
 end
 
-def have_func(func)
+def have_func(func, header=nil)
   printf "checking for %s()... ", func
   STDOUT.flush
 
   libs = $libs
-
-  if /mswin32/ =~ RUBY_PLATFORM
-    r = try_link(<<"SRC", libs)
+  src = 
+    if /mswin32|mingw/ =~ RUBY_PLATFORM
+      r = <<"SRC"
 #include <windows.h>
 #include <winsock.h>
+SRC
+    else
+      ""
+    end
+  unless header.nil?
+  src << <<"SRC"
+#include <#{header}>
+SRC
+  end
+  r = try_link(src + <<"SRC", libs)
 int main() { return 0; }
 int t() { #{func}(); return 0; }
 SRC
-    unless r
-      r = try_link(<<"SRC", libs)
-#include <windows.h>
-#include <winsock.h>
+  unless r
+    r = try_link(src + <<"SRC", libs)
 int main() { return 0; }
 int t() { void ((*p)()); p = (void ((*)()))#{func}; return 0; }
-SRC
-    end
-  else
-    r = try_link(<<"SRC", libs)
-int main() { return 0; }
-int t() { #{func}(); return 0; }
 SRC
   end
   unless r
@@ -296,18 +313,23 @@ def create_header()
   end
 end
 
-def dir_config(target)
-  dir = with_config("%s-dir"%target)
+def dir_config(target, idefault=nil, ldefault=nil)
+  if idefault && ldefault == nil
+    default = idefault
+    idefault = default + "/include"
+    ldefault = default + "/lib"
+  end
+  dir = with_config("%s-dir"%target, default)
   if dir
     idir = " -I"+dir+"/include"
     ldir = " -L"+dir+"/lib"
   end
   unless idir
-    dir = with_config("%s-include"%target)
+    dir = with_config("%s-include"%target, idefault)
     idir = " -I"+dir if dir
   end
   unless ldir
-    dir = with_config("%s-lib"%target)
+    dir = with_config("%s-lib"%target, ldefault)
     ldir = " -L"+dir if dir
   end
 
@@ -317,8 +339,14 @@ end
 
 def create_makefile(target)
   print "creating Makefile\n"
-  system "rm -f conftest*"
+  rm_f "conftest*"
   STDOUT.flush
+  if target.rindex(%r!/!)
+    target = $'
+    target_prefix = "/"+$`
+  else
+    target_prefix = ""
+  end
   if CONFIG["DLEXT"] == $OBJEXT
     libs = $libs.split
     for lib in libs
@@ -328,21 +356,17 @@ def create_makefile(target)
   end
   $DLDFLAGS = CONFIG["DLDFLAGS"]
 
-  if $configure_args['--enable-shared']
+  if $configure_args['--enable-shared'] or /cygwin|mingw/ == RUBY_PLATFORM
     $libs = CONFIG["LIBRUBYARG"] + " " + $libs
     $DLDFLAGS += " -L" + CONFIG["libdir"]
   end
 
-  if RUBY_PLATFORM =~ /beos/
-    $libs = $libs + " " + CONFIG["LIBRUBYARG"]
-    $DLDFLAGS = $DLDFLAGS + " -L" + CONFIG["prefix"] + "/lib"
-  end
-
   defflag = ''
-  if RUBY_PLATFORM =~ /cygwin/
-    if File.exist? target + ".def"
-      defflag = "--def=" + target + ".def"
+  if RUBY_PLATFORM =~ /cygwin|mingw/
+    open(target + '.def', 'wb') do |f|
+      f.print "EXPORTS\n", "Init_", target, "\n"
     end
+    defflag = "--def=" + target + ".def"
   end
 
   unless $objs then
@@ -352,10 +376,15 @@ def create_makefile(target)
       f.sub!(/(#{SRC_EXT.join(%q{|})})$/, $OBJEXT)
       $objs.push f
     end
+  else
+    for i in $objs
+      i.sub!(/\.o\z/, ".#{$OBJEXT}")
+    end
   end
   $objs = $objs.join(" ")
 
   mfile = open("Makefile", "w")
+  mfile.binmode if /mingw/ =~ RUBY_PLATFORM
   mfile.print  <<EOMF
 SHELL = /bin/sh
 
@@ -367,19 +396,22 @@ hdrdir = #{$hdrdir}
 
 CC = #{CONFIG["CC"]}
 
-CFLAGS   = #{CONFIG["CCDLFLAGS"]} -I$(hdrdir) #{CFLAGS} #{$CFLAGS} -I#{CONFIG["includedir"]} #{$defs.join(" ")}
+CFLAGS   = #{CONFIG["CCDLFLAGS"]} #{CFLAGS} #{$CFLAGS}
+CPPFLAGS = -I$(hdrdir) -I#{CONFIG["includedir"]} #{$defs.join(" ")} #{CONFIG["CPPFLAGS"]}
 CXXFLAGS = $(CFLAGS)
 DLDFLAGS = #{$DLDFLAGS} #{$LDFLAGS}
 LDSHARED = #{CONFIG["LDSHARED"]} #{defflag}
 
 RUBY_INSTALL_NAME = #{CONFIG["RUBY_INSTALL_NAME"]}
+RUBY_SO_NAME = #{CONFIG["RUBY_SO_NAME"]}
 
-prefix = #{CONFIG["prefix"]}
-exec_prefix = #{CONFIG["exec_prefix"]}
-libdir = #{$libdir}
-archdir = #{$archdir}
-sitelibdir = #{$sitelibdir}
-sitearchdir = #{$sitearchdir}
+DESTDIR =
+prefix = $(DESTDIR)#{CONFIG["prefix"]}
+exec_prefix = $(DESTDIR)#{CONFIG["exec_prefix"]}
+libdir = $(DESTDIR)#{$libdir}#{target_prefix}
+archdir = $(DESTDIR)#{$archdir}#{target_prefix}
+sitelibdir = $(DESTDIR)#{$sitelibdir}#{target_prefix}
+sitearchdir = $(DESTDIR)#{$sitearchdir}#{target_prefix}
 
 #### End of system configuration section. ####
 
@@ -391,17 +423,20 @@ TARGET = #{target}
 DLLIB = $(TARGET).#{CONFIG["DLEXT"]}
 
 RUBY = #{CONFIG["ruby_install_name"]}
+RM = $(RUBY) -r ftools -e 'File::rm_f(*Dir[ARGV.join(" ")])'
 
 EXEEXT = #{CONFIG["EXEEXT"]}
 
 all:		$(DLLIB)
 
-clean:;		@rm -f *.#{$OBJEXT} *.so *.sl *.a $(DLLIB)
-		@rm -f $(TARGET).lib $(TARGET).exp $(TARGET).ilk *.pdb
-		@rm -f Makefile extconf.h conftest.*
-		@rm -f core ruby$(EXEEXT) *~
+clean:;		@$(RM) *.#{$OBJEXT} *.so *.sl *.a $(DLLIB)
+		@$(RM) $(TARGET).lib $(TARGET).exp $(TARGET).ilk *.pdb
 
-realclean:	clean
+distclean:	clean
+		@$(RM) Makefile extconf.h conftest.*
+		@$(RM) core ruby$(EXEEXT) *~
+
+realclean:	distclean
 
 install:	$(archdir)/$(DLLIB)
 
@@ -422,11 +457,28 @@ EOMF
   install_rb(mfile, "$(sitelibdir)")
   mfile.printf "\n"
 
+  if /mswin32/ =~ RUBY_PLATFORM
+    mfile.print "
+.c.obj:
+	$(CC) $(CFLAGS) $(CPPFLAGS) -c -o $@ $<
+
+{$(srcdir)}.c.obj:
+	$(CC) $(CFLAGS) $(CPPFLAGS) -c -o $@ $<
+
+"
+  else
+    mfile.print "
+.c.#{$OBJEXT}:
+	$(CC) $(CFLAGS) $(CPPFLAGS) -c -o $@ $<
+"
+  end
+
   if CONFIG["DLEXT"] != $OBJEXT
-    mfile.printf <<EOMF
-$(DLLIB): $(OBJS)
-	$(LDSHARED) $(DLDFLAGS) -o $(DLLIB) $(OBJS) $(LIBS) $(LOCAL_LIBS)
-EOMF
+    mfile.print "$(DLLIB): $(OBJS)\n"
+    if /mswin32/ =~ RUBY_PLATFORM
+      mfile.print "\tset LIB=$(topdir:/=\\);$(LIB)\n"
+    end
+    mfile.print "\t$(LDSHARED) $(DLDFLAGS) -o $(DLLIB) $(OBJS) $(LIBS) $(LOCAL_LIBS)\n"
   elsif not File.exist?(target + ".c") and not File.exist?(target + ".cc")
     mfile.print "$(DLLIB): $(OBJS)\n"
     case RUBY_PLATFORM
@@ -441,36 +493,20 @@ EOMF
     dfile = open("depend", "r")
     mfile.printf "###\n"
     while line = dfile.gets()
-      mfile.printf "%s", line.gsub(/\.o/, ".#{$OBJEXT}")
+      mfile.printf "%s", line.gsub(/\.o\b/, ".#{$OBJEXT}")
     end
     dfile.close
   end
   mfile.close
-
-  if RUBY_PLATFORM =~ /beos/
-    if RUBY_PLATFORM =~ /^powerpc/ then
-      deffilename = "ruby.exp"
-    else
-      deffilename = "ruby.def"
-    end
-    print "creating #{deffilename}\n"
-    open(deffilename, "w") do |file|
-      file.print("EXPORTS\n") if RUBY_PLATFORM =~ /^i/
-      file.print("Init_#{target}\n")
-    end
-  end
 end
 
 $OBJEXT = CONFIG["OBJEXT"]
 $objs = nil
-$libs = "-lc"
+$libs = CONFIG["DLDLIBS"]
 $local_flags = ""
 case RUBY_PLATFORM
-when /cygwin|beos|openstep|nextstep|rhapsody/
-  $libs = ""
 when /mswin32/
-  $libs = ""
-  $local_flags = "rubymw.lib -link /LIBPATH:$(topdir) /EXPORT:Init_$(TARGET)"
+  $local_flags = "$(RUBY_SO_NAME).lib -link /EXPORT:Init_$(TARGET)"
 end
 $LOCAL_LIBS = ""
 $defs = []
