@@ -1,12 +1,13 @@
 =begin
 
-= net/http.rb
+= net/http.rb version 1.1.28
 
 maintained by Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 This file is derived from "http-access.rb".
 
-This library is distributed under the terms of the Ruby license.
-You can freely distribute/modify this library.
+This program is free software.
+You can distribute/modify this program under
+the terms of the Ruby Distribute License.
 
 
 = class HTTP
@@ -25,8 +26,6 @@ You can freely distribute/modify this library.
 : port
   HTTP default port, 80
 
-: command_type
-  Command class for Net::HTTP, HTTPCommand
 
 == Methods
 
@@ -34,7 +33,7 @@ You can freely distribute/modify this library.
 : start {|http| .... }
   creates a new Net::HTTP object and starts HTTP session.
 
-  When this method is called as iterator, gives HTTP object to block
+  When this method is called with a block, gives HTTP object to block
   and close HTTP session after block call finished.
 
 : get( path, header = nil, dest = '' )
@@ -47,7 +46,7 @@ You can freely distribute/modify this library.
     # example
     response, body = http.get( '/index.html' )
 
-  If called as iterator, give a part String of entity body.
+  If called with a block, give a part String of entity body.
 
   Note:
   If status is not 2xx(success), ProtocolError exception is
@@ -82,7 +81,7 @@ You can freely distribute/modify this library.
   "header" must be a Hash like { 'Accept' => '*/*', ... }.
   This method returns Net::HTTPResponse object and "dest".
 
-  If called as iterator, gives a part String of entity body.
+  If called with a block, gives a part String of entity body.
 
 : get2( path, header = nil ) {|adapter| .... }
   send GET request for "path".
@@ -116,9 +115,9 @@ You can freely distribute/modify this library.
 
     ex.
     
-    http.post2( '/index.html', 'data data data...' ) do |f|
-      f.header
-      f.body
+    http.post2( '/index.html', 'data data data...' ) do |adapter|
+      adapter.header
+      adapter.body
     end
 
 
@@ -166,7 +165,7 @@ All "key" is case-insensitive.
   entity body. A body is written to "dest" using "<<" method.
 
 : body {|str| ... }
-  get entity body by using iterator.
+  get entity body by using block.
   If this method is called twice, block is not called and
   returns first "dest".
 
@@ -214,13 +213,13 @@ module Net
   class HTTP < Protocol
 
     protocol_param :port,         '80'
-    protocol_param :command_type, '::Net::HTTPCommand'
+    protocol_param :command_type, '::Net::NetPrivate::HTTPCommand'
 
     class << self
 
       def procdest( dest, block )
         if block then
-          return ReadAdapter.new( block ), nil
+          return NetPrivate::ReadAdapter.new( block ), nil
         else
           dest ||= ''
           return dest, dest
@@ -351,11 +350,13 @@ module Net
         @socket.reopen
       end
 
-      resp = yield( u_header )
+      yield( u_header )
       if ublock then
         adapter = HTTPReadAdapter.new( @command )
         ublock.call adapter
         resp = adapter.off
+      else
+	resp = @command.get_response
       end
       
       unless keep_alive? u_header, resp then
@@ -420,43 +421,6 @@ module Net
   HTTPSession = HTTP
 
 
-  class HTTPReadAdapter
-
-    def initialize( command )
-      @command = command
-      @header = @body = nil
-    end
-
-    def inspect
-      "#<#{type}>"
-    end
-
-    def header
-      unless @header then
-        @header = @command.get_response
-      end
-      @header
-    end
-    alias response header
-
-    def body( dest = nil, &block )
-      dest, ret = HTTP.procdest( dest, block )
-      unless @body then
-        @body = @command.get_body( response, dest )
-      end
-      @body
-    end
-    alias entity body
-
-    def off
-      body
-      @command = nil
-      @header
-    end
-  
-  end
-
-
   class HTTPResponse < Response
 
     def initialize( code_type, bexist, code, msg )
@@ -470,7 +434,7 @@ module Net
     attr_accessor :body
 
     def inspect
-      "#<Net::HTTPResponse #{code}>"
+      "#<#{type.name} #{code}>"
     end
 
     def []( key )
@@ -559,6 +523,47 @@ module Net
   HTTPVersionNotSupported           = HTTPFatalErrorCode.mkchild
 
 
+  class HTTPReadAdapter
+
+    def initialize( command )
+      @command = command
+      @header = @body = nil
+    end
+
+    def inspect
+      "#<#{type}>"
+    end
+
+    def header
+      unless @header then
+        @header = @command.get_response
+      end
+      @header
+    end
+    alias response header
+
+    def body( dest = nil, &block )
+      dest, ret = HTTP.procdest( dest, block )
+      unless @body then
+        @body = @command.get_body( response, dest )
+      end
+      @body
+    end
+    alias entity body
+
+    def off
+      body
+      @command = nil
+      @header
+    end
+  
+  end
+
+
+
+  module NetPrivate
+
+
   class HTTPCommand < Command
 
     HTTPVersion = '1.1'
@@ -819,4 +824,6 @@ module Net
   end
 
 
+  end   # module Net::NetPrivate
+
 end   # module Net
