@@ -1,23 +1,13 @@
 =begin
 
-= net/pop.rb
+= net/pop.rb version 1.1.28
 
 written by Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 
-This library is distributed under the terms of the Ruby license.
-You can freely distribute/modify this library.
+This program is free software.
+You can distribute/modify this program under
+the terms of the Ruby Distribute License.
 
-=end
-
-
-require 'net/protocol'
-require 'md5'
-
-
-module Net
-
-
-=begin
 
 == Net::POP3
 
@@ -28,68 +18,39 @@ Net::Protocol
 === Class Methods
 
 : new( address = 'localhost', port = 110 )
-  This method create a new POP3 object.
-  This will not open connection yet.
+  creates a new Net::POP3 object.
+  This method does not open TCP connection yet.
 
+: start( address = 'localhost', port = 110, *protoargs )
+: start( address = 'localhost', port = 110, *protoargs ) {|pop| .... }
+  equals to Net::POP3.new( address, port ).start( *protoargs )
 
 === Methods
 
 : start( account, password )
-  This method start POP3.
+: start( account, password ) {|pop| .... }
+  starts POP3 session.
 
-: each{|popmail| ...}
+  When called with a block, give a POP3 object to block and
+  close session after block call is finished.
+
+: each {|popmail| .... }
   This method is equals to "pop3.mails.each"
 
 : mails
-  This method returns an array of ((URL:#POPMail)).
-  This array is renewed when login.
-
-=end
-
-  class POP3 < Protocol
-
-    protocol_param :port,         '110'
-    protocol_param :command_type, '::Net::POP3Command'
-
-    protocol_param :mail_type,    '::Net::POPMail'
-
-    def initialize( addr = nil, port = nil )
-      super
-      @mails = [].freeze
-    end
-
-        
-    attr :mails
-
-    def each
-      @mails.each {|m| yield m }
-    end
+  an array of ((URL:#POPMail)).
+  This array is renewed when session started.
 
 
-    private
+== Net::APOP
 
+This class defines no new methods.
+Only difference from POP3 is using APOP authentification.
 
-    def do_start( acnt, pwd )
-      @command.auth( acnt, pwd )
+=== Super Class
 
-      @mails = []
-      t = type.mail_type
-      @command.list.each_with_index do |size,idx|
-        if size then
-          @mails.push t.new( idx, size, @command )
-        end
-      end
-      @mails.freeze
-    end
+Net::POP3
 
-  end
-
-  POP         = POP3
-  POPSession  = POP3
-  POP3Session = POP3
-
-
-=begin
 
 == Net::POPMail
 
@@ -100,7 +61,7 @@ A class of mail which exists on POP server.
 Object
 
 
-=== Method
+=== Methods
 
 : all( dest = '' )
 : pop
@@ -119,7 +80,7 @@ Object
     end
 
 : all {|str| .... }
-  You can use all/pop/mail as the iterator.
+  You can use all/pop/mail with a block.
   argument 'str' is a read string (a part of mail).
 
     # usage example
@@ -148,6 +109,60 @@ Object
 
 =end
 
+require 'net/protocol'
+require 'md5'
+
+
+module Net
+
+  class POP3 < Protocol
+
+    protocol_param :port,         '110'
+    protocol_param :command_type, '::Net::NetPrivate::POP3Command'
+
+    protocol_param :mail_type,    '::Net::POPMail'
+
+    def initialize( addr = nil, port = nil )
+      super
+      @mails = [].freeze
+    end
+
+    attr :mails
+
+    def each
+      @mails.each {|m| yield m }
+    end
+
+
+    private
+
+    def do_start( acnt, pwd )
+      @command.auth( acnt, pwd )
+
+      @mails = []
+      t = type.mail_type
+      @command.list.each_with_index do |size,idx|
+        if size then
+          @mails.push t.new( idx, size, @command )
+        end
+      end
+      @mails.freeze
+    end
+
+  end
+
+  POP         = POP3
+  POPSession  = POP3
+  POP3Session = POP3
+
+
+  class APOP < POP3
+    protocol_param :command_type, 'Net::NetPrivate::APOPCommand'
+  end
+
+  APOPSession = APOP
+
+
   class POPMail
 
     def initialize( n, s, cmd )
@@ -158,12 +173,15 @@ Object
       @deleted = false
     end
 
-
     attr :size
 
+    def inspect
+      "#<#{type} #{@num}#{@deleted ? ' deleted' : ''}>"
+    end
+
     def all( dest = '' )
-      if iterator? then
-        dest = ReadAdapter.new( Proc.new )
+      if block_given? then
+        dest = NetPrivate::ReadAdapter.new( Proc.new )
       end
       @command.retr( @num, dest )
     end
@@ -195,27 +213,8 @@ Object
   end
 
 
-=begin
-
-== Net::APOP
-
-This class defines no new methods.
-Only difference from POP3 is using APOP authentification.
-
-=== Super Class
-
-Net::POP3
-
-=end
-
-  class APOP < POP3
-
-    protocol_param :command_type, 'Net::APOPCommand'
-
-  end
-
-  APOPSession = APOP
 
+  module NetPrivate
 
 
   class POP3Command < Command
@@ -227,7 +226,6 @@ Net::POP3
       }
     end
 
-
     def auth( acnt, pass )
       critical {
         @socket.writeline 'USER ' + acnt
@@ -238,7 +236,6 @@ Net::POP3
       }
     end
 
-
     def list
       arr = []
       critical {
@@ -251,7 +248,6 @@ Net::POP3
       arr
     end
 
-
     def rset
       critical {
         getok 'RSET'
@@ -266,14 +262,12 @@ Net::POP3
       }
     end
 
-
     def retr( num, dest = '', &block )
       critical {
         getok sprintf( 'RETR %d', num )
         @socket.read_pendstr( dest, &block )
       }
     end
-
     
     def dele( num )
       critical {
@@ -281,14 +275,12 @@ Net::POP3
       }
     end
 
-
     def uidl( num )
       critical {
         getok( sprintf 'UIDL %d', num ).msg.split(' ')[1]
       }
     end
 
-
     def quit
       critical {
         getok 'QUIT'
@@ -298,7 +290,6 @@ Net::POP3
 
     private
 
-
     def check_reply_auth
       begin
         cod = check_reply( SuccessCode )
@@ -309,7 +300,6 @@ Net::POP3
       return cod
     end
 
-
     def get_reply
       str = @socket.readline
 
@@ -323,7 +313,6 @@ Net::POP3
   end
 
 
-
   class APOPCommand < POP3Command
 
     def initialize( sock )
@@ -336,21 +325,17 @@ Net::POP3
       @stamp = m[0]
     end
 
-
-    def auth( acnt, pass )
+    def auth( account, pass )
       critical {
-        @socket.writeline( "APOP #{acnt} #{digest(@stamp + pass)}" )
+        @socket.writeline sprintf( 'APOP %s %s',
+                            account, MD5.new(@stamp + pass).hexdigest )
         check_reply_auth
       }
     end
 
-
-    def digest( str )
-      ret = ''
-      MD5.new( str ).digest.each_byte {|i| ret << sprintf('%02x', i) }
-      ret
-    end
-      
   end
 
-end
+
+  end   # module Net::NetPrivate
+
+end   # module Net
