@@ -1,23 +1,13 @@
 =begin
 
-= net/protocol.rb
+= net/protocol.rb version 1.1.28
 
 written by Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 
-This library is distributed under the terms of the Ruby license.
-You can freely distribute/modify this library.
+This program is free software.
+You can distribute/modify this program under
+the terms of the Ruby Distribute License.
 
-=end
-
-
-require 'socket'
-
-
-module Net
-
-  Version = '1.1.13'
-
-=begin
 
 == Net::Protocol
 
@@ -32,17 +22,15 @@ Object
 : new( address = 'localhost', port = nil )
   This method Creates a new protocol object.
 
-: start( address = 'localhost', port = nil, *args )
-: start( address = 'localhost', port = nil, *args ){|proto| .... }
-  This method creates a new Protocol object and start session.
-  If you call this method with block, Protocol object give itself
-  to block and finish session when block returns.
+: start( address = 'localhost', port = nil, *protoargs )
+: start( address = 'localhost', port = nil, *protoargs ) {|proto| .... }
+  This method creates a new Protocol object and opens a session.
+  equals to Net::Protocol.new( address, port ).start( *protoargs )
 
 : Proxy( address, port )
   This method creates a proxy class of its protocol.
   Arguments are address/port of proxy host.
 
-
 === Methods
 
 : address
@@ -52,11 +40,15 @@ Object
   connecting port number
 
 : start( *args )
-  This method start protocol. If you call this method when the protocol
-  is already started, this only returns false without doing anything.
+: start( *args ) {|proto| .... }
+  This method starts protocol. If protocol was already started,
+  do nothing and returns false.
 
   '*args' are specified in subclasses.
 
+  When is called with a block, gives Protocol object to block and
+  close session when block finished.
+
 : finish
   This method ends protocol. If you call this method before protocol starts,
   it only return false without doing anything.
@@ -66,24 +58,31 @@ Object
 
 =end
 
+require 'socket'
+
+
+module Net
+
   class Protocol
 
-    Version = ::Net::Version
+    Version = '1.1.28'
+
 
     class << self
 
       def start( address = 'localhost', port = nil, *args )
         instance = new( address, port )
 
-        if iterator? then
+        if block_given? then
           instance.start( *args ) { yield instance }
         else
-          instance.start *args
+          instance.start( *args )
           instance
         end
       end
 
-      def Proxy( p_addr, p_port )
+      def Proxy( p_addr, p_port = nil )
+        p_port ||= self.port
         klass = Class.new( self )
         klass.module_eval %-
 
@@ -112,7 +111,7 @@ Object
       def proxy?
         false
       end
-            
+
 
       private
 
@@ -139,7 +138,7 @@ Object
 
     protocol_param :port,         'nil'
     protocol_param :command_type, 'nil'
-    protocol_param :socket_type,  '::Net::Socket'
+    protocol_param :socket_type,  '::Net::NetPrivate::Socket'
 
 
     def initialize( addr = nil, port = nil )
@@ -153,32 +152,34 @@ Object
       @socket  = nil
     end
 
-
     attr_reader :address, :port,
                 :command, :socket
 
+    def inspect
+      "#<#{type} #{address}:#{port} open=#{active?}>"
+    end
+
 
     def start( *args )
       return false if active?
 
       begin
         connect
-        do_start *args
+        do_start( *args )
         @active = true
-        yield if iterator?
+        yield self if block_given?
       ensure
-        finish if iterator?
+        finish if block_given?
       end
     end
 
     def finish
-      ret = active?
+      return false unless active?
 
       do_finish
       disconnect
       @active = false
-
-      ret
+      true
     end
 
     def active?
@@ -220,17 +221,151 @@ Object
 
 
 
+
+  class Response
+
+    def initialize( ctype, cno, msg )
+      @code_type = ctype
+      @code      = cno
+      @message   = msg
+      super()
+    end
+
+    attr_reader :code_type, :code, :message
+    alias msg message
+
+    def inspect
+      "#<#{type} #{code}>"
+    end
+
+    def error!( data = nil )
+      raise code_type.error_type.new( code + ' ' + Net.quote(msg), data )
+    end
+
+  end
+
+
+  class ProtocolError          < StandardError; end
+  class ProtoSyntaxError       < ProtocolError; end
+  class ProtoFatalError        < ProtocolError; end
+  class ProtoUnknownError      < ProtocolError; end
+  class ProtoServerError       < ProtocolError; end
+  class ProtoAuthError         < ProtocolError; end
+  class ProtoCommandError      < ProtocolError; end
+  class ProtoRetriableError    < ProtocolError; end
+  ProtocRetryError = ProtoRetriableError
+
+  class ProtocolError
+  
+    def initialize( msg, data )
+      super msg
+      @data = data
+    end
+
+    attr :data
+
+    def inspect
+      "#<#{type}>"
+    end
+  
+  end
+
+
+  class Code
+
+    def initialize( paren, err )
+      @parents = paren
+      @err = err
+
+      @parents.push self
+    end
+
+    attr_reader :parents
+
+    def inspect
+      "#<#{type}>"
+    end
+
+    def error_type
+      @err
+    end
+
+    def ===( response )
+      response.code_type.parents.reverse_each {|i| return true if i == self }
+      false
+    end
+
+    def mkchild( err = nil )
+      type.new( @parents + [self], err || @err )
+    end
+  
+  end
+  
+  ReplyCode       = Code.new( [], ProtoUnknownError )
+  SuccessCode     = ReplyCode.mkchild( ProtoUnknownError )
+  ContinueCode    = ReplyCode.mkchild( ProtoUnknownError )
+  ErrorCode       = ReplyCode.mkchild( ProtocolError )
+  SyntaxErrorCode = ErrorCode.mkchild( ProtoSyntaxError )
+  FatalErrorCode  = ErrorCode.mkchild( ProtoFatalError )
+  ServerErrorCode = ErrorCode.mkchild( ProtoServerError )
+  AuthErrorCode   = ErrorCode.mkchild( ProtoAuthError )
+  RetriableCode   = ReplyCode.mkchild( ProtoRetriableError )
+  UnknownCode     = ReplyCode.mkchild( ProtoUnknownError )
+
+
+
+  module NetPrivate
+
+
+  class WriteAdapter
+
+    def initialize( sock, mid )
+      @sock = sock
+      @mid = mid
+    end
+
+    def inspect
+      "#<#{type}>"
+    end
+
+    def write( str )
+      @sock.__send__ @mid, str
+    end
+    alias << write
+  
+  end
+
+  class ReadAdapter
+
+    def initialize( block )
+      @block = block
+    end
+
+    def inspect
+      "#<#{type}>"
+    end
+
+    def <<( str )
+      @block.call str
+    end
+  
+  end
+
+
   class Command
 
     def initialize( sock )
       @socket = sock
-      @error_occured = false
       @last_reply = nil
       @critical = false
     end
 
-    attr_reader :socket, :error_occured, :last_reply
-    attr_writer :socket
+    attr_accessor :socket
+    attr_reader :last_reply
+
+    def inspect
+      "#<#{type}>"
+    end
 
     # abstract quit
 
@@ -250,9 +385,7 @@ Object
           return rep
         end
       end
-
-      @error_occured = true
-      rep.error! @socket.sending
+      rep.error!
     end
 
     def getok( line, ok = SuccessCode )
@@ -286,110 +419,6 @@ Object
   end
 
 
-  class Response
-
-    def initialize( ctype, cno, msg )
-      @code_type = ctype
-      @code      = cno
-      @message   = msg
-      super()
-    end
-
-    attr_reader :code_type, :code, :message
-    alias msg message
-
-    def error!( sending )
-      raise @code_type.error_type,
-            sprintf( <<MSG, @code, Net.quote(sending), Net.quote(@message) )
-
-status %s
-writing string is:
-%s
-
-error message from server is:
-%s
-MSG
-    end
-
-  end
-
-
-  class ProtocolError          < StandardError; end
-  class ProtoSyntaxError       < ProtocolError; end
-  class ProtoFatalError        < ProtocolError; end
-  class ProtoUnknownError      < ProtocolError; end
-  class ProtoServerError       < ProtocolError; end
-  class ProtoAuthError         < ProtocolError; end
-  class ProtoCommandError      < ProtocolError; end
-  class ProtoRetriableError    < ProtocolError; end
-  ProtocRetryError = ProtoRetriableError
-
-
-  class Code
-
-    def initialize( paren, err )
-      @parents = paren
-      @err = err
-
-      @parents.push self
-    end
-
-    attr_reader :parents
-
-    def error_type
-      @err
-    end
-
-    def ===( response )
-      response.code_type.parents.reverse_each {|i| return true if i == self }
-      false
-    end
-
-    def mkchild( err = nil )
-      type.new( @parents + [self], err || @err )
-    end
-  
-  end
-  
-  ReplyCode       = Code.new( [], ProtoUnknownError )
-  SuccessCode     = ReplyCode.mkchild( ProtoUnknownError )
-  ContinueCode    = ReplyCode.mkchild( ProtoUnknownError )
-  ErrorCode       = ReplyCode.mkchild( ProtocolError )
-  SyntaxErrorCode = ErrorCode.mkchild( ProtoSyntaxError )
-  FatalErrorCode  = ErrorCode.mkchild( ProtoFatalError )
-  ServerErrorCode = ErrorCode.mkchild( ProtoServerError )
-  RetriableCode   = ReplyCode.mkchild( ProtoRetriableError )
-  UnknownCode     = ReplyCode.mkchild( ProtoUnknownError )
-
-
-
-  class WriteAdapter
-
-    def initialize( sock, mid )
-      @sock = sock
-      @mid = mid
-    end
-
-    def write( str )
-      @sock.__send__ @mid, str
-    end
-    alias << write
-  
-  end
-
-  class ReadAdapter
-
-    def initialize( block )
-      @block = block
-    end
-
-    def <<( str )
-      @block.call str
-    end
-  
-  end
-
-
   class Socket
 
     def initialize( addr, port, pipe = nil )
@@ -414,6 +443,10 @@ MSG
       alias open new
     end
 
+    def inspect
+      "#<#{type} open=#{!@closed}>"
+    end
+
     def reopen
       unless closed? then
         @socket.close
@@ -449,9 +482,6 @@ MSG
 
 
     CRLF    = "\r\n"
-    D_CRLF  = ".\r\n"
-    TERMEXP = /\n|\r\n|\r/o
-
 
     def read( len, dest = '' )
       @pipe << "reading #{len} bytes...\n" if @pipe; pipeoff
@@ -500,7 +530,7 @@ MSG
 
         
     def readline
-      ret = readuntil( CRLF )
+      ret = readuntil( "\r\n" )
       ret.chop!
       ret
     end
@@ -511,9 +541,9 @@ MSG
 
       rsize = 0
 
-      while (str = readuntil( CRLF )) != D_CRLF do
+      while (str = readuntil( "\r\n" )) != ".\r\n" do
         rsize += str.size
-        str.gsub!( /\A\./o, '' )
+        str.gsub!( /\A\./, '' )
         dest << str
       end
 
@@ -528,10 +558,10 @@ MSG
       arr = []
       str = nil
 
-      while (str = readuntil( CRLF )) != D_CRLF do
+      while (str = readuntil( "\r\n" )) != ".\r\n" do
         str.chop!
         arr.push str
-        yield str if iterator?
+        yield str if block_given?
       end
 
       @pipe << "read #{arr.size} lines\n" if pipeon
@@ -562,47 +592,45 @@ MSG
 
 
     def write( str )
-      do_write_beg
-      do_write_do str
-      do_write_fin
+      writing {
+        do_write str
+      }
     end
 
 
     def writeline( str )
-      do_write_beg
-      do_write_do str
-      do_write_do CRLF
-      do_write_fin
+      writing {
+        do_write str
+        do_write "\r\n"
+      }
     end
 
 
     def write_bin( src, block )
-      do_write_beg
-      if block then
-        block.call WriteAdapter.new( self, :do_write_do )
-      else
-        src.each do |bin|
-          do_write_do bin
+      writing {
+        if block then
+          block.call WriteAdapter.new( self, :do_write )
+        else
+          src.each do |bin|
+            do_write bin
+          end
         end
-      end
-      do_write_fin
+      }
     end
 
 
     def write_pendstr( src, block )
       @pipe << "writing text from #{src.type}\n" if @pipe; pipeoff
 
-      do_write_beg
-      if block then
-        block.call WriteAdapter.new( self, :write_pendstr_inner )
-      else
-        write_pendstr_inner src
-      end
-      each_crlf_line2( :i_w_pend )
-      do_write_do D_CRLF
-      wsize = do_write_fin
+      wsize = use_each_crlf_line {
+        if block then
+          block.call WriteAdapter.new( self, :wpend_in )
+        else
+          wpend_in src
+        end
+      }
 
-      @pipe << "wrote #{wsize} bytes text" if pipeon
+      @pipe << "wrote #{wsize} bytes text\n" if pipeon
       wsize
     end
 
@@ -610,38 +638,56 @@ MSG
     private
 
 
-    def write_inner( src )
-      each_crlf_line( src, :do_write_do )
+    def wpend_in( src )
+      line = nil
+      pre = @writtensize
+      each_crlf_line( src ) do |line|
+        do_write '.' if line[0] == ?.
+        do_write line
+      end
+
+      @writtensize - pre
     end
 
+    def use_each_crlf_line
+      writing {
+        @wbuf = ''
 
-    def write_pendstr_inner( src )
-      each_crlf_line src, :i_w_pend
+        yield
+
+        if not @wbuf.empty? then       # un-terminated last line
+          if @wbuf[-1] == ?\r then
+            @wbuf.chop!
+          end
+          @wbuf.concat "\r\n"
+          do_write @wbuf
+        elsif @writtensize == 0 then   # empty src
+          do_write "\r\n"
+        end
+        do_write ".\r\n"
+
+        @wbuf = nil
+      }
     end
 
-    def i_w_pend( line )
-      do_write_do '.' if line[0] == ?.
-      do_write_do line
-    end
-
-
-    def each_crlf_line( src, mid )
-      beg = 0
-      buf = pos = s = bin = nil
+    def each_crlf_line( src )
+      str = m = beg = nil
 
       adding( src ) do
         beg = 0
         buf = @wbuf
-        while true do
-          pos = buf.index( TERMEXP, beg )
-          break unless pos
-          s = $&.size
-          break if pos + s == buf.size - 1 and buf[-1] == ?\r
-
-          __send__ mid, buf[ beg, pos - beg ] << CRLF
-          beg = pos + s
+        while buf.index( /\n|\r\n|\r/, beg ) do
+          m = $~
+          if m.begin(0) == buf.size - 1 and buf[-1] == ?\r then
+            # "...\r" : can follow "\n..."
+            break
+          end
+          str = buf[ beg, m.begin(0) - beg ]
+          str.concat "\r\n"
+          yield str
+          beg = m.end(0)
         end
-        @wbuf = buf[ beg, buf.size - beg ] if beg != 0
+        @wbuf = buf[ beg, buf.size - beg ]
       end
     end
 
@@ -650,50 +696,47 @@ MSG
 
       case src
       when String
-        0.step( src.size, 512 ) do |i|
-          @wbuf << src[ i, 512 ]
+        0.step( src.size - 1, 2048 ) do |i|
+          @wbuf << src[i,2048]
           yield
         end
 
       when File
         while true do
-          i = src.read( 512 )
+          i = src.read( 2048 )
           break unless i
-          @wbuf << i
+          i[0,0] = @wbuf
+          @wbuf = i
           yield
         end
 
       else
-        src.each do |bin|
-          @wbuf << bin
-          yield if @wbuf.size > 512
+        src.each do |i|
+          @wbuf << i
+          if @wbuf.size > 2048 then
+            yield
+          end
         end
       end
     end
 
-    def each_crlf_line2( mid )
-      buf = @wbuf
-      beg = pos = nil
 
-      buf << "\n" unless /\n|\r/o === buf[-1,1]
-
-      beg = 0
-      while true do
-        pos = buf.index( TERMEXP, beg )
-        break unless pos
-        __send__ mid, buf[ beg, pos - beg ] << CRLF
-        beg = pos + $&.size
-      end
-    end
-
-
-    def do_write_beg
+    def writing
       @writtensize = 0
       @sending = ''
-      @wbuf = ''
+
+      yield
+
+      if @pipe then
+        @pipe << 'write "'
+        @pipe << @sending
+        @pipe << "\"\n"
+      end
+      @socket.flush
+      @writtensize
     end
 
-    def do_write_do( arg )
+    def do_write( arg )
       if @pipe or @sending.size < 128 then
         @sending << Net.quote( arg )
       else
@@ -705,17 +748,6 @@ MSG
       s
     end
 
-    def do_write_fin
-      if @pipe then
-        @pipe << 'write "'
-        @pipe << @sending
-        @pipe << "\"\n"
-      end
-
-      @socket.flush
-      @writtensize
-    end
-
 
     def pipeoff
       @prepipe = @pipe
@@ -732,6 +764,9 @@ MSG
   end
 
 
+  end   # module Net::NetPrivate
+
+
   def Net.quote( str )
     str = str.gsub( "\n", '\\n' )
     str.gsub!( "\r", '\\r' )
