@@ -43,13 +43,12 @@ end
 if $0 == __FILE__
   a = Open3.popen3("nroff -man")
   Thread.start do
-    while gets
-      a[0].print $_
+    while line = gets
+      a[0].print line
     end
     a[0].close
   end
-  while a[1].gets
-    print ":", $_
+  while line = a[1].gets
+    print ":", line
   end
 end
-
