@@ -1,5 +1,5 @@
 #
-# $Id: tempfile.rb,v 1.2 1999/08/13 05:45:18 matz Exp $
+# $Id: tempfile.rb,v 1.5 2000/07/15 13:37:03 matz Exp $
 #
 # The class for temporary files.
 #  o creates a temporary file, which name is "basename.pid.n" with mode "w+".
@@ -10,14 +10,14 @@
 #  o file mode of the temporary files are 0600.
 
 require 'delegate'
-require 'final'
 
 class Tempfile < SimpleDelegator
   Max_try = 10
 
-  def Tempfile.callback(path)
+  def Tempfile.callback(path, data)
     lambda{
       print "removing ", path, "..." if $DEBUG
+      data[0].close if data[0]
       if File.exist?(path)
 	File.unlink(path) 
       end
@@ -47,10 +47,12 @@ class Tempfile < SimpleDelegator
 	n += 1
       end
 
-      @clean_files = Tempfile.callback(tmpname)
+      @protect = []
+      @clean_files = Tempfile.callback(tmpname, @protect)
       ObjectSpace.define_finalizer(self, @clean_files)
 
-      @tmpfile = File.open(tmpname, 'w+')
+      @tmpfile = File.open(tmpname, File::RDWR|File::CREAT|File::EXCL)
+      @protect[0] = @tmpfile
       @tmpname = tmpname
       super(@tmpfile)
       Dir.rmdir(lock)
@@ -66,12 +68,13 @@ class Tempfile < SimpleDelegator
   def open
     @tmpfile.close if @tmpfile
     @tmpfile = File.open(@tmpname, 'r+')
+    @protect[0] = @tmpfile
     __setobj__(@tmpfile)
   end
 
   def close(real=false)
     @tmpfile.close if @tmpfile
-    @tmpfile = nil
+    @protect[0] = @tmpfile = nil
     if real
       @clean_files.call
       ObjectSpace.undefine_finalizer(self)
