@@ -1,6 +1,6 @@
 #
 #		thread.rb - thread support classes
-#			$Date: 2000/05/30 04:21:35 $
+#			$Date: 2000/08/07 05:02:10 $
 #			by Yukihiro Matsumoto <matz@netlab.co.jp>
 #
 # Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
@@ -21,13 +21,13 @@ if $DEBUG
 end
 
 def Thread.exclusive
+  _old = Thread.critical
   begin
     Thread.critical = true
-    r = yield
+    return yield
   ensure
-    Thread.critical = false
+    Thread.critical = _old
   end
-  r
 end
 
 class Mutex
