@@ -1,6 +1,9 @@
 #
 # timeout.rb -- execution timeout
 #
+# Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+# Copyright (C) 2000  Information-technology Promotion Agency, Japan
+#
 #= SYNOPSIS
 #
 #   require 'timeout'
@@ -26,15 +29,16 @@ class TimeoutError<StandardError
 end
 
 def timeout(sec)
+  return yield if sec == nil
   begin
     x = Thread.current
     y = Thread.start {
       sleep sec
-      x.raise TimeoutError, "execution expired" if x.status
+      x.raise TimeoutError, "execution expired" if x.alive?
     }
     yield sec
     return true
   ensure
-    Thread.kill y if y.status
+    Thread.kill y if y.alive?
   end
 end
