@@ -1,12 +1,14 @@
-/************************************************
+/**********************************************************************
 
   main.c -
 
-  $Author: matz $
-  $Date: 2000/01/05 04:41:09 $
+  $Author: eban $
+  $Date: 2000/05/29 02:10:18 $
   created at: Fri Aug 19 13:19:58 JST 1994
 
-************************************************/
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
+
+**********************************************************************/
 
 #include "ruby.h"
 
@@ -18,6 +20,10 @@ unsigned int _stklen = 0x180000;
 int _stacksize = 262144;
 #endif
 
+#if defined __MINGW32__
+int _CRT_glob = 0;
+#endif
+
 #if defined(__MACOS__) && defined(__MWERKS__)
 #include <console.h>
 #endif
