@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   math.c -
 
   $Author: matz $
-  $Date: 2000/01/17 08:24:05 $
+  $Date: 2000/05/01 09:41:23 $
   created at: Tue Jan 25 14:12:56 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include <math.h>
@@ -22,11 +22,11 @@ VALUE rb_mMath;
 }
 
 static VALUE
-math_atan2(obj, x, y)
+math_atan2(obj, y, x)
     VALUE obj, x, y;
 {
-    Need_Float2(x, y);
-    return rb_float_new(atan2(RFLOAT(x)->value, RFLOAT(y)->value));
+    Need_Float2(y, x);
+    return rb_float_new(atan2(RFLOAT(y)->value, RFLOAT(x)->value));
 }
 
 static VALUE
