@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   node.h -
 
   $Author: matz $
-  $Date: 2000/03/23 04:14:19 $
+  $Date: 2000/09/07 06:59:36 $
   created at: Fri May 28 15:14:02 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #ifndef NODE_H
 #define NODE_H
@@ -45,10 +45,12 @@ enum node_type {
     NODE_MASGN,
     NODE_LASGN,
     NODE_DASGN,
-    NODE_DASGN_PUSH,
+    NODE_DASGN_CURR,
     NODE_GASGN,
     NODE_IASGN,
-    NODE_CASGN,
+    NODE_CDECL,
+    NODE_CVASGN2,
+    NODE_CVDECL,
     NODE_OP_ASGN1,
     NODE_OP_ASGN2,
     NODE_OP_ASGN_AND,
@@ -67,7 +69,9 @@ enum node_type {
     NODE_DVAR,
     NODE_GVAR,
     NODE_IVAR,
+    NODE_CONST,
     NODE_CVAR,
+    NODE_CVAR2,
     NODE_NTH_REF,
     NODE_BACK_REF,
     NODE_MATCH,
@@ -83,6 +87,7 @@ enum node_type {
     NODE_DREGX_ONCE,
     NODE_ARGS,
     NODE_ARGSCAT,
+    NODE_ARGSPUSH,
     NODE_RESTARGS,
     NODE_BLOCK_ARG,
     NODE_BLOCK_PASS,
@@ -96,7 +101,6 @@ enum node_type {
     NODE_SCLASS,
     NODE_COLON2,
     NODE_COLON3,
-    NODE_CNAME,
     NODE_CREF,
     NODE_DOT2,
     NODE_DOT3,
@@ -113,6 +117,8 @@ enum node_type {
 #ifdef C_ALLOCA
     NODE_ALLOCA,
 #endif
+    NODE_MEMO,
+    NODE_LAST
 };
 
 typedef struct RNode {
@@ -128,15 +134,15 @@ typedef struct RNode {
     union {
 	struct RNode *node;
 	ID id;
-	int argc;
+	long argc;
 	VALUE value;
     } u2;
     union {
 	struct RNode *node;
 	ID id;
-	int state;
+	long state;
 	struct global_entry *entry;
-	int cnt;
+	long cnt;
 	VALUE value;
     } u3;
 } NODE;
@@ -257,9 +263,11 @@ typedef struct RNode {
 #define NEW_GASGN(v,val) rb_node_newnode(NODE_GASGN,v,val,rb_global_entry(v))
 #define NEW_LASGN(v,val) rb_node_newnode(NODE_LASGN,v,val,local_cnt(v))
 #define NEW_DASGN(v,val) rb_node_newnode(NODE_DASGN,v,val,0);
-#define NEW_DASGN_PUSH(v,val) rb_node_newnode(NODE_DASGN_PUSH,v,val,0);
+#define NEW_DASGN_CURR(v,val) rb_node_newnode(NODE_DASGN_CURR,v,val,0);
 #define NEW_IASGN(v,val) rb_node_newnode(NODE_IASGN,v,val,0)
-#define NEW_CASGN(v,val) rb_node_newnode(NODE_CASGN,v,val,0)
+#define NEW_CDECL(v,val) rb_node_newnode(NODE_CDECL,v,val,0)
+#define NEW_CVASGN2(v,val) rb_node_newnode(NODE_CVASGN2,v,val,0)
+#define NEW_CVDECL(v,val) rb_node_newnode(NODE_CVDECL,v,val,0)
 #define NEW_OP_ASGN1(p,id,a) rb_node_newnode(NODE_OP_ASGN1,p,id,a)
 #define NEW_OP_ASGN2(r,i,o,val) rb_node_newnode(NODE_OP_ASGN2,r,val,NEW_OP_ASGN22(i,o))
 #define NEW_OP_ASGN22(i,o) rb_node_newnode(NODE_OP_ASGN2,i,o,rb_id_attrset(i))
@@ -269,7 +277,9 @@ typedef struct RNode {
 #define NEW_LVAR(v) rb_node_newnode(NODE_LVAR,v,0,local_cnt(v))
 #define NEW_DVAR(v) rb_node_newnode(NODE_DVAR,v,0,0);
 #define NEW_IVAR(v) rb_node_newnode(NODE_IVAR,v,0,0)
+#define NEW_CONST(v) rb_node_newnode(NODE_CONST,v,0,0)
 #define NEW_CVAR(v) rb_node_newnode(NODE_CVAR,v,0,0)
+#define NEW_CVAR2(v) rb_node_newnode(NODE_CVAR2,v,0,0)
 #define NEW_NTH_REF(n)  rb_node_newnode(NODE_NTH_REF,0,n,local_cnt('~'))
 #define NEW_BACK_REF(n) rb_node_newnode(NODE_BACK_REF,0,n,local_cnt('~'))
 #define NEW_MATCH(c) rb_node_newnode(NODE_MATCH,c,0,0)
@@ -288,6 +298,7 @@ typedef struct RNode {
 #define NEW_ZSUPER() rb_node_newnode(NODE_ZSUPER,0,0,0)
 #define NEW_ARGS(f,o,r) rb_node_newnode(NODE_ARGS,o,r,f)
 #define NEW_ARGSCAT(a,b) rb_node_newnode(NODE_ARGSCAT,a,b,0)
+#define NEW_ARGSPUSH(a,b) rb_node_newnode(NODE_ARGSPUSH,a,b,0)
 #define NEW_RESTARGS(a) rb_node_newnode(NODE_RESTARGS,a,0,0)
 #define NEW_BLOCK_ARG(v) rb_node_newnode(NODE_BLOCK_ARG,v,0,local_cnt(v))
 #define NEW_BLOCK_PASS(b) rb_node_newnode(NODE_BLOCK_PASS,0,b,0)
