@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   object.c -
 
   $Author: matz $
-  $Date: 2000/03/23 04:14:19 $
+  $Date: 2000/09/15 06:00:26 $
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "st.h"
@@ -23,12 +25,14 @@ VALUE rb_cData;
 VALUE rb_cNilClass;
 VALUE rb_cTrueClass;
 VALUE rb_cFalseClass;
+VALUE rb_cSymbol;
 
 VALUE rb_f_sprintf();
 VALUE rb_obj_alloc();
 
 static ID eq, eql;
 static ID inspect;
+static ID clone;
 
 VALUE
 rb_equal(obj1, obj2)
@@ -38,16 +42,15 @@ rb_equal(obj1, obj2)
 
     if (obj1 == obj2) return Qtrue;
     result = rb_funcall(obj1, eq, 1, obj2);
-    if (result == Qfalse || NIL_P(result))
-	return Qfalse;
-    return Qtrue;
+    if (RTEST(result)) return Qtrue;
+    return Qfalse;
 }
 
 int
 rb_eql(obj1, obj2)
     VALUE obj1, obj2;
 {
-    return rb_funcall(obj1, eql, 1, obj2) == Qtrue;
+    return RTEST(rb_funcall(obj1, eql, 1, obj2));
 }
 
 static VALUE
@@ -58,28 +61,14 @@ rb_obj_equal(obj1, obj2)
     return Qfalse;
 }
 
-static VALUE
-rb_any_to_a(obj)
-    VALUE obj;
-{
-    return rb_ary_new3(1, obj);
-}
-
-static VALUE
-rb_obj_hash(obj)
-    VALUE obj;
-{
-    return (long)obj|FIXNUM_FLAG;
-}
-
 VALUE
 rb_obj_id(obj)
     VALUE obj;
 {
-    if (rb_special_const_p(obj)) {
+    if (SPECIAL_CONST_P(obj)) {
 	return INT2NUM((long)obj);
     }
-    return (long)obj|FIXNUM_FLAG;
+    return (VALUE)((long)obj|FIXNUM_FLAG);
 }
 
 static VALUE
@@ -107,18 +96,33 @@ rb_obj_clone(obj)
     CLONESETUP(clone,obj);
     if (ROBJECT(obj)->iv_tbl) {
 	ROBJECT(clone)->iv_tbl = st_copy(ROBJECT(obj)->iv_tbl);
-	RBASIC(clone)->klass = rb_singleton_class_clone(RBASIC(obj)->klass);
-	RBASIC(clone)->flags = RBASIC(obj)->flags;
     }
 
     return clone;
 }
 
-static VALUE
+VALUE
 rb_obj_dup(obj)
     VALUE obj;
 {
-    return rb_funcall(obj, rb_intern("clone"), 0, 0);
+    VALUE dup;
+
+    dup = rb_funcall(obj, clone, 0, 0);
+    if (TYPE(dup) != TYPE(obj)) {
+	rb_raise(rb_eTypeError, "dupulicated object must be same type");
+    }
+    if (!SPECIAL_CONST_P(dup)) {
+	OBJSETUP(dup, rb_obj_type(obj), BUILTIN_TYPE(obj));
+	OBJ_INFECT(dup, obj);
+    }
+    return dup;
+}
+
+static VALUE
+rb_any_to_a(obj)
+    VALUE obj;
+{
+    return rb_ary_new3(1, obj);
 }
 
 VALUE
@@ -156,18 +160,18 @@ inspect_i(id, value, str)
     /* need not to show internal data */
     if (CLASS_OF(value) == 0) return ST_CONTINUE;
     if (!rb_is_instance_id(id)) return ST_CONTINUE;
-    if (RSTRING(str)->ptr[0] == '-') {
+    if (RSTRING(str)->ptr[0] == '-') { /* first element */
 	RSTRING(str)->ptr[0] = '#';
-	rb_str_cat(str, ": ", 2);
     }
     else {
-	rb_str_cat(str, ", ", 2);
+	rb_str_cat2(str, ", ");
     }
     ivname = rb_id2name(id);
-    rb_str_cat(str, ivname, strlen(ivname));
-    rb_str_cat(str, "=", 1);
+    rb_str_cat2(str, ivname);
+    rb_str_cat2(str, "=");
     str2 = rb_inspect(value);
-    rb_str_cat(str, RSTRING(str2)->ptr, RSTRING(str2)->len);
+    rb_str_append(str, str2);
+    OBJ_INFECT(str, str2);
 
     return ST_CONTINUE;
 }
@@ -177,7 +181,8 @@ inspect_obj(obj, str)
     VALUE obj, str;
 {
     st_foreach(ROBJECT(obj)->iv_tbl, inspect_i, str);
-    rb_str_cat(str, ">", 1);
+    rb_str_cat2(str, ">");
+    OBJ_INFECT(str, obj);
 
     return str;
 }
@@ -190,16 +195,17 @@ rb_obj_inspect(obj)
 	&& ROBJECT(obj)->iv_tbl
 	&& ROBJECT(obj)->iv_tbl->num_entries > 0) {
 	VALUE str;
-	char *b;
+	char *c, *b;
 
-	b = rb_class2name(CLASS_OF(obj));
+	c = rb_class2name(CLASS_OF(obj));
 	if (rb_inspecting_p(obj)) {
-	    char *buf = ALLOCA_N(char, strlen(b)+8);
-	    sprintf(buf, "#<%s:...>", b);
-	    return rb_str_new2(buf);
+	    b = ALLOCA_N(char, strlen(c)+8+16+1); /* 8:tags 16:addr 1:eos */
+	    sprintf(b, "#<%s:0x%lx ...>", c, obj);
+	    return rb_str_new2(b);
 	}
-	str = rb_str_new2("-<");
-	rb_str_cat(str, b, strlen(b));
+	b = ALLOCA_N(char, strlen(c)+4+16+1); 	  /* 4:tags 16:addr 1:eos */
+	sprintf(b, "-<%s:0x%lx ", c, obj);
+	str = rb_str_new2(b);
 	return rb_protect_inspect(inspect_obj, obj, str);
     }
     return rb_funcall(obj, rb_intern("to_s"), 0, 0);
@@ -209,11 +215,10 @@ VALUE
 rb_obj_is_instance_of(obj, c)
     VALUE obj, c;
 {
-    VALUE cl;
-
     switch (TYPE(c)) {
       case T_MODULE:
       case T_CLASS:
+      case T_ICLASS:
 	break;
 
       case T_NIL:
@@ -232,11 +237,7 @@ rb_obj_is_instance_of(obj, c)
 	rb_raise(rb_eTypeError, "class or module required");
     }
 
-    cl = CLASS_OF(obj);
-    while (FL_TEST(cl, FL_SINGLETON) || TYPE(cl) == T_ICLASS) {
-	cl = RCLASS(cl)->super;
-    }
-    if (c == cl) return Qtrue;
+    if (rb_obj_type(obj) == c) return Qtrue;
     return Qfalse;
 }
 
@@ -249,6 +250,7 @@ rb_obj_is_kind_of(obj, c)
     switch (TYPE(c)) {
       case T_MODULE:
       case T_CLASS:
+      case T_ICLASS:
 	break;
 
       default:
@@ -264,8 +266,7 @@ rb_obj_is_kind_of(obj, c)
 }
 
 static VALUE
-rb_obj_dummy(obj)
-    VALUE obj;
+rb_obj_dummy()
 {
     return Qnil;
 }
@@ -283,6 +284,7 @@ VALUE
 rb_obj_taint(obj)
     VALUE obj;
 {
+    rb_secure(4);
     OBJ_TAINT(obj);
     return obj;
 }
@@ -296,6 +298,25 @@ rb_obj_untaint(obj)
     return obj;
 }
 
+VALUE
+rb_obj_freeze(obj)
+    VALUE obj;
+{
+    if (rb_safe_level() >= 4 && !OBJ_TAINTED(obj))
+	rb_raise(rb_eSecurityError, "Insecure: can't freeze object");
+	
+    OBJ_FREEZE(obj);
+    return obj;
+}
+
+static VALUE
+rb_obj_frozen_p(obj)
+    VALUE obj;
+{
+    if (OBJ_FROZEN(obj)) return Qtrue;
+    return Qfalse;
+}
+
 static VALUE
 nil_to_i(obj)
     VALUE obj;
@@ -455,6 +476,42 @@ rb_obj_alloc(klass)
     return (VALUE)obj;
 }
 
+static VALUE
+sym_type(sym)
+    VALUE sym;
+{
+    return rb_cSymbol;
+}
+
+static VALUE
+sym_to_i(sym)
+    VALUE sym;
+{
+    ID id = SYM2ID(sym);
+
+    return INT2FIX(id);
+}
+
+static VALUE
+sym_inspect(sym)
+    VALUE sym;
+{
+    char *name, *buf;
+
+    name = rb_id2name(SYM2ID(sym));
+    buf = ALLOCA_N(char, strlen(name)+2);
+    sprintf(buf, ":%s", name);
+
+    return rb_str_new2(buf);
+}
+
+static VALUE
+sym_to_s(sym)
+    VALUE sym;
+{
+    return rb_str_new2(rb_id2name(SYM2ID(sym)));
+}
+
 static VALUE
 rb_mod_clone(module)
     VALUE module;
@@ -463,8 +520,6 @@ rb_mod_clone(module)
     CLONESETUP(clone, module);
 
     clone->super = RCLASS(module)->super;
-    clone->iv_tbl = 0;
-    clone->m_tbl = 0;		/* avoid GC crashing  */
     if (RCLASS(module)->iv_tbl) {
 	clone->iv_tbl = st_copy(RCLASS(module)->iv_tbl);
     }
@@ -563,6 +618,14 @@ rb_mod_cmp(mod, arg)
     return INT2FIX(1);
 }
 
+static VALUE
+rb_mod_initialize(argc, argv)
+    int argc;
+    VALUE *argv;
+{
+    return Qnil;
+}
+
 static VALUE
 rb_module_s_new(klass)
     VALUE klass;
@@ -570,6 +633,7 @@ rb_module_s_new(klass)
     VALUE mod = rb_module_new();
 
     RBASIC(mod)->klass = klass;
+    rb_obj_call_init(klass, 0, 0);
     return mod;
 }
 
@@ -591,6 +655,7 @@ rb_class_s_new(argc, argv)
     /* make metaclass */
     RBASIC(klass)->klass = rb_singleton_class_new(RBASIC(super)->klass);
     rb_singleton_class_attached(RBASIC(klass)->klass, klass);
+    rb_obj_call_init(klass, argc, argv);
     rb_funcall(super, rb_intern("inherited"), 1, klass);
 
     return klass;
@@ -600,6 +665,7 @@ static VALUE
 rb_class_s_inherited()
 {
     rb_raise(rb_eTypeError, "can't make subclass of Class");
+    return Qnil;		/* dummy */
 }
 
 static VALUE
@@ -623,12 +689,20 @@ rb_to_id(name)
 {
     ID id;
 
-    if (TYPE(name) == T_STRING) {
+    switch (TYPE(name)) {
+      case T_STRING:
 	return rb_intern(RSTRING(name)->ptr);
-    }
-    id = NUM2UINT(name);
-    if (!rb_id2name(id)) {
-	rb_raise(rb_eArgError, "%d is not a symbol", id);
+      case T_FIXNUM:
+	id = FIX2INT(name);
+	if (!rb_id2name(id)) {
+	    rb_raise(rb_eArgError, "%d is not a symbol", id);
+	}
+	break;
+      case T_SYMBOL:
+	id = SYM2ID(name);
+	break;
+      default:
+	rb_raise(rb_eTypeError, "%s is not a symbol", STR2CSTR(rb_inspect(name)));
     }
     return id;
 }
@@ -692,7 +766,7 @@ static VALUE
 rb_mod_const_get(mod, name)
     VALUE mod, name;
 {
-    return rb_const_get_at(mod, rb_to_id(name));
+    return rb_const_get(mod, rb_to_id(name));
 }
 
 static VALUE
@@ -764,6 +838,7 @@ fail_to_type(arg)
 	     arg->val == Qfalse ? "false" :
 	     rb_class2name(CLASS_OF(arg->val)), 
 	     arg->s);
+    return Qnil;		/* not reached */
 }
 
 VALUE
@@ -778,8 +853,12 @@ rb_convert_type(val, type, tname, method)
     arg1.val = arg2.val = val;
     arg1.s = method;
     arg2.s = tname;
-    val = rb_rescue(to_type, (VALUE)&arg1, fail_to_type, (VALUE)&arg2);
-    Check_Type(val, type);
+    val = rb_rescue2(to_type, (VALUE)&arg1, fail_to_type, (VALUE)&arg2,
+		     rb_eStandardError, rb_eNameError, 0);
+    if (TYPE(val) != type) {
+	rb_raise(rb_eTypeError, "%s#%s should return %s",
+		 rb_class2name(CLASS_OF(arg1.val)), method, tname);
+    }
     return val;
 }
 
@@ -801,10 +880,10 @@ rb_Integer(val)
 	return val;
 
       case T_STRING:
-	return rb_str2inum(RSTRING(val)->ptr, 0);
+	return rb_str2inum(val, 0);
 
-      case T_NIL:
-	return INT2FIX(0);
+      case T_FIXNUM:
+	return val;
 
       default:
 	break;
@@ -813,9 +892,11 @@ rb_Integer(val)
     arg1.val = arg2.val = val;
     arg1.s = "to_i";
     arg2.s = "Integer";
-    val = rb_rescue(to_type, (VALUE)&arg1, fail_to_type, (VALUE)&arg2);
+    val = rb_rescue2(to_type, (VALUE)&arg1, fail_to_type, (VALUE)&arg2,
+		     rb_eStandardError, rb_eNameError, 0);
     if (!rb_obj_is_kind_of(val, rb_cInteger)) {
-	rb_raise(rb_eTypeError, "to_i should return Integer");
+	rb_raise(rb_eTypeError, "%s#to_i should return Integer",
+		 rb_class2name(CLASS_OF(arg1.val)));
     }
     return val;
 }
@@ -827,8 +908,6 @@ rb_f_integer(obj, arg)
     return rb_Integer(arg);
 }
 
-double rb_big2dbl _((VALUE));
-
 VALUE
 rb_Float(val)
     VALUE val;
@@ -911,8 +990,16 @@ VALUE
 rb_Array(val)
     VALUE val;
 {
+    ID to_ary;
+
     if (TYPE(val) == T_ARRAY) return val;
-    val = rb_funcall(val, rb_intern("to_a"), 0);
+    to_ary = rb_intern("to_ary");
+    if (rb_respond_to(val, to_ary)) {
+	val = rb_funcall(val, to_ary, 0);
+    }
+    else {
+	val = rb_funcall(val, rb_intern("to_a"), 0);
+    }
     if (TYPE(val) != T_ARRAY) {
 	rb_raise(rb_eTypeError, "`to_a' did not return Array");
     }
@@ -960,7 +1047,7 @@ Init_Object()
 
     rb_mKernel = rb_define_module("Kernel");
     rb_include_module(rb_cObject, rb_mKernel);
-    rb_define_private_method(rb_cObject, "initialize", rb_obj_dummy, -1);
+    rb_define_private_method(rb_cObject, "initialize", rb_obj_dummy, 0);
     rb_define_private_method(rb_cClass, "inherited", rb_obj_dummy, 1);
 
     /*
@@ -995,10 +1082,11 @@ Init_Object()
 
     rb_define_method(rb_mKernel, "eql?", rb_obj_equal, 1);
 
-    rb_define_method(rb_mKernel, "hash", rb_obj_hash, 0);
+    rb_define_method(rb_mKernel, "hash", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "id", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "__id__", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "type", rb_obj_type, 0);
+    rb_define_method(rb_mKernel, "class", rb_obj_type, 0);
 
     rb_define_method(rb_mKernel, "clone", rb_obj_clone, 0);
     rb_define_method(rb_mKernel, "dup", rb_obj_dup, 0);
@@ -1006,6 +1094,8 @@ Init_Object()
     rb_define_method(rb_mKernel, "taint", rb_obj_taint, 0);
     rb_define_method(rb_mKernel, "tainted?", rb_obj_tainted, 0);
     rb_define_method(rb_mKernel, "untaint", rb_obj_untaint, 0);
+    rb_define_method(rb_mKernel, "freeze", rb_obj_freeze, 0);
+    rb_define_method(rb_mKernel, "frozen?", rb_obj_frozen_p, 0);
 
     rb_define_method(rb_mKernel, "to_a", rb_any_to_a, 0);
     rb_define_method(rb_mKernel, "to_s", rb_any_to_s, 0);
@@ -1017,7 +1107,7 @@ Init_Object()
     rb_define_method(rb_mKernel, "private_methods", rb_obj_private_methods, 0);
     rb_define_method(rb_mKernel, "instance_variables", rb_obj_instance_variables, 0);
     rb_define_private_method(rb_mKernel, "remove_instance_variable",
-			     rb_obj_remove_instance_variable, 0);
+			     rb_obj_remove_instance_variable, 1);
 
     rb_define_method(rb_mKernel, "instance_of?", rb_obj_is_instance_of, 1);
     rb_define_method(rb_mKernel, "kind_of?", rb_obj_is_kind_of, 1);
@@ -1048,6 +1138,14 @@ Init_Object()
     rb_undef_method(CLASS_OF(rb_cNilClass), "new");
     rb_define_global_const("NIL", Qnil);
 
+    rb_cSymbol = rb_define_class("Symbol", rb_cObject);
+    rb_undef_method(CLASS_OF(rb_cSymbol), "new");
+    rb_define_method(rb_cSymbol, "type", sym_type, 0);
+    rb_define_method(rb_cSymbol, "to_i", sym_to_i, 0);
+    rb_define_method(rb_cSymbol, "inspect", sym_inspect, 0);
+    rb_define_method(rb_cSymbol, "to_s", sym_to_s, 0);
+    rb_define_method(rb_cSymbol, "id2name", sym_to_s, 0);
+
     rb_define_method(rb_cModule, "===", rb_mod_eqq, 1);
     rb_define_method(rb_cModule, "<=>",  rb_mod_cmp, 1);
     rb_define_method(rb_cModule, "<",  rb_mod_lt, 1);
@@ -1066,6 +1164,7 @@ Init_Object()
     rb_define_private_method(rb_cModule, "attr_accessor", rb_mod_attr_accessor, -1);
 
     rb_define_singleton_method(rb_cModule, "new", rb_module_s_new, 0);
+    rb_define_method(rb_cModule, "initialize", rb_mod_initialize, -1);
     rb_define_method(rb_cModule, "instance_methods", rb_class_instance_methods, -1);
     rb_define_method(rb_cModule, "public_instance_methods", rb_class_instance_methods, -1);
     rb_define_method(rb_cModule, "protected_instance_methods", rb_class_protected_instance_methods, -1);
@@ -1077,6 +1176,7 @@ Init_Object()
     rb_define_method(rb_cModule, "const_defined?", rb_mod_const_defined, 1);
     rb_define_private_method(rb_cModule, "remove_const", rb_mod_remove_const, 1);
     rb_define_private_method(rb_cModule, "method_added", rb_obj_dummy, 1);
+    rb_define_method(rb_cModule, "class_variables", rb_mod_class_variables, 0);
 
     rb_define_method(rb_cClass, "new", rb_class_new_instance, -1);
     rb_define_method(rb_cClass, "superclass", rb_class_superclass, 0);
@@ -1113,4 +1213,5 @@ Init_Object()
     eq = rb_intern("==");
     eql = rb_intern("eql?");
     inspect = rb_intern("inspect");
+    clone = rb_intern("clone");
 }
