@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   pack.c -
 
   $Author: matz $
-  $Date: 2000/05/30 04:21:04 $
+  $Date: 2000/07/27 09:49:20 $
   created at: Thu Feb 10 15:17:05 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include <sys/types.h>
@@ -301,18 +301,6 @@ static void qpencode _((VALUE,VALUE,int));
 static int uv_to_utf8 _((char*,unsigned long));
 static unsigned long utf8_to_uv _((char*,int*));
 
-static void
-pack_add_ptr(str, add)
-    VALUE str, add;
-{
-#define STR_NO_ORIG FL_USER3	/* copied from string.c */
-    if (!RSTRING(str)->orig) {
-	RSTRING(str)->orig = rb_ary_new();
-	FL_SET(str, STR_NO_ORIG);
-    }
-    rb_ary_push(RSTRING(str)->orig, add);
-}
-
 static VALUE
 pack_pack(ary, fmt)
     VALUE ary, fmt;
@@ -329,7 +317,7 @@ pack_pack(ary, fmt)
     int natint;		/* native integer */
 #endif
     
-    p = str2cstr(fmt, &plen);
+    p = rb_str2cstr(fmt, &plen);
     pend = p + plen;
     res = rb_str_new(0, 0);
 
@@ -345,7 +333,7 @@ pack_pack(ary, fmt)
 #endif
 
 	if (ISSPACE(type)) continue;
-        if (*p == '_') {
+        if (*p == '_' || *p == '!') {
 	    char *natstr = "sSiIlL";
 
 	    if (strchr(natstr, type)) {
@@ -355,7 +343,7 @@ pack_pack(ary, fmt)
 		p++;
 	    }
 	    else {
-		rb_raise(rb_eArgError, "'_' allowed only after types %s", natstr);
+		rb_raise(rb_eArgError, "'%c' allowed only after types %s", *p, natstr);
 	    }
 	}
 	if (*p == '*') {	/* set data length */
@@ -379,7 +367,7 @@ pack_pack(ary, fmt)
 		plen = 0;
 	    }
 	    else {
-		ptr = str2cstr(from, &plen);
+		ptr = rb_str2cstr(from, &plen);
 	    }
 
 	    if (p[-1] == '*')
@@ -813,7 +801,7 @@ pack_pack(ary, fmt)
 
 	  case 'u':
 	  case 'm':
-	    ptr = str2cstr(NEXTFROM, &plen);
+	    ptr = rb_str2cstr(NEXTFROM, &plen);
 
 	    if (len <= 1)
 		len = 45;
@@ -849,12 +837,59 @@ pack_pack(ary, fmt)
 		if (NIL_P(from)) t = "";
 		else {
 		    t = STR2CSTR(from);
-		    pack_add_ptr(res, from);
+		    rb_str_associate(res, from);
 		}
 		rb_str_cat(res, (char*)&t, sizeof(char*));
 	    }
 	    break;
 
+	  case 'w':
+	    while (len-- > 0) {
+		unsigned long ul;
+		VALUE buf = rb_str_new(0, 0);
+		char c, *bufs, *bufe;
+
+		from = NEXTFROM;
+
+		if (TYPE(from) == T_BIGNUM) {
+		    VALUE big128 = rb_uint2big(128);
+		    while (TYPE(from) == T_BIGNUM) {
+			from = rb_big_divmod(from, big128);
+			c = NUM2INT(RARRAY(from)->ptr[1]) | 0x80; /* mod */
+			rb_str_cat(buf, &c, sizeof(char));
+			from = RARRAY(from)->ptr[0]; /* div */
+		    }
+		}
+
+		if (NIL_P(from)) ul = 0;
+		else {
+		    ul = NUM2ULONG(from);
+		}
+
+		while (ul) {
+		    c = ((ul & 0x7f) | 0x80);
+		    rb_str_cat(buf, &c, sizeof(char));
+		    ul >>=  7;
+		}
+
+		if (RSTRING(buf)->len) {
+		    bufs = RSTRING(buf)->ptr;
+		    bufe = bufs + RSTRING(buf)->len - 1;
+		    *bufs &= 0x7f; /* clear continue bit */
+		    while (bufs < bufe) { /* reverse */
+			c = *bufs;
+			*bufs++ = *bufe;
+			*bufe-- = c;
+		    }
+		    rb_str_cat(res, RSTRING(buf)->ptr, RSTRING(buf)->len);
+		}
+		else {
+		    c = 0;
+		    rb_str_cat(res, &c, sizeof(char));
+		}
+	    }
+	    break;
+
 	  default:
 	    break;
 	}
@@ -1030,9 +1065,9 @@ pack_unpack(str, fmt)
     int natint;			/* native integer */
 #endif
 
-    s = str2cstr(str, &len);
+    s = rb_str2cstr(str, &len);
     send = s + len;
-    p = str2cstr(fmt, &len);
+    p = rb_str2cstr(fmt, &len);
     pend = p + len;
 
     ary = rb_ary_new();
@@ -1042,7 +1077,7 @@ pack_unpack(str, fmt)
 #endif
 	star = 0;
 	type = *p++;
-	if (*p == '_') {
+	if (*p == '_' || *p == '!') {
 	    char *natstr = "sSiIlL";
 
 	    if (strchr(natstr, type)) {
@@ -1052,7 +1087,7 @@ pack_unpack(str, fmt)
 		p++;
 	    }
 	    else {
-		rb_raise(rb_eArgError, "'_' allowed only after types %s", natstr);
+		rb_raise(rb_eArgError, "'%c' allowed only after types %s", *p, natstr);
 	    }
 	}
 	if (p >= pend)
@@ -1577,13 +1612,45 @@ pack_unpack(str, fmt)
 		    VALUE str = rb_str_new(0, 0);
 		    memcpy(&t, s, sizeof(char *));
 		    s += sizeof(char *);
-		    if (t)
-			rb_str_cat(str, t, strlen(t));
+		    if (t) {
+			rb_str_cat2(str, t);
+		    }
 		    rb_ary_push(ary, str);
 		}
 	    }
 	    break;
 
+	  case 'w':
+	    {
+		unsigned long ul = 0;
+		unsigned long ulmask = 0xfeL << ((sizeof(unsigned long) - 1) * 8);
+
+		while (len > 0 && s < send) {
+		    ul <<= 7;
+		    ul |= (*s & 0x7f);
+		    if (!(*s++ & 0x80)) {
+			rb_ary_push(ary, rb_uint2inum(ul));
+			len--;
+			ul = 0;
+		    }
+		    else if (ul & ulmask) {
+			VALUE big = rb_uint2big(ul);
+			VALUE big128 = rb_uint2big(128);
+			while (s < send) {
+			    big = rb_big_mul(big, big128);
+			    big = rb_big_plus(big, rb_uint2big(*s & 0x7f));
+			    if (!(*s++ & 0x80)) {
+				rb_ary_push(ary, big);
+				len--;
+				ul = 0;
+				break;
+			    }
+			}
+		    }
+		}
+	    }
+	    break;
+
 	  default:
 	    break;
 	}
