@@ -1,4 +1,4 @@
-/************************************************
+/**********************************************************************
 
   re.c -
 
@@ -7,12 +7,12 @@
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "re.h"
 
-static VALUE rb_eRegxpError;
+static VALUE rb_eRegexpError;
 
 #define BEG(no) regs->beg[no]
 #define END(no) regs->end[no]
@@ -71,80 +71,77 @@ static const char casetable[] = {
 #define MIN(a,b) (((a)>(b))?(b):(a))
 
 int
-rb_str_cicmp(str1, str2)
-    VALUE str1, str2;
-{
-    int len, i;
+rb_memcmp(p1, p2, len)
     char *p1, *p2;
+    size_t len;
+{
+    int tmp;
 
-    len = MIN(RSTRING(str1)->len, RSTRING(str2)->len);
-    p1 = RSTRING(str1)->ptr; p2 = RSTRING(str2)->ptr;
-
-    for (i = 0; i < len; i++, p1++, p2++) {
-	if (casetable[(unsigned)*p1] != casetable[(unsigned)*p2])
-	    return casetable[(unsigned)*p1] - casetable[(unsigned)*p2];
+    if (!ruby_ignorecase) {
+	return memcmp(p1, p2, len);
     }
-    return RSTRING(str1)->len - RSTRING(str2)->len;
+
+    while (len--) {
+	if (tmp = casetable[(unsigned)*p1++] - casetable[(unsigned)*p2++])
+	    return tmp;
+    }
+    return 0;
 }
 
 #define REG_CASESTATE  FL_USER0
-#define REG_IGNORECASE FL_USER1
-#define REG_EXTENDED   FL_USER2
-#define REG_POSIXLINE  FL_USER3
-
 #define KCODE_NONE  0
-#define KCODE_EUC   FL_USER4
-#define KCODE_SJIS  FL_USER5
-#define KCODE_UTF8  FL_USER6
-#define KCODE_FIXED FL_USER7
+#define KCODE_EUC   FL_USER1
+#define KCODE_SJIS  FL_USER2
+#define KCODE_UTF8  FL_USER3
+#define KCODE_FIXED FL_USER4
 #define KCODE_MASK (KCODE_EUC|KCODE_SJIS|KCODE_UTF8)
 
 static int reg_kcode = DEFAULT_KCODE;
 
 static void
-kcode_euc(reg)
-    struct RRegexp *reg;
+kcode_euc(re)
+    struct RRegexp *re;
 {
-    FL_UNSET(reg, KCODE_MASK);
-    FL_SET(reg, KCODE_EUC);
-    FL_SET(reg, KCODE_FIXED);
+    FL_UNSET(re, KCODE_MASK);
+    FL_SET(re, KCODE_EUC);
+    FL_SET(re, KCODE_FIXED);
 }
 
 static void
-kcode_sjis(reg)
-    struct RRegexp *reg;
+kcode_sjis(re)
+    struct RRegexp *re;
 {
-    FL_UNSET(reg, KCODE_MASK);
-    FL_SET(reg, KCODE_SJIS);
-    FL_SET(reg, KCODE_FIXED);
+    FL_UNSET(re, KCODE_MASK);
+    FL_SET(re, KCODE_SJIS);
+    FL_SET(re, KCODE_FIXED);
 }
 
 static void
-kcode_utf8(reg)
-    struct RRegexp *reg;
+kcode_utf8(re)
+    struct RRegexp *re;
 {
-    FL_UNSET(reg, KCODE_MASK);
-    FL_SET(reg, KCODE_UTF8);
-    FL_SET(reg, KCODE_FIXED);
+    FL_UNSET(re, KCODE_MASK);
+    FL_SET(re, KCODE_UTF8);
+    FL_SET(re, KCODE_FIXED);
 }
 
 static void
-kcode_none(reg)
-    struct RRegexp *reg;
+kcode_none(re)
+    struct RRegexp *re;
 {
-    FL_UNSET(reg, KCODE_MASK);
-    FL_SET(reg, KCODE_FIXED);
+    FL_UNSET(re, KCODE_MASK);
+    FL_SET(re, KCODE_FIXED);
 }
 
 static int curr_kcode;
 
 static void
-kcode_set_option(reg)
-    VALUE reg;
+kcode_set_option(re)
+    VALUE re;
 {
-    if (!FL_TEST(reg, KCODE_FIXED)) return;
+    if (!FL_TEST(re, KCODE_FIXED)) return;
 
-    curr_kcode = RBASIC(reg)->flags & KCODE_MASK;
+    curr_kcode = RBASIC(re)->flags & KCODE_MASK;
     if (reg_kcode == curr_kcode) return;
     switch (curr_kcode) {
       case KCODE_NONE:
@@ -183,7 +180,7 @@ kcode_reset_option()
 }
 
 int
-rb_mbclen2(c, re)
+rb_reg_mbclen2(c, re)
     unsigned int c;
     VALUE re;
 {
@@ -215,17 +212,17 @@ rb_reg_expr_str(str, s, len)
     int len;
 {
     const char *p, *pend;
-    int slash = 0;
+    int need_escape = 0;
 
     p = s; pend = p + len;
     while (p<pend) {
-	if (*p == '/') {
-	    slash = 1;
+	if (*p == '/' || (!ISPRINT(*p) && !ismbchar(*p))) {
+	    need_escape = 1;
 	    break;
 	}
 	p++;
     }
-    if (!slash) {
+    if (!need_escape) {
 	rb_str_cat(str, s, len);
     }
     else {
@@ -236,9 +233,44 @@ rb_reg_expr_str(str, s, len)
 		rb_str_cat(str, &c, 1);
 		rb_str_cat(str, p, 1);
 	    }
-	    else {
+	    else if (ismbchar(*p)) {
+	    	rb_str_cat(str, p, mbclen(*p));
+		p += mbclen(*p);
+		continue;
+	    }
+	    else if (ISPRINT(*p)) {
 		rb_str_cat(str, p, 1);
 	    }
+	    else {
+		char b[8];
+		switch (*p) {
+		case '\r':
+		    rb_str_cat(str, "\\r", 2);
+		    break;
+		case '\n':
+		    rb_str_cat(str, "\\n", 2);
+		    break;
+		case '\t':
+		    rb_str_cat(str, "\\t", 2);
+		    break;
+		case '\f':
+		    rb_str_cat(str, "\\f", 2);
+		    break;
+		case 007:
+		    rb_str_cat(str, "\\a", 2);
+		    break;
+		case 013:
+		    rb_str_cat(str, "\\v", 2);
+		    break;
+		case 033:
+		    rb_str_cat(str, "\\e", 2);
+		    break;
+		default:
+		    sprintf(b, "\\%03o", *p & 0377);
+		    rb_str_cat(str, b, 4);
+		    break;
+		}
+	    }
 	    p++;
 	}
     }
@@ -251,32 +283,39 @@ rb_reg_desc(s, len, re)
     VALUE re;
 {
     VALUE str = rb_str_new2("/");
+
     rb_reg_expr_str(str, s, len);
-    rb_str_cat(str, "/", 1);
+    rb_str_cat2(str, "/");
     if (re) {
-	if (FL_TEST(re, REG_IGNORECASE))
-	    rb_str_cat(str, "i", 1);
-	if (FL_TEST(re, REG_EXTENDED))
-	    rb_str_cat(str, "x", 1);
-	if (FL_TEST(re, REG_POSIXLINE))
-	    rb_str_cat(str, "p", 1);
+	rb_reg_check(re);
+	if (RREGEXP(re)->ptr->options & RE_OPTION_MULTILINE)
+	    rb_str_cat2(str, "m");
+	/* /p is obsolete; to be removed */
+	if ((RREGEXP(re)->ptr->options & RE_OPTION_POSIXLINE) == RE_OPTION_POSIXLINE)
+	    rb_str_cat2(str, "p");
+	if (RREGEXP(re)->ptr->options & RE_OPTION_IGNORECASE)
+	    rb_str_cat2(str, "i");
+	if (RREGEXP(re)->ptr->options & RE_OPTION_EXTENDED)
+	    rb_str_cat2(str, "x");
+	
 	if (FL_TEST(re, KCODE_FIXED)) {
 	    switch ((RBASIC(re)->flags & KCODE_MASK)) {
 	      case KCODE_NONE:
-		rb_str_cat(str, "n", 1);
+		rb_str_cat2(str, "n");
 		break;
 	      case KCODE_EUC:
-		rb_str_cat(str, "e", 1);
+		rb_str_cat2(str, "e");
 		break;
 	      case KCODE_SJIS:
-		rb_str_cat(str, "s", 1);
+		rb_str_cat2(str, "s");
 		break;
 	      case KCODE_UTF8:
-		rb_str_cat(str, "u", 1);
+		rb_str_cat2(str, "u");
 		break;
 	    }
 	}
     }
+    OBJ_INFECT(str, re);
     return str;
 }
 
@@ -284,7 +323,10 @@ static VALUE
 rb_reg_source(re)
     VALUE re;
 {
-    VALUE str = rb_str_new(RREGEXP(re)->str,RREGEXP(re)->len);
+    VALUE str;
+
+    rb_reg_check(re);
+    str = rb_str_new(RREGEXP(re)->str,RREGEXP(re)->len);
     if (OBJ_TAINTED(re)) OBJ_TAINT(str);
     return str;
 }
@@ -293,6 +335,7 @@ static VALUE
 rb_reg_inspect(re)
     VALUE re;
 {
+    rb_reg_check(re);
     return rb_reg_desc(RREGEXP(re)->str, RREGEXP(re)->len, re);
 }
 
@@ -308,22 +351,23 @@ rb_reg_raise(s, len, err, re)
     if (ruby_in_compile)
 	rb_compile_error("%s: %s", err, RSTRING(desc)->ptr);
     else
-	rb_raise(rb_eRegxpError, "%s: %s", err, RSTRING(desc)->ptr);
+	rb_raise(rb_eRegexpError, "%s: %s", err, RSTRING(desc)->ptr);
 }
 
 static VALUE
 rb_reg_casefold_p(re)
     VALUE re;
 {
-    if (FL_TEST(re, REG_IGNORECASE)) return Qtrue;
+    rb_reg_check(re);
+    if (RREGEXP(re)->ptr->options & RE_OPTION_IGNORECASE) return Qtrue;
     return Qfalse;
 }
 
 static VALUE
-rb_reg_kcode_method(re)
+rb_reg_kcode_m(re)
     VALUE re;
 {
-    char *kcode = "$KCODE";
+    char *kcode;
 
     if (FL_TEST(re, KCODE_FIXED)) {
 	switch (RBASIC(re)->flags & KCODE_MASK) {
@@ -336,11 +380,12 @@ rb_reg_kcode_method(re)
 	  case KCODE_UTF8:
 	    kcode = "utf8"; break;
 	  default:
+	    rb_bug("unknow kcode - should not happen");
 	    break;
 	}
+	return rb_str_new2(kcode);
     }
-
-    return rb_str_new2(kcode);
+    return Qnil;
 }
 
 static Regexp*
@@ -391,21 +436,20 @@ match_alloc()
 }
 
 static VALUE
-match_clone(orig)
-    VALUE orig;
+match_clone(match)
+    VALUE match;
 {
-    NEWOBJ(match, struct RMatch);
-    OBJSETUP(match, rb_cMatch, T_MATCH);
+    NEWOBJ(clone, struct RMatch);
+    CLONESETUP(clone, match);
 
-    match->str = RMATCH(orig)->str;
-    match->regs = 0;
+    clone->str = RMATCH(match)->str;
+    clone->regs = 0;
 
-    match->regs = ALLOC(struct re_registers);
-    match->regs->allocated = 0;
-    re_copy_registers(match->regs, RMATCH(orig)->regs);
-    CLONESETUP(match, orig);
+    clone->regs = ALLOC(struct re_registers);
+    clone->regs->allocated = 0;
+    re_copy_registers(clone->regs, RMATCH(match)->regs);
 
-    return (VALUE)match;
+    return (VALUE)clone;
 }
 
 static VALUE
@@ -464,28 +508,20 @@ match_end(match, n)
 #define MATCH_BUSY FL_USER2
 
 void
-rb_match_busy(match, busy)
+rb_match_busy(match)
     VALUE match;
-    int busy;
 {
-    if (busy) {
-	FL_SET(match, MATCH_BUSY);
-    }
-    else {
-	FL_UNSET(match, MATCH_BUSY);
-    }
+    FL_SET(match, MATCH_BUSY);
 }
 
 int ruby_ignorecase;
 static int may_need_recompile;
-static VALUE matchcache;
 
 static void
 rb_reg_prepare_re(re)
     VALUE re;
 {
     int need_recompile = 0;
-
     int state;
 
     rb_reg_check(re);
@@ -514,6 +550,7 @@ rb_reg_prepare_re(re)
 
 	if (FL_TEST(re, KCODE_FIXED))
 	    kcode_set_option(re);
+	rb_reg_check(re);
 	RREGEXP(re)->ptr->fastmap_accurate = 0;
 	err = re_compile_pattern(RREGEXP(re)->str, RREGEXP(re)->len, RREGEXP(re)->ptr);
 	if (err != NULL) {
@@ -523,25 +560,73 @@ rb_reg_prepare_re(re)
 }
 
 int
-rb_reg_search(reg, str, pos, reverse)
-    VALUE reg, str;
+rb_reg_adjust_startpos(re, str, pos, reverse)
+    VALUE re, str;
+    int pos, reverse;
+{
+    int range;
+
+    rb_reg_check(re);
+    if (may_need_recompile) rb_reg_prepare_re(re);
+
+    if (FL_TEST(re, KCODE_FIXED))
+	kcode_set_option(re);
+    else if (reg_kcode != curr_kcode)
+	kcode_reset_option();
+
+    if (reverse) {
+	range = -pos;
+    }
+    else {
+	range = RSTRING(str)->len - pos;
+    }
+    return re_adjust_startpos(RREGEXP(re)->ptr,
+			      RSTRING(str)->ptr, RSTRING(str)->len,
+			      pos, range);
+}
+
+int
+rb_reg_search(re, str, pos, reverse)
+    VALUE re, str;
     int pos, reverse;
 {
     int result;
     VALUE match;
-    struct re_registers *regs = 0;
+    static struct re_registers regs;
     int range;
 
     if (pos > RSTRING(str)->len) return -1;
 
-    if (may_need_recompile)
-	rb_reg_prepare_re(reg);
+    rb_reg_check(re);
+    if (may_need_recompile) rb_reg_prepare_re(re);
 
-    if (FL_TEST(reg, KCODE_FIXED))
-	kcode_set_option(reg);
+    if (FL_TEST(re, KCODE_FIXED))
+	kcode_set_option(re);
     else if (reg_kcode != curr_kcode)
 	kcode_reset_option();
 
+    if (reverse) {
+	range = -pos;
+    }
+    else {
+	range = RSTRING(str)->len - pos;
+    }
+    result = re_search(RREGEXP(re)->ptr,RSTRING(str)->ptr,RSTRING(str)->len,
+		       pos, range, &regs);
+
+    if (FL_TEST(re, KCODE_FIXED))
+	kcode_reset_option();
+
+    if (result == -2) {
+	rb_reg_raise(RREGEXP(re)->str, RREGEXP(re)->len,
+		  "Stack overflow in regexp matcher", re);
+    }
+
+    if (result < 0) {
+	rb_backref_set(Qnil);
+	return result;
+    }
+
     if (rb_thread_scope_shared_p()) {
 	match = Qnil;
     }
@@ -549,41 +634,14 @@ rb_reg_search(reg, str, pos, reverse)
 	match = rb_backref_get();
     }
     if (NIL_P(match) || FL_TEST(match, MATCH_BUSY)) {
-	if (matchcache) {
-	    match = matchcache;
-	    matchcache = 0;
-	}
-	else {
-	    match = match_alloc();
-	}
-    }
-    regs = RMATCH(match)->regs;
-
-    if (reverse) {
-	range = -pos;
-    }
-    else {
-	range = RSTRING(str)->len - pos;
-    }
-    result = re_search(RREGEXP(reg)->ptr,RSTRING(str)->ptr,RSTRING(str)->len,
-		       pos, range, regs);
-
-    if (FL_TEST(reg, KCODE_FIXED))
-	kcode_reset_option();
-
-    if (result == -2) {
-	rb_reg_raise(RREGEXP(reg)->str, RREGEXP(reg)->len,
-		  "Stack overflow in regexp matcher", reg);
-    }
-    if (result < 0) {
-	matchcache = match;
-	rb_backref_set(Qnil);
-    }
-    else {
-	RMATCH(match)->str = rb_str_new4(str);
-	rb_backref_set(match);
+	match = match_alloc();
     }
+    re_copy_registers(RMATCH(match)->regs, &regs);
+    RMATCH(match)->str = rb_str_new4(str);
+    rb_backref_set(match);
 
+    OBJ_INFECT(match, re);
+    OBJ_INFECT(match, str);
     return result;
 }
 
@@ -605,6 +663,7 @@ rb_reg_nth_match(nth, match)
     int nth;
     VALUE match;
 {
+    VALUE str;
     int start, end, len;
 
     if (NIL_P(match)) return Qnil;
@@ -615,7 +674,9 @@ rb_reg_nth_match(nth, match)
     if (start == -1) return Qnil;
     end = RMATCH(match)->END(nth);
     len = end - start;
-    return rb_str_new(RSTRING(RMATCH(match)->str)->ptr + start, len);
+    str = rb_str_new(RSTRING(RMATCH(match)->str)->ptr + start, len);
+    if (OBJ_TAINTED(match)) OBJ_TAINT(str);
+    return str;
 }
 
 VALUE
@@ -716,14 +777,7 @@ match_aref(argc, argv, match)
     if (!NIL_P(rest) || !FIXNUM_P(idx) || FIX2INT(idx) < 0) {
 	return rb_ary_aref(argc, argv, match_to_a(match));
     }
-
-    regs = RMATCH(match)->regs;
-    i = FIX2INT(idx);
-
-    if (i >= regs->num_regs) return Qnil;
-
-    ptr = RSTRING(RMATCH(match)->str)->ptr;
-    return rb_str_new(ptr+regs->beg[i], regs->end[i]-regs->beg[i]);
+    return rb_reg_nth_match(FIX2INT(idx), match);
 }
 
 static VALUE
@@ -732,7 +786,9 @@ match_to_s(match)
 {
     VALUE str = rb_reg_last_match(match);
 
-    if (NIL_P(str)) return rb_str_new(0,0);
+    if (NIL_P(str)) str = rb_str_new(0,0);
+    if (OBJ_TAINTED(match)) OBJ_TAINT(str);
+    if (OBJ_TAINTED(RMATCH(match)->str)) OBJ_TAINT(str);
     return str;
 }
 
@@ -745,69 +801,60 @@ match_string(match)
 
 VALUE rb_cRegexp;
 
-static VALUE
-rb_reg_new_1(klass, s, len, options)
-    VALUE klass;
+static void
+rb_reg_initialize(obj, s, len, options)
+    VALUE obj;
     const char *s;
     int len;
     int options;		/* CASEFOLD  = 1 */
 				/* EXTENDED  = 2 */
-				/* POSIXLINE = 4 */
-				/* CODE_NONE = 8 */
-				/* CODE_EUC  = 16 */
-				/* CODE_SJIS = 24 */
-				/* CODE_UTF8 = 32 */
+				/* MULTILINE = 4 */
+				/* CODE_NONE = 16 */
+				/* CODE_EUC  = 32 */
+				/* CODE_SJIS = 48 */
+				/* CODE_UTF8 = 64 */
 {
-    NEWOBJ(re, struct RRegexp);
-    OBJSETUP(re, klass, T_REGEXP);
+    struct RRegexp *re = RREGEXP(obj);
+
+    if (re->ptr) re_free_pattern(re->ptr);
+    if (re->str) free(re->str);
     re->ptr = 0;
     re->str = 0;
 
-    if (options & RE_OPTION_IGNORECASE) {
-	FL_SET(re, REG_IGNORECASE);
-    }
-    if (options & RE_OPTION_EXTENDED) {
-	FL_SET(re, REG_EXTENDED);
-    }
-    if (options & RE_OPTION_POSIXLINE) {
-	FL_SET(re, REG_POSIXLINE);
-    }
-    switch (options & ~0x7) {
+    switch (options & ~0xf) {
       case 0:
       default:
 	FL_SET(re, reg_kcode);
 	break;
-      case 8:
+      case 16:
 	kcode_none(re);
 	break;
-      case 16:
+      case 32:
 	kcode_euc(re);
 	break;
-      case 24:
+      case 48:
 	kcode_sjis(re);
 	break;
-      case 32:
+      case 64:
 	kcode_utf8(re);
 	break;
     }
 
-    if (options & ~0x7) {
+    if (options & ~0xf) {
 	kcode_set_option((VALUE)re);
     }
     if (ruby_ignorecase) {
 	options |= RE_OPTION_IGNORECASE;
 	FL_SET(re, REG_CASESTATE);
     }
-    re->ptr = make_regexp(s, len, options & 0x7);
+    re->ptr = make_regexp(s, len, options & 0xf);
     re->str = ALLOC_N(char, len+1);
     memcpy(re->str, s, len);
     re->str[len] = '\0';
     re->len = len;
-    if (options & ~0x7) {
+    if (options & ~0xf) {
 	kcode_reset_option();
     }
-
-    return (VALUE)re;
 }
 
 VALUE
@@ -816,7 +863,12 @@ rb_reg_new(s, len, options)
     long len;
     int options;
 {
-    return rb_reg_new_1(rb_cRegexp, s, len, options);
+    NEWOBJ(re, struct RRegexp);
+    OBJSETUP(re, rb_cRegexp, T_REGEXP);
+
+    re->ptr = 0; re->len = 0; re->str = 0;
+    rb_reg_initialize(re, s, len, options);
+    return (VALUE)re;
 }
 
 static int case_cache;
@@ -857,12 +909,14 @@ rb_reg_equal(re1, re2)
 
     if (re1 == re2) return Qtrue;
     if (TYPE(re2) != T_REGEXP) return Qfalse;
+    rb_reg_check(re1); rb_reg_check(re2);
     if (RREGEXP(re1)->len != RREGEXP(re2)->len) return Qfalse;
     min = RREGEXP(re1)->len;
     if (min > RREGEXP(re2)->len) min = RREGEXP(re2)->len;
     if (memcmp(RREGEXP(re1)->str, RREGEXP(re2)->str, min) == 0 &&
 	rb_reg_cur_kcode(re1) == rb_reg_cur_kcode(re2) &&
-	!(FL_TEST(re1,REG_IGNORECASE) ^ FL_TEST(re2,REG_IGNORECASE))) {
+	!((RREGEXP(re1)->ptr->options & RE_OPTION_IGNORECASE) ^
+	  (RREGEXP(re2)->ptr->options & RE_OPTION_IGNORECASE))) {
 	return Qtrue;
     }
     return Qfalse;
@@ -901,19 +955,19 @@ rb_reg_match2(re)
 }
 
 static VALUE
-rb_reg_match_method(re, str)
+rb_reg_match_m(re, str)
     VALUE re, str;
 {
     VALUE result = rb_reg_match(re, str);
 
     if (NIL_P(result)) return Qnil;
     result = rb_backref_get();
-    rb_match_busy(result, Qtrue);
+    rb_match_busy(result);
     return result;
 }
 
 static VALUE
-rb_reg_s_new(argc, argv, self)
+rb_reg_initialize_m(argc, argv, self)
     int argc;
     VALUE *argv;
     VALUE self;
@@ -933,16 +987,16 @@ rb_reg_s_new(argc, argv, self)
 
 	switch (kcode[0]) {
 	  case 'n': case 'N':
-	    flag |= 8;
-	    break;
-	  case 'e': case 'E':
 	    flag |= 16;
 	    break;
+	  case 'e': case 'E':
+	    flag |= 32;
+	    break;
 	  case 's': case 'S':
-	    flag |= 24;
+	    flag |= 48;
 	    break;
 	  case 'u': case 'U':
-	    flag |= 32;
+	    flag |= 64;
 	    break;
 	  default:
 	    break;
@@ -951,17 +1005,31 @@ rb_reg_s_new(argc, argv, self)
 
     src = argv[0];
     if (TYPE(src) == T_REGEXP) {
-	return rb_reg_new_1(self, RREGEXP(src)->str, RREGEXP(src)->len, flag);
+	rb_reg_check(src);
+	rb_reg_initialize(self, RREGEXP(src)->str, RREGEXP(src)->len, flag);
     }
     else {
 	char *p;
 	int len;
 
-	p = str2cstr(src, &len);
-	return rb_reg_new_1(self, p, len, flag);
+	p = rb_str2cstr(src, &len);
+	rb_reg_initialize(self, p, len, flag);
     }
 }
 
+static VALUE
+rb_reg_s_new(argc, argv, klass)
+    int argc;
+    VALUE *argv;
+    VALUE klass;
+{
+    NEWOBJ(re, struct RRegexp);
+    OBJSETUP(re, klass, T_REGEXP);
+    re->ptr = 0; re->len = 0; re->str = 0;
+    rb_obj_call_init((VALUE)re, argc, argv);
+    return (VALUE)re;
+}
+
 static VALUE
 rb_reg_s_quote(argc, argv)
     int argc;
@@ -979,7 +1047,7 @@ rb_reg_s_quote(argc, argv)
 	curr_kcode = reg_kcode;
 	reg_kcode = kcode_saved;
     }
-    s = str2cstr(str, &len);
+    s = rb_str2cstr(str, &len);
     send = s + len;
     tmp = ALLOCA_N(char, len*2);
     t = tmp;
@@ -1029,22 +1097,20 @@ static int
 rb_reg_get_kcode(re)
     VALUE re;
 {
-    int kcode = 0;
+    int kcode;
 
     switch (RBASIC(re)->flags & KCODE_MASK) {
       case KCODE_NONE:
-	kcode |= 4; break;
+	return 16;
       case KCODE_EUC:
-	kcode |= 8; break;
+	return 32;
       case KCODE_SJIS:
-	kcode |= 12; break;
+	return 48;
       case KCODE_UTF8:
-	kcode |= 16; break;
+	return 64;
       default:
-	break;
+	return 0;
     }
-
-    return kcode;
 }
 
 int
@@ -1053,7 +1119,8 @@ rb_reg_options(re)
 {
     int options = 0;
 
-    if (FL_TEST(re, REG_IGNORECASE)) 
+    rb_reg_check(re);
+    if (RREGEXP(re)->ptr->options & RE_OPTION_IGNORECASE)
 	options |= RE_OPTION_IGNORECASE;
     if (FL_TEST(re, KCODE_FIXED)) {
 	options |= rb_reg_get_kcode(re);
@@ -1062,15 +1129,16 @@ rb_reg_options(re)
 }
 
 static VALUE
-rb_reg_clone(orig)
-    VALUE orig;
+rb_reg_clone(re)
+    VALUE re;
 {
-    VALUE reg;
-    
-    reg = rb_reg_new_1(CLASS_OF(orig), RREGEXP(orig)->str, RREGEXP(orig)->len,
-		       rb_reg_options(orig));
-    CLONESETUP(reg, orig);
-    return reg;
+    NEWOBJ(clone, struct RRegexp);
+    CLONESETUP(clone, re);
+    rb_reg_check(re);
+    clone->ptr = 0; clone->len = 0; clone->str = 0;
+    rb_reg_initialize(clone, RREGEXP(re)->str, RREGEXP(re)->len,
+		      rb_reg_options(re));
+    return (VALUE)re;
 }
 
 VALUE
@@ -1133,6 +1201,7 @@ rb_reg_regsub(str, src, regs)
 	}
 
 	if (no >= 0) {
+	    if (no >= regs->num_regs) continue;
 	    if (BEG(no) == -1) continue;
 	    rb_str_cat(val, RSTRING(src)->ptr+BEG(no), END(no)-BEG(no));
 	}
@@ -1230,7 +1299,8 @@ match_getter()
     VALUE match = rb_backref_get();
 
     if (NIL_P(match)) return Qnil;
-    return match_clone(match);
+    rb_match_busy(match);
+    return match;
 }
 
 static void
@@ -1244,7 +1314,7 @@ match_setter(val)
 void
 Init_Regexp()
 {
-    rb_eRegxpError = rb_define_class("RegxpError", rb_eStandardError);
+    rb_eRegexpError = rb_define_class("RegexpError", rb_eStandardError);
 
     re_set_casetable(casetable);
 #if DEFAULT_KCODE == KCODE_EUC
@@ -1276,26 +1346,28 @@ Init_Regexp()
     rb_define_singleton_method(rb_cRegexp, "compile", rb_reg_s_new, -1);
     rb_define_singleton_method(rb_cRegexp, "quote", rb_reg_s_quote, -1);
     rb_define_singleton_method(rb_cRegexp, "escape", rb_reg_s_quote, -1);
+    rb_define_singleton_method(rb_cRegexp, "last_match", match_getter, 0);
 
+    rb_define_method(rb_cRegexp, "initialize", rb_reg_initialize_m, -1);
     rb_define_method(rb_cRegexp, "clone", rb_reg_clone, 0);
     rb_define_method(rb_cRegexp, "==", rb_reg_equal, 1);
     rb_define_method(rb_cRegexp, "=~", rb_reg_match, 1);
     rb_define_method(rb_cRegexp, "===", rb_reg_match, 1);
     rb_define_method(rb_cRegexp, "~", rb_reg_match2, 0);
-    rb_define_method(rb_cRegexp, "match", rb_reg_match_method, 1);
+    rb_define_method(rb_cRegexp, "match", rb_reg_match_m, 1);
     rb_define_method(rb_cRegexp, "inspect", rb_reg_inspect, 0);
     rb_define_method(rb_cRegexp, "source", rb_reg_source, 0);
     rb_define_method(rb_cRegexp, "casefold?", rb_reg_casefold_p, 0);
-    rb_define_method(rb_cRegexp, "kcode", rb_reg_kcode_method, 0);
+    rb_define_method(rb_cRegexp, "kcode", rb_reg_kcode_m, 0);
 
     rb_define_const(rb_cRegexp, "IGNORECASE", INT2FIX(RE_OPTION_IGNORECASE));
     rb_define_const(rb_cRegexp, "EXTENDED", INT2FIX(RE_OPTION_EXTENDED));
-    rb_define_const(rb_cRegexp, "POSIXLINE", INT2FIX(RE_OPTION_POSIXLINE));
+    rb_define_const(rb_cRegexp, "MULTILINE", INT2FIX(RE_OPTION_MULTILINE));
 
     rb_global_variable(&reg_cache);
-    rb_global_variable(&matchcache);
 
-    rb_cMatch  = rb_define_class("MatchingData", rb_cObject);
+    rb_cMatch  = rb_define_class("MatchData", rb_cObject);
+    rb_define_global_const("MatchingData", rb_cMatch);
     rb_undef_method(CLASS_OF(rb_cMatch), "new");
 
     rb_define_method(rb_cMatch, "clone", match_clone, 0);
