@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   ruby.c -
 
-  $Author: eban $
-  $Date: 2000/07/29 14:05:15 $
+  $Author: matz $
+  $Date: 2000/08/15 06:22:42 $
   created at: Tue Aug 10 12:47:31 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #ifdef _WIN32
 #include <windows.h>
@@ -19,7 +21,6 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <ctype.h>
-#include <fcntl.h>
 
 #ifdef __hpux
 #include <sys/pstat.h>
@@ -29,10 +30,6 @@
 #include <unistd.h>
 #endif
 
-#ifdef USE_CWGUSI
-#include "macruby_missing.h"
-#endif
-
 #ifndef HAVE_STRING_H
 char *strchr _((const char*,const char));
 char *strrchr _((const char*,const char));
@@ -45,19 +42,13 @@ char *strstr _((const char*,const char*));
 char *getenv();
 #endif
 
-static int version, copyright;
-
 VALUE ruby_debug = Qfalse;
 VALUE ruby_verbose = Qfalse;
-static int sflag = Qfalse;
+static int sflag = 0;
+static int xflag = 0;
+extern int yydebug;
 
 char *ruby_inplace_mode = Qfalse;
-# ifndef strdup
-char *strdup();
-# endif
-
-extern int yydebug;
-static int xflag = Qfalse;
 
 static void load_stdin _((void));
 static void load_file _((char *, int));
@@ -83,6 +74,7 @@ usage(name)
 "-0[octal]       specify record separator (\\0, if no argument)",
 "-a              autosplit mode with -n or -p (splits $_ into $F)",
 "-c              check syntax only",
+"-Cdirectory     cd to directory, before executing your script",
 "-d              set debugging flags (set $DEBUG to true)",
 "-e 'command'    one line of script. Several -e's allowed. Omit [programfile]",
 "-Fpattern       split() pattern for autosplit (-a)",
@@ -90,33 +82,28 @@ usage(name)
 "-Idirectory     specify $LOAD_PATH directory (may be used more than once)",
 "-Kkcode         specifies KANJI (Japanese) code-set",
 "-l              enable line ending processing",
-"-n              assume 'while gets; ...; end' loop around your script",
+"-n              assume 'while gets(); ... end' loop around your script",
 "-p              assume loop like -n but print line also like sed",
 "-rlibrary       require the library, before executing your script",
 "-s              enable some switch parsing for switches after script name",
 "-S              look for the script using PATH environment variable",
 "-T[level]       turn on tainting checks",
-"-v              enables verbose mode",
-"-w              turn warnings on for compilation of your script",
+"-v              print version number, then turn on verbose mode",
+"-w              turn warnings on for your script",
 "-x[directory]   strip off text before #!ruby line and perhaps cd to directory",
-"-Xdirectory     cd to directory, before executing your script",
 "--copyright     print the copyright",
 "--version       print the version",
-"\n",
 NULL
 };
     char **p = usage_msg;
 
-    printf("\nUsage: %s [switches] [--] [programfile] [arguments]", name);
+    printf("Usage: %s [switches] [--] [programfile] [arguments]\n", name);
     while (*p)
-	printf("\n  %s", *p++);
+	printf("  %s\n", *p++);
 }
 
 extern VALUE rb_load_path;
 
-static FILE *e_fp;
-static char *e_tmpname;
-
 #define STATIC_FILE_LENGTH 255
 
 #if defined(_WIN32) || defined(DJGPP)
@@ -178,8 +165,8 @@ rubylib_mangle(s, l)
 #define rubylib_mangled_path2(s) rb_str_new2(s)
 #endif
 
-static void
-addpath(path)
+void
+ruby_incpush(path)
     const char *path;
 {
     const char sep = PATH_SEP_CHAR;
@@ -208,15 +195,82 @@ addpath(path)
 		break;
 	    }
 	}
-	rb_load_path = rb_ary_plus(ary, rb_load_path);
+	rb_ary_concat(rb_load_path, ary);
     }
     else {
-	rb_ary_unshift(rb_load_path, rubylib_mangled_path2(path));
+	rb_ary_push(rb_load_path, rubylib_mangled_path2(path));
+    }
+}
+
+void
+ruby_init_loadpath()
+{
+#if defined(_WIN32) || defined(DJGPP)
+    char libpath[FILENAME_MAX+1];
+    char *p;
+    size_t rest;
+#if defined(_WIN32)
+    GetModuleFileName(NULL, libpath, sizeof libpath);
+#elif defined(DJGPP)
+    extern char *__dos_argv0;
+    strncpy(libpath, __dos_argv0, FILENAME_MAX);
+#endif
+    p = strrchr(libpath, '\\');
+    if (p) {
+	*p = 0;
+	if (p-libpath > 3 && !strcasecmp(p-4, "\\bin")) {
+	    p -= 4;
+	    *p = 0;
+	}
+    } else {
+	strcpy(libpath, ".");
+	p = libpath + 1;
+    }
+
+#if !defined(__CYGWIN32__)
+#ifndef CharNext		/* defined as CharNext[AW] on Windows. */
+#define CharNext(p) ((p) + 1)
+#endif
+    for (p = libpath; *p; p = CharNext(p))
+	if (*p == '\\')
+	    *p = '/';
+#endif
+    rest = FILENAME_MAX - (p - libpath);
+
+#define RUBY_RELATIVE(path) (strncpy(p, (path), rest), libpath)
+#else
+#define RUBY_RELATIVE(path) (path)
+#endif
+
+    if (rb_safe_level() == 0) {
+	ruby_incpush(getenv("RUBYLIB"));
+    }
+
+#ifdef RUBY_SEARCH_PATH
+    ruby_incpush(RUBY_RELATIVE(RUBY_SEARCH_PATH));
+#endif
+
+#ifdef RUBY_SITE_THIN_ARCHLIB
+    ruby_incpush(RUBY_RELATIVE(RUBY_SITE_THIN_ARCHLIB));
+#endif
+    ruby_incpush(RUBY_RELATIVE(RUBY_SITE_ARCHLIB));
+    ruby_incpush(RUBY_RELATIVE(RUBY_SITE_LIB2));
+    ruby_incpush(RUBY_RELATIVE(RUBY_SITE_LIB));
+
+#ifdef RUBY_THIN_ARCHLIB
+    ruby_incpush(RUBY_RELATIVE(RUBY_THIN_ARCHLIB));
+#endif
+    ruby_incpush(RUBY_RELATIVE(RUBY_ARCHLIB));
+
+    ruby_incpush(RUBY_RELATIVE(RUBY_LIB));
+
+    if (rb_safe_level() == 0) {
+	ruby_incpush(".");
     }
 }
 
 struct req_list {
-    const char *name;
+    char *name;
     struct req_list *next;
 } req_list_head;
 struct req_list *req_list_last = &req_list_head;
@@ -228,45 +282,118 @@ add_modules(mod)
     struct req_list *list;
 
     list = ALLOC(struct req_list);
-    list->name = mod;
+    list->name = ALLOC_N(char, strlen(mod)+1);
+    strcpy(list->name, mod);
     list->next = 0;
     req_list_last->next = list;
     req_list_last = list;
 }
 
+extern void Init_ext _((void));
+
 void
-ruby_require_libraries()
+require_libraries()
 {
+    extern NODE *ruby_eval_tree;
+    extern NODE *ruby_eval_tree_begin;
+    char *orig_sourcefile = ruby_sourcefile;
+    NODE *save[2];
     struct req_list *list = req_list_head.next;
     struct req_list *tmp;
 
+    Init_ext();		/* should be called here for some reason :-( */
+    ruby_sourcefile = 0;
+    save[0] = ruby_eval_tree;
+    save[1] = ruby_eval_tree_begin;
+    ruby_eval_tree = ruby_eval_tree_begin = 0;
     req_list_last = 0;
     while (list) {
 	rb_require(list->name);
 	tmp = list->next;
+	free(list->name);
 	free(list);
 	list = tmp;
     }
+    req_list_head.next = 0;
+    ruby_eval_tree = save[0];
+    ruby_eval_tree_begin = save[1];
+    ruby_sourcefile = orig_sourcefile;
 }
 
-extern void Init_ext _((void));
+static void
+process_sflag()
+{
+    if (sflag) {
+	int n;
+	VALUE *args;
+
+	n = RARRAY(rb_argv)->len;
+	args = RARRAY(rb_argv)->ptr;
+	while (n > 0) {
+	    char *s = STR2CSTR(*args++);
+	    char *p;
+
+	    if (s[0] != '-') break;
+	    n--;
+	    if (s[1] == '-' && s[2] == '\0') break;
+
+	    s[0] = '$';
+	    if (p = strchr(s, '=')) {
+		*p++ = '\0';
+		rb_gv_set(s, rb_str_new2(p));
+	    }
+	    else {
+		rb_gv_set(s, Qtrue);
+	    }
+	    s[0] = '-';
+	}
+	n = RARRAY(rb_argv)->len - n;
+	while (n--) {
+	    rb_ary_shift(rb_argv);
+	}
+    }
+    sflag = 0;
+}
+
+static void proc_options _((int argc, char **argv));
+
+static char*
+moreswitches(s)
+    char *s;
+{
+    int argc; char *argv[3];
+    char *p = s;
+
+    argc = 2; argv[0] = argv[2] = 0;
+    while (*s && !ISSPACE(*s))
+	s++;
+    argv[1] = ALLOCA_N(char, s-p+2);
+    argv[1][0] = '-';
+    strncpy(argv[1]+1, p, s-p);
+    argv[1][s-p+1] = '\0';
+    proc_options(argc, argv);
+    while (*s && ISSPACE(*s))
+	s++;
+    return s;
+}
 
 static void
-proc_options(argcp, argvp)
-    int *argcp;
-    char ***argvp;
+proc_options(argc, argv)
+    int argc;
+    char **argv;
 {
-    int argc = *argcp;
-    char **argv = *argvp;
-    int script_given, do_search;
+    char *argv0 = argv[0];
+    int do_search;
     char *s;
 
+    int version = 0;
+    int copyright = 0;
+    int verbose = 0;
+    VALUE e_script = Qfalse;
+
     if (argc == 0) return;
 
-    version = Qfalse;
     do_search = Qfalse;
-    script_given = 0;
-    e_tmpname = NULL;
 
     for (argc--,argv++; argc > 0; argc--,argv++) {
 	if (argv[0][0] != '-' || !argv[0][1]) break;
@@ -289,7 +416,7 @@ proc_options(argcp, argvp)
 
 	  case 'd':
 	    ruby_debug = Qtrue;
-	    ruby_verbose |= 1;
+	    ruby_verbose = Qtrue;
 	    s++;
 	    goto reswitch;
 
@@ -300,9 +427,9 @@ proc_options(argcp, argvp)
 
 	  case 'v':
 	    ruby_show_version();
-	    ruby_verbose = 2;
+	    verbose = 1;
 	  case 'w':
-	    ruby_verbose |= 1;
+	    ruby_verbose = Qtrue;
 	    s++;
 	    goto reswitch;
 
@@ -313,7 +440,7 @@ proc_options(argcp, argvp)
 
 	  case 's':
 	    forbid_setid("-s");
-	    sflag = Qtrue;
+	    sflag = 1;
 	    s++;
 	    goto reswitch;
 
@@ -343,22 +470,12 @@ proc_options(argcp, argvp)
 		fprintf(stderr, "%s: no code specified for -e\n", origargv[0]);
 		exit(2);
 	    }
-	    if (!e_fp) {
-		int fd;
-		e_tmpname = ruby_mktemp();
-		if (!e_tmpname) rb_fatal("Can't mktemp");
-		fd = open(e_tmpname, O_CREAT|O_EXCL|O_RDWR, 0600);
-		if (fd < 0) {
-		    rb_fatal("Cannot open temporary file: %s", e_tmpname);
-		}
-		e_fp = fdopen(fd, "w");
-		if (!e_fp) {
-		    rb_fatal("Cannot open temporary file: %s", e_tmpname);
-		}
-		if (script == 0) script = e_tmpname;
+	    if (!e_script) {
+		e_script = rb_str_new(0,0);
+		if (script == 0) script = "-e";
 	    }
-	    fputs(s, e_fp);
-	    putc('\n', e_fp);
+	    rb_str_cat2(e_script, s);
+	    rb_str_cat2(e_script, "\n");
 	    break;
 
 	  case 'r':
@@ -386,6 +503,7 @@ proc_options(argcp, argvp)
 	    }
 	    break;
 
+	  case 'C':
 	  case 'X':
 	    s++;
 	    if (!*s) {
@@ -401,13 +519,16 @@ proc_options(argcp, argvp)
 	    break;
 
 	  case 'F':
-	    rb_fs = rb_str_new2(s+1);
+	    if (*++s) {
+		rb_fs = rb_str_new2(s);
+	    }
 	    break;
 
 	  case 'K':
-	    s++;
-	    rb_set_kcode(s);
-	    s++;
+	    if (*++s) {
+		rb_set_kcode(s);
+		s++;
+	    }
 	    goto reswitch;
 
 	  case 'T':
@@ -426,9 +547,9 @@ proc_options(argcp, argvp)
 	  case 'I':
 	    forbid_setid("-I");
 	    if (*++s)
-		addpath(s);
+		ruby_incpush(s);
 	    else if (argv[1]) {
-		addpath(argv[1]);
+		ruby_incpush(argv[1]);
 		argc--,argv++;
 	    }
 	    break;
@@ -464,8 +585,10 @@ proc_options(argcp, argvp)
 		ruby_debug = 1;
 	    else if (strcmp("version", s) == 0)
 		version = 1;
-	    else if (strcmp("verbose", s) == 0)
-		ruby_verbose = 2;
+	    else if (strcmp("verbose", s) == 0) {
+		verbose = 1;
+		ruby_verbose = Qtrue;
+	    }
 	    else if (strcmp("yydebug", s) == 0)
 		yydebug = 1;
 	    else if (strcmp("help", s) == 0) {
@@ -495,14 +618,39 @@ proc_options(argcp, argvp)
     }
 
   switch_end:
-    if (*argvp[0] == 0) return;
+    if (argv0 == 0) return;
 
-    if (e_fp) {
-	if (fflush(e_fp) || ferror(e_fp) || fclose(e_fp))
-	    rb_fatal("Cannot write to temp file for -e");
-	e_fp = NULL;
+    if (rb_safe_level() == 0 && (s = getenv("RUBYOPT"))) {
+	while (ISSPACE(*s)) s++;
+	if (*s == '-' && *(s+1) == 'T') {
+	    int numlen;
+	    int v = 1;
+
+	    s += 2;
+	    if (*++s) {
+		v = scan_oct(s, 2, &numlen);
+		if (numlen == 0) v = 1;
+	    }
+	    rb_set_safe_level(v);
+	}
+	else {
+	    while (s && *s) {
+		while (ISSPACE(*s)) s++;
+		if (*s == '-') {
+		    s++;
+		    if (ISSPACE(*s)) continue;
+		}
+		if (!*s) break;
+		if (!strchr("IdvwrK", *s))
+		    rb_raise(rb_eRuntimeError, "Illegal switch in RUBYOPT: -%c", *s);
+		s = moreswitches(s);
+	    }
+	}
+    }
+
+    if (e_script) {
 	argc++, argv--;
-	argv[0] = e_tmpname;
+	argv[0] = script;
     }
 
     if (version) {
@@ -513,71 +661,51 @@ proc_options(argcp, argvp)
 	ruby_show_copyright();
     }
 
-    Init_ext();		/* should be called here for some reason :-( */
-    if (script_given == Qfalse) {
-	if (argc == 0) {	/* no more args */
-	    if (ruby_verbose == 3) exit(0);
+    if (!e_script && argc == 0) { /* no more args */
+	if (verbose) exit(0);
+	script = "-";
+    }
+    else {
+	script = argv[0];
+	if (script[0] == '\0') {
 	    script = "-";
-	    load_stdin();
 	}
 	else {
-	    script = argv[0];
-	    if (script[0] == '\0') {
-		script = "-";
-		load_stdin();
-	    }
-	    else {
-		if (do_search) {
-		    char *path = getenv("RUBYPATH");
+	    if (do_search) {
+		char *path = getenv("RUBYPATH");
 
-		    script = 0;
-		    if (path) {
-			script = dln_find_file(argv[0], path);
-		    }
-		    if (!script) {
-			script = dln_find_file(argv[0], getenv("PATH"));
-		    }
-		    if (!script) script = argv[0];
+		script = 0;
+		if (path) {
+		    script = dln_find_file(argv[0], path);
 		}
-		load_file(script, 1);
+		if (!script) {
+		    script = dln_find_file(argv[0], getenv("PATH"));
+		}
+		if (!script) script = argv[0];
 	    }
-	    argc--; argv++;
 	}
-    }
-    if (ruby_verbose) ruby_verbose = Qtrue;
-    if (ruby_debug) ruby_debug = Qtrue;
-
-    if (rb_safe_level() == 0) {
-	rb_ary_push(rb_load_path, rb_str_new2("."));
-	addpath(getenv("RUBYLIB"));
+	argc--; argv++;
     }
 
-    xflag = Qfalse;
-    *argvp = argv;
-    *argcp = argc;
+    ruby_script(script);
+    ruby_set_argv(argc, argv);
+    process_sflag();
 
-    if (sflag) {
-	char *s;
-
-	argc = *argcp; argv = *argvp;
-	for (; argc > 0 && argv[0][0] == '-'; argc--,argv++) {
-	    if (argv[0][1] == '-') {
-		argc--,argv++;
-		break;
-	    }
-	    argv[0][0] = '$';
-	    if (s = strchr(argv[0], '=')) {
-		*s++ = '\0';
-		rb_gv_set(argv[0], rb_str_new2(s));
-	    }
-	    else {
-		rb_gv_set(argv[0], Qtrue);
-	    }
-	    argv[0][0] = '-';
-	}
-	*argcp = argc; *argvp = argv;
+    ruby_init_loadpath();
+    ruby_sourcefile = argv0;
+    if (e_script) {
+	require_libraries();
+	rb_compile_string(script, e_script, 1);
+    }
+    else if (strlen(script) == 1 && script[0] == '-') {
+	load_stdin();
+    }
+    else {
+	load_file(script, 1);
     }
 
+    process_sflag();
+    xflag = 0;
 }
 
 extern int ruby__end__seen;
@@ -603,6 +731,13 @@ load_file(fname, script)
 	fclose(fp);
 
 	f = rb_file_open(fname, "r");
+#if defined DOSISH || defined __CYGWIN__
+	{
+	    char *ext = strrchr(fname, '.');
+	    if (ext && strcasecmp(ext, ".exe") == 0)
+		rb_io_binmode(f);
+	}
+#endif
     }
 
     if (script) {
@@ -648,7 +783,6 @@ load_file(fname, script)
 		    while (p < pend && !ISSPACE(*p))
 			p++;
 		    *p++ = '\0';
-
 		    if (p < pend) {
 			argv = ALLOCA_N(char*, origargc+3);
 			argv[1] = p;
@@ -658,9 +792,8 @@ load_file(fname, script)
 			argv = origargv;
 		    }
 		    argv[0] = path;
-#ifndef USE_CWGUSI
 		    execv(path, argv);
-#endif
+
 		    ruby_sourcefile = fname;
 		    ruby_sourceline = 1;
 		    rb_fatal("Can't exec %s", path);
@@ -672,30 +805,17 @@ load_file(fname, script)
 		if (RSTRING(line)->ptr[RSTRING(line)->len-2] == '\r')
 		    RSTRING(line)->ptr[RSTRING(line)->len-2] = '\0';
 		if (p = strstr(p, " -")) {
-		    int argc; char *argv[3]; char **argvp = argv;
-		    char *s = ++p;
-
-		    argc = 2; argv[0] = argv[2] = 0;
+		    p++;	/* skip space before `-' */
 		    while (*p == '-') {
-			while (*s && !ISSPACE(*s))
-			    s++;
-			*s = '\0';
-			argv[1] = p;
-			proc_options(&argc, &argvp);
-			p = ++s;
-			while (*p && ISSPACE(*p))
-			    p++;
+			p = moreswitches(p+1);
 		    }
 		}
 	    }
 	}
-	else if (NIL_P(c)) {
-	    rb_io_close(f);
-	    return;
-	}
-	else {
+	else if (!NIL_P(c)) {
 	    rb_io_ungetc(f, c);
 	}
+	require_libraries();	/* Why here? unnatural */
     }
     rb_compile_file(fname, f, line_start);
     if (script && ruby__end__seen) {
@@ -817,45 +937,6 @@ forbid_setid(s)
         rb_raise(rb_eSecurityError, "No %s allowed in tainted mode", s);
 }
 
-#if defined(_WIN32) || defined(DJGPP)
-static char *
-ruby_libpath()
-{
-    static char libpath[FILENAME_MAX+1];
-    char *p;
-#if defined(_WIN32)
-    GetModuleFileName(NULL, libpath, sizeof libpath);
-#elif defined(DJGPP)
-    extern char *__dos_argv0;
-    strcpy(libpath, __dos_argv0);
-#endif
-    p = strrchr(libpath, '\\');
-    if (p) {
-	*p = 0;
-	if (!strcasecmp(p-4, "\\bin"))
-	    p -= 4;
-    } else {
-	strcpy(libpath, ".");
-	p = libpath + 1;
-    }
-
-    strcpy(p, "\\lib");
-#if defined(__CYGWIN32__)
-    p = (char *)malloc(strlen(libpath)+10);
-    if (!p)
-	return 0;
-    cygwin32_conv_to_posix_path(libpath, p);
-    strcpy(libpath, p);
-    free(p);
-#else
-    for (p = libpath; *p; p++)
-	if (*p == '\\')
-	    *p = '/';
-#endif
-    return libpath;
-}
-#endif
-
 void
 ruby_prog_init()
 {
@@ -864,32 +945,12 @@ ruby_prog_init()
     ruby_sourcefile = "ruby";
     rb_define_variable("$VERBOSE", &ruby_verbose);
     rb_define_variable("$-v", &ruby_verbose);
+    rb_define_variable("$-w", &ruby_verbose);
     rb_define_variable("$DEBUG", &ruby_debug);
     rb_define_variable("$-d", &ruby_debug);
     rb_define_readonly_variable("$-p", &do_print);
     rb_define_readonly_variable("$-l", &do_line);
 
-    addpath(RUBY_LIB);
-#if defined(_WIN32) || defined(DJGPP)
-    addpath(ruby_libpath());
-#endif
-
-    addpath(RUBY_ARCHLIB);
-#ifdef RUBY_THIN_ARCHLIB
-    addpath(RUBY_THIN_ARCHLIB);
-#endif
-
-    addpath(RUBY_SITE_LIB);
-    addpath(RUBY_SITE_LIB2);
-    addpath(RUBY_SITE_ARCHLIB);
-#ifdef RUBY_SITE_THIN_ARCHLIB
-    addpath(RUBY_SITE_THIN_ARCHLIB);
-#endif
-
-#ifdef RUBY_SEARCH_PATH
-    addpath(RUBY_SEARCH_PATH);
-#endif
-
     rb_define_hooked_variable("$0", &rb_progname, 0, set_arg0);
 
     rb_argv = rb_ary_new();
@@ -935,9 +996,7 @@ ruby_process_options(argc, argv)
 #if defined(USE_DLN_A_OUT)
     dln_argv0 = argv[0];
 #endif
-    proc_options(&argc, &argv);
-    ruby_script(script);
-    ruby_set_argv(argc, argv);
+    proc_options(argc, argv);
 
     if (do_check && ruby_nerrs == 0) {
 	printf("Syntax OK\n");
@@ -949,12 +1008,4 @@ ruby_process_options(argc, argv)
     if (do_loop) {
 	rb_parser_while_loop(do_line, do_split);
     }
-    if (e_fp) {
-	fclose(e_fp);
-	e_fp = NULL;
-    }
-    if (e_tmpname) {
-	unlink(e_tmpname);
-	e_tmpname = NULL;
-    }
 }
