@@ -1,4 +1,4 @@
-/************************************************
+/**********************************************************************
 
   ruby.h -
 
@@ -6,8 +6,10 @@
   created at: Thu Jun 10 14:26:32 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-*************************************************/
+**********************************************************************/
 
 #ifndef RUBY_H
 #define RUBY_H
@@ -71,16 +73,6 @@ extern "C" {
 #include <alloca.h>
 #endif
 
-#if defined(__CYGWIN32__)
-#if defined(USEIMPORTLIB)
-#include "import.h"
-#else
-#if !defined(__CYGWIN__)
-#define environ (*__imp___cygwin_environ)
-#endif
-#endif
-#endif
-
 #ifdef _AIX
 #pragma alloca
 #endif
@@ -114,11 +106,14 @@ typedef unsigned long ID;
 #define FIXNUM_MIN RSHIFT((long)LONG_MIN,1)
 
 #define FIXNUM_FLAG 0x01
-#define INT2FIX(i) (VALUE)(((long)(i))<<1 | FIXNUM_FLAG)
+#define INT2FIX(i) ((VALUE)(((long)(i))<<1 | FIXNUM_FLAG))
+#define rb_fix_new(v) INT2FIX(v)
 VALUE rb_int2inum _((long));
 #define INT2NUM(v) rb_int2inum(v)
+#define rb_int_new(v) rb_int2inum(v)
 VALUE rb_uint2inum _((unsigned long));
 #define UINT2NUM(v) rb_uint2inum(v)
+#define rb_uint_new(v) rb_uint2inum(v)
 
 #define FIX2LONG(x) RSHIFT((long)x,1)
 #define FIX2ULONG(x) (((unsigned long)(x))>>1)
@@ -127,12 +122,21 @@ VALUE rb_uint2inum _((unsigned long));
 #define NEGFIXABLE(f) ((f) >= FIXNUM_MIN)
 #define FIXABLE(f) (POSFIXABLE(f) && NEGFIXABLE(f))
 
+#define IMMEDIATE_MASK 0x03
+#define IMMEDIATE_P(x) ((VALUE)(x) & IMMEDIATE_MASK)
+
+#define SYMBOL_FLAG 0x0e
+#define SYMBOL_P(x) (((VALUE)(x)&0xff)==SYMBOL_FLAG)
+#define ID2SYM(x) ((VALUE)(((long)(x))<<8|SYMBOL_FLAG))
+#define SYM2ID(x) RSHIFT((long)x,8)
+
 /* special contants - i.e. non-zero and non-fixnum constants */
 #define Qfalse 0
 #define Qtrue  2
 #define Qnil   4
+#define Qundef 6		/* undefined value for placeholder */
 
-#define RTEST(v) rb_test_false_or_nil((VALUE)(v))
+#define RTEST(v) ((VALUE)(v) & ~Qnil)
 #define NIL_P(v) ((VALUE)(v) == Qnil)
 
 #define CLASS_OF(v) rb_class_of((VALUE)(v))
@@ -158,12 +162,14 @@ VALUE rb_uint2inum _((unsigned long));
 #define T_FALSE  0x21
 #define T_DATA   0x22
 #define T_MATCH  0x23
+#define T_SYMBOL 0x24
 
-#define T_VARMAP 0x7d
-#define T_SCOPE  0x7e
-#define T_NODE   0x7f
+#define T_UNDEF  0x3c
+#define T_VARMAP 0x3d
+#define T_SCOPE  0x3e
+#define T_NODE   0x3f
 
-#define T_MASK   0x7f
+#define T_MASK   0x3f
 
 #define BUILTIN_TYPE(x) (((struct RBasic*)(x))->flags & T_MASK)
 
@@ -175,6 +181,10 @@ void rb_check_safe_str _((VALUE));
 #define Check_SafeStr(v) rb_check_safe_str((VALUE)(v))
 void rb_secure _((int));
 
+EXTERN int ruby_safe_level;
+#define rb_safe_level() (ruby_safe_level)
+void rb_set_safe_level _((int));
+
 long rb_num2long _((VALUE));
 unsigned long rb_num2ulong _((VALUE));
 #define NUM2LONG(x) (FIXNUM_P(x)?FIX2LONG(x):rb_num2long((VALUE)x))
@@ -192,8 +202,6 @@ int rb_fix2int _((VALUE));
 #define FIX2INT(x) FIX2LONG(x)
 #define FIX2UINT(x) FIX2ULONG(x)
 #endif
-#define ID2SYM(x) INT2FIX(x)
-#define SYM2ID(x) FIX2INT(x)
 
 double rb_num2dbl _((VALUE));
 #define NUM2DBL(x) rb_num2dbl((VALUE)(x))
@@ -209,8 +217,8 @@ char *rb_str2cstr _((VALUE,int*));
 VALUE rb_newobj _((void));
 #define NEWOBJ(obj,type) type *obj = (type*)rb_newobj()
 #define OBJSETUP(obj,c,t) {\
-    RBASIC(obj)->klass = (c);\
     RBASIC(obj)->flags = (t);\
+    RBASIC(obj)->klass = (c);\
     if (rb_safe_level() >= 3) FL_SET(obj, FL_TAINT);\
 }
 #define CLONESETUP(clone,obj) do {\
@@ -275,22 +283,28 @@ struct RFile {
 
 struct RData {
     struct RBasic basic;
-    void (*dmark)();
-    void (*dfree)();
+    void (*dmark) _((void*));
+    void (*dfree) _((void*));
     void *data;
 };
 
 #define DATA_PTR(dta) (RDATA(dta)->data)
 
-VALUE rb_data_object_alloc _((VALUE,void*,void (*)(),void (*)()));
+/*
+#define RUBY_DATA_FUNC(func) ((void (*)_((void*)))func)
+*/
+typedef void (*RUBY_DATA_FUNC) _((void*));
+
+VALUE rb_data_object_alloc _((VALUE,void*,RUBY_DATA_FUNC,RUBY_DATA_FUNC));
+
+#define Data_Wrap_Struct(klass,mark,free,sval) (\
+    rb_data_object_alloc(klass,sval,(RUBY_DATA_FUNC)mark,(RUBY_DATA_FUNC)free)\
+)
+
 #define Data_Make_Struct(klass,type,mark,free,sval) (\
     sval = ALLOC(type),\
     memset(sval, 0, sizeof(type)),\
-    rb_data_object_alloc(klass,sval,mark,free)\
-)
-
-#define Data_Wrap_Struct(klass,mark,free,sval) (\
-    rb_data_object_alloc(klass,sval,mark,free)\
+    Data_Wrap_Struct(klass,mark,free,sval)\
 )
 
 #define Data_Get_Struct(obj,type,sval) {\
@@ -308,7 +322,7 @@ struct RBignum {
     struct RBasic basic;
     char sign;
     long len;
-    unsigned short *digits;
+    void *digits;
 };
 
 #define R_CAST(st)   (struct st*)
@@ -327,10 +341,11 @@ struct RBignum {
 #define RFILE(obj)   (R_CAST(RFile)(obj))
 
 #define FL_SINGLETON FL_USER0
-#define FL_MARK      (1<<7)
-#define FL_FINALIZE  (1<<8)
-#define FL_TAINT     (1<<9)
-#define FL_EXIVAR    (1<<10)
+#define FL_MARK      (1<<6)
+#define FL_FINALIZE  (1<<7)
+#define FL_TAINT     (1<<8)
+#define FL_EXIVAR    (1<<9)
+#define FL_FREEZE    (1<<10)
 
 #define FL_USHIFT    11
 
@@ -345,18 +360,31 @@ struct RBignum {
 
 #define FL_UMASK  (0xff<<FL_USHIFT)
 
-#define FL_ABLE(x) (!(FIXNUM_P(x)||rb_special_const_p((VALUE)(x))))
+#define SPECIAL_CONST_P(x) (IMMEDIATE_P(x) || !RTEST(x))
+
+#define FL_ABLE(x) (!SPECIAL_CONST_P(x))
 #define FL_TEST(x,f) (FL_ABLE(x)?(RBASIC(x)->flags&(f)):0)
-#define FL_SET(x,f) if (FL_ABLE(x)) {RBASIC(x)->flags |= (f);}
-#define FL_UNSET(x,f) if(FL_ABLE(x)){RBASIC(x)->flags &= ~(f);}
-#define FL_REVERSE(x,f) if(FL_ABLE(x)){RBASIC(x)->flags ^= f;}
+#define FL_SET(x,f) do {if (FL_ABLE(x)) RBASIC(x)->flags |= (f);} while (0)
+#define FL_UNSET(x,f) do {if (FL_ABLE(x)) RBASIC(x)->flags &= ~(f);} while (0)
+#define FL_REVERSE(x,f) do {if (FL_ABLE(x)) RBASIC(x)->flags ^= (f);} while (0)
 
 #define OBJ_TAINTED(x) FL_TEST((x), FL_TAINT)
 #define OBJ_TAINT(x) FL_SET((x), FL_TAINT)
+#define OBJ_INFECT(x,s) do {if (FL_ABLE(x) && FL_ABLE(s)) RBASIC(x)->flags |= RBASIC(s)->flags & FL_TAINT;} while (0)
+
+#define OBJ_FROZEN(x) FL_TEST((x), FL_FREEZE)
+#define OBJ_FREEZE(x) FL_SET((x), FL_FREEZE)
+
+#define xmalloc ruby_xmalloc
+#define xcalloc ruby_xcalloc
+#define xrealloc ruby_xrealloc
+#define xfree ruby_xfree
 
 void *xmalloc _((size_t));
 void *xcalloc _((size_t,size_t));
 void *xrealloc _((void*,size_t));
+void xfree _((void*));
+
 #define ALLOC_N(type,n) (type*)xmalloc(sizeof(type)*(n))
 #define ALLOC(type) (type*)xmalloc(sizeof(type))
 #define REALLOC_N(var,type,n) (var)=(type*)xrealloc((char*)(var),sizeof(type)*(n))
@@ -366,6 +394,9 @@ void *xrealloc _((void*,size_t));
 #define MEMZERO(p,type,n) memset((p), 0, sizeof(type)*(n))
 #define MEMCPY(p1,p2,type,n) memcpy((p1), (p2), sizeof(type)*(n))
 #define MEMMOVE(p1,p2,type,n) memmove((p1), (p2), sizeof(type)*(n))
+#define MEMCMP(p1,p2,type,n) memcmp((p1), (p2), sizeof(type)*(n))
+
+void rb_glob _((char*,void(*)(),VALUE));
 
 VALUE rb_define_class _((const char*,VALUE));
 VALUE rb_define_module _((const char*));
@@ -382,6 +413,7 @@ void rb_define_readonly_variable _((const char*,VALUE*));
 void rb_define_const _((VALUE,const char*,VALUE));
 void rb_define_global_const _((const char*,VALUE));
 
+#define RUBY_METHOD_FUNC(func) ((VALUE (*)__((...)))func)
 void rb_define_method _((VALUE,const char*,VALUE(*)(),int));
 void rb_define_module_function _((VALUE,const char*,VALUE(*)(),int));
 void rb_define_global_function _((const char*,VALUE(*)(),int));
@@ -390,6 +422,10 @@ void rb_undef_method _((VALUE,const char*));
 void rb_define_alias _((VALUE,const char*,const char*));
 void rb_define_attr _((VALUE,const char*,int,int));
 
+void rb_global_variable _((VALUE*));
+void rb_gc_register_address _((VALUE*));
+void rb_gc_unregister_address _((VALUE*));
+
 ID rb_intern _((const char*));
 char *rb_id2name _((ID));
 ID rb_to_id _((VALUE));
@@ -405,22 +441,17 @@ VALUE rb_funcall __((VALUE, ID, int, ...));
 VALUE rb_funcall2 _((VALUE, ID, int, VALUE*));
 VALUE rb_funcall3 _((VALUE, ID, int, VALUE*));
 int rb_scan_args __((int, VALUE*, const char*, ...));
+VALUE rb_call_super _((int, VALUE*));
 
 VALUE rb_gv_set _((const char*, VALUE));
 VALUE rb_gv_get _((const char*));
 VALUE rb_iv_get _((VALUE, const char*));
 VALUE rb_iv_set _((VALUE, const char*, VALUE));
-VALUE rb_const_get _((VALUE, ID));
-VALUE rb_const_get_at _((VALUE, ID));
-void rb_const_set _((VALUE, ID, VALUE));
 
 VALUE rb_equal _((VALUE,VALUE));
 
 EXTERN VALUE ruby_verbose, ruby_debug;
 
-int rb_safe_level _((void));
-void rb_set_safe_level _((int));
-
 void rb_raise __((VALUE, const char*, ...)) NORETURN;
 void rb_fatal __((const char*, ...)) NORETURN;
 void rb_bug __((const char*, ...)) NORETURN;
@@ -434,9 +465,10 @@ void rb_warning __((const char*, ...));		/* reports if `-w' specified */
 
 VALUE rb_each _((VALUE));
 VALUE rb_yield _((VALUE));
-int rb_iterator_p _((void));
+int rb_block_given_p _((void));
 VALUE rb_iterate _((VALUE(*)(),VALUE,VALUE(*)(),VALUE));
 VALUE rb_rescue _((VALUE(*)(),VALUE,VALUE(*)(),VALUE));
+VALUE rb_rescue2 __((VALUE(*)(),VALUE,VALUE(*)(),VALUE,...));
 VALUE rb_ensure _((VALUE(*)(),VALUE,VALUE(*)(),VALUE));
 VALUE rb_catch _((const char*,VALUE(*)(),VALUE));
 void rb_throw _((const char*,VALUE)) NORETURN;
@@ -477,6 +509,7 @@ EXTERN VALUE rb_cProc;
 EXTERN VALUE rb_cRange;
 EXTERN VALUE rb_cRegexp;
 EXTERN VALUE rb_cString;
+EXTERN VALUE rb_cSymbol;
 EXTERN VALUE rb_cThread;
 EXTERN VALUE rb_cTime;
 EXTERN VALUE rb_cTrueClass;
@@ -491,23 +524,26 @@ EXTERN VALUE rb_eFatal;
 EXTERN VALUE rb_eArgError;
 EXTERN VALUE rb_eEOFError;
 EXTERN VALUE rb_eIndexError;
+EXTERN VALUE rb_eRangeError;
 EXTERN VALUE rb_eIOError;
-EXTERN VALUE rb_eLoadError;
-EXTERN VALUE rb_eNameError;
 EXTERN VALUE rb_eRuntimeError;
 EXTERN VALUE rb_eSecurityError;
-EXTERN VALUE rb_eSyntaxError;
 EXTERN VALUE rb_eSystemCallError;
 EXTERN VALUE rb_eTypeError;
 EXTERN VALUE rb_eZeroDivError;
 EXTERN VALUE rb_eNotImpError;
+EXTERN VALUE rb_eNoMemError;
 EXTERN VALUE rb_eFloatDomainError;
 
+EXTERN VALUE rb_eScriptError;
+EXTERN VALUE rb_eNameError;
+EXTERN VALUE rb_eSyntaxError;
+EXTERN VALUE rb_eLoadError;
+
 #if defined(__GNUC__) && __GNUC__ >= 2 && !defined(RUBY_NO_INLINE)
 extern __inline__ VALUE rb_class_of _((VALUE));
 extern __inline__ int rb_type _((VALUE));
 extern __inline__ int rb_special_const_p _((VALUE));
-extern __inline__ int rb_test_false_or_nil _((VALUE));
 
 extern __inline__ VALUE
 rb_class_of(VALUE obj)
@@ -516,6 +552,7 @@ rb_class_of(VALUE obj)
     if (obj == Qnil) return rb_cNilClass;
     if (obj == Qfalse) return rb_cFalseClass;
     if (obj == Qtrue) return rb_cTrueClass;
+    if (SYMBOL_P(obj)) return rb_cSymbol;
 
     return RBASIC(obj)->klass;
 }
@@ -527,29 +564,22 @@ rb_type(VALUE obj)
     if (obj == Qnil) return T_NIL;
     if (obj == Qfalse) return T_FALSE;
     if (obj == Qtrue) return T_TRUE;
+    if (obj == Qundef) return T_UNDEF;
+    if (SYMBOL_P(obj)) return T_SYMBOL;
     return BUILTIN_TYPE(obj);
 }
 
 extern __inline__ int
 rb_special_const_p(VALUE obj)
 {
-    if (FIXNUM_P(obj)) return Qtrue;
-    if (obj == Qnil) return Qtrue;
-    if (obj == Qfalse) return Qtrue;
-    if (obj == Qtrue) return Qtrue;;
+    if (SPECIAL_CONST_P(obj)) return Qtrue;
     return Qfalse;
 }
 
-extern __inline__ int
-rb_test_false_or_nil(VALUE v)
-{
-    return (v != Qnil) && (v != Qfalse);
-}
 #else
 VALUE rb_class_of _((VALUE));
 int rb_type _((VALUE));
 int rb_special_const_p _((VALUE));
-int rb_test_false_or_nil _((VALUE));
 #endif
 
 #include "intern.h"
