@@ -1,12 +1,16 @@
-/************************************************
+/**********************************************************************
 
   signal.c -
 
   $Author: matz $
-  $Date: 2000/01/05 04:41:12 $
+  $Date: 2000/09/19 07:54:22 $
   created at: Tue Dec 20 10:13:44 JST 1994
 
-************************************************/
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
+
+**********************************************************************/
 
 #include "ruby.h"
 #include "rubysig.h"
@@ -25,11 +29,6 @@
 # endif
 #endif
 
-#ifdef USE_CWGUSI
-# undef NSIG
-# define NSIG __signal_max
-#endif
-
 static struct signals {
     char *signm;
     int  signo;
@@ -196,9 +195,6 @@ rb_f_kill(argc, argv)
     int argc;
     VALUE *argv;
 {
-#ifdef USE_CWGUSI
-    rb_notimplement();
-#else
     int negative = 0;
     int sig;
     int i;
@@ -209,14 +205,14 @@ rb_f_kill(argc, argv)
 	rb_raise(rb_eArgError, "wrong # of arguments -- kill(sig, pid...)");
     switch (TYPE(argv[0])) {
       case T_FIXNUM:
-	sig = FIX2UINT(argv[0]);
-	if (sig >= NSIG) {
-	    s = rb_id2name(sig);
-	    if (!s) rb_raise(rb_eArgError, "bad signal");
-	    goto str_signal;
-	}
+	sig = FIX2INT(argv[0]);
 	break;
 
+      case T_SYMBOL:
+	s = rb_id2name(SYM2ID(argv[0]));
+	if (!s) rb_raise(rb_eArgError, "bad signal");
+	goto str_signal;
+
       case T_STRING:
         {
 	    s = RSTRING(argv[0])->ptr;
@@ -261,7 +257,6 @@ rb_f_kill(argc, argv)
 	}
     }
     return INT2FIX(i-1);
-#endif /* USE_CWGUSI */
 }
 
 static VALUE trap_list[NSIG];
@@ -287,7 +282,7 @@ rb_gc_mark_trap_list()
 void
 posix_signal(signum, handler)
     int signum;
-    RETSIGTYPE (*handler)();
+    RETSIGTYPE (*handler)_((int));
 {
     struct sigaction sigact;
 
@@ -317,7 +312,7 @@ signal_exec(sig)
 	  case SIGINT:
 	    rb_thread_interrupt();
 	    break;
-#if !defined(NT) && !defined(__human68k__)
+#ifdef SIGHUP
 	  case SIGHUP:
 #endif
 #ifdef SIGQUIT
@@ -341,6 +336,7 @@ signal_exec(sig)
     }
 }
 
+static RETSIGTYPE sighandle _((int));
 static RETSIGTYPE
 sighandle(sig)
     int sig;
@@ -365,6 +361,7 @@ sighandle(sig)
 }
 
 #ifdef SIGBUS
+static RETSIGTYPE sigbus _((int));
 static RETSIGTYPE
 sigbus(sig)
     int sig;
@@ -374,6 +371,7 @@ sigbus(sig)
 #endif
 
 #ifdef SIGSEGV
+static RETSIGTYPE sigsegv _((int));
 static RETSIGTYPE
 sigsegv(sig)
     int sig;
@@ -409,7 +407,7 @@ rb_trap_exec()
 }
 
 struct trap_arg {
-#if !defined(NT) && !defined(USE_CWGUSI)
+#if !defined(NT)
 # ifdef HAVE_SIGPROCMASK
     sigset_t mask;
 # else
@@ -425,8 +423,10 @@ static sigset_t trap_last_mask;
 static int trap_last_mask;
 # endif
 
+static RETSIGTYPE sigexit _((int));
 static RETSIGTYPE
-sigexit()
+sigexit(sig)
+    int sig;
 {
     rb_exit(0);
 }
@@ -435,9 +435,10 @@ static VALUE
 trap(arg)
     struct trap_arg *arg;
 {
-    RETSIGTYPE (*func)();
+    RETSIGTYPE (*func)_((int));
     VALUE command, old;
     int sig;
+    char *s;
 
     func = sighandle;
     command = arg->cmd;
@@ -475,18 +476,27 @@ trap(arg)
 	command = 0;
     }
 
-    if (TYPE(arg->sig) == T_STRING) {
-	char *s = RSTRING(arg->sig)->ptr;
+    switch (TYPE(arg->sig)) {
+      case T_FIXNUM:
+	sig = NUM2INT(arg->sig);
+	break;
 
+      case T_SYMBOL:
+	s = rb_id2name(SYM2ID(arg->sig));
+	if (!s) rb_raise(rb_eArgError, "bad signal");
+	goto str_signal;
+
+      case T_STRING:
+	s = RSTRING(arg->sig)->ptr;
+
+      str_signal:
 	if (strncmp("SIG", s, 3) == 0)
 	    s += 3;
 	sig = signm2signo(s);
 	if (sig == 0 && strcmp(s, "EXIT") != 0)
 	    rb_raise(rb_eArgError, "invalid signal SIG%s", s);
     }
-    else {
-	sig = NUM2INT(arg->sig);
-    }
+
     if (sig < 0 || sig > NSIG) {
 	rb_raise(rb_eArgError, "invalid signal number (%d)", sig);
     }
@@ -498,7 +508,7 @@ trap(arg)
     if (func == SIG_DFL) {
 	switch (sig) {
 	  case SIGINT:
-#if !defined(NT) && !defined(__human68k__)
+#ifdef SIGHUP
 	  case SIGHUP:
 #endif
 #ifdef SIGQUIT
@@ -538,7 +548,7 @@ trap(arg)
 
     trap_list[sig] = command;
     /* enable at least specified signal. */
-#if !defined(NT) && !defined(USE_CWGUSI)
+#if !defined(NT)
 #ifdef HAVE_SIGPROCMASK
     sigdelset(&arg->mask, sig);
 #else
@@ -548,7 +558,7 @@ trap(arg)
     return old;
 }
 
-#if !defined(NT) && !defined(USE_CWGUSI)
+#if !defined(NT)
 static VALUE
 trap_ensure(arg)
     struct trap_arg *arg;
@@ -567,7 +577,7 @@ trap_ensure(arg)
 void
 rb_trap_restore_mask()
 {
-#if !defined(NT) && !defined(USE_CWGUSI)
+#if !defined(NT)
 # ifdef HAVE_SIGPROCMASK
     sigprocmask(SIG_SETMASK, &trap_last_mask, NULL);
 # else
@@ -596,7 +606,7 @@ rb_f_trap(argc, argv)
 	arg.cmd = argv[1];
     }
 
-#if !defined(NT) && !defined(USE_CWGUSI)
+#if !defined(NT)
     /* disable interrupt */
 # ifdef HAVE_SIGPROCMASK
     sigfillset(&arg.mask);
@@ -617,7 +627,7 @@ Init_signal()
 #ifndef MACOS_UNUSE_SIGNAL
     rb_define_global_function("trap", rb_f_trap, -1);
     ruby_signal(SIGINT, sighandle);
-#ifndef NT
+#ifdef SIGHUP
     ruby_signal(SIGHUP, sighandle);
 #endif
 #ifdef SIGQUIT
