@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   string.c -
 
   $Author: matz $
-  $Date: 2000/07/25 06:15:18 $
+  $Date: 2000/09/07 06:59:36 $
   created at: Mon Aug  9 17:12:58 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "re.h"
@@ -22,16 +24,11 @@
 #include <unistd.h>
 #endif
 
-#ifndef atof
-double strtod();
-#endif
-
 VALUE rb_cString;
 
-#define STR_FREEZE FL_USER1
-#define STR_NO_ORIG FL_USER3
+#define STR_NO_ORIG FL_USER2
 
-extern VALUE rb_rs;
+VALUE rb_fs;
 
 VALUE
 rb_str_new(ptr, len)
@@ -66,7 +63,7 @@ rb_tainted_str_new(ptr, len)
 {
     VALUE str = rb_str_new(ptr, len);
 
-    FL_SET(str, FL_TAINT);
+    OBJ_TAINT(str);
     return str;
 }
 
@@ -76,7 +73,7 @@ rb_tainted_str_new2(ptr)
 {
     VALUE str = rb_str_new2(ptr);
 
-    FL_SET(str, FL_TAINT);
+    OBJ_TAINT(str);
     return str;
 }
 
@@ -90,6 +87,7 @@ rb_str_new3(str)
     str2->len = RSTRING(str)->len;
     str2->ptr = RSTRING(str)->ptr;
     str2->orig = str;
+    OBJ_INFECT(str2, str);
 
     return (VALUE)str2;
 }
@@ -98,11 +96,15 @@ VALUE
 rb_str_new4(orig)
     VALUE orig;
 {
-    if (FL_TEST(orig, STR_FREEZE)) {
-	return orig;
-    }
-    else if (RSTRING(orig)->orig && !FL_TEST(orig, STR_NO_ORIG)) {
-	return rb_str_freeze(RSTRING(orig)->orig);
+    if (OBJ_FROZEN(orig)) return orig;
+    if (RSTRING(orig)->orig) {
+	if (FL_TEST(orig, STR_NO_ORIG)) {
+	    orig = rb_str_new(RSTRING(orig)->ptr, RSTRING(orig)->len);
+	    OBJ_FREEZE(orig);
+	    return orig;
+	}
+	OBJ_FREEZE(RSTRING(orig)->orig);
+	return RSTRING(orig)->orig;
     }
     else {
 	NEWOBJ(str, struct RString);
@@ -112,10 +114,9 @@ rb_str_new4(orig)
 	str->ptr = RSTRING(orig)->ptr;
 	RSTRING(orig)->orig = (VALUE)str;
 	str->orig = 0;
-	if (FL_TEST(orig, FL_TAINT)) {
-	    FL_SET(str, FL_TAINT);
-	}
-	FL_SET(str, STR_FREEZE);
+	OBJ_INFECT(str, orig);
+	OBJ_FREEZE(str);
+
 	return (VALUE)str;
     }
 }
@@ -128,7 +129,7 @@ rb_str_to_str(str)
 }
 
 static void
-rb_str_assign(str, str2)
+rb_str_become(str, str2)
     VALUE str, str2;
 {
     if (str == str2) return;
@@ -138,7 +139,7 @@ rb_str_assign(str, str2)
 	RSTRING(str)->orig = 0;
 	return;
     }
-    if ((!RSTRING(str)->orig||FL_TEST(str, STR_NO_ORIG))&&RSTRING(str)->ptr)
+    if ((!RSTRING(str)->orig||FL_TEST(str,STR_NO_ORIG))&&RSTRING(str)->ptr)
 	free(RSTRING(str)->ptr);
     RSTRING(str)->ptr = RSTRING(str2)->ptr;
     RSTRING(str)->len = RSTRING(str2)->len;
@@ -148,6 +149,20 @@ rb_str_assign(str, str2)
     if (OBJ_TAINTED(str2)) OBJ_TAINT(str);
 }
 
+void
+rb_str_associate(str, add)
+    VALUE str, add;
+{
+    if (!FL_TEST(str, STR_NO_ORIG)) {
+	if (RSTRING(str)->orig) {
+	    rb_str_modify(str);
+	}
+	RSTRING(str)->orig = rb_ary_new();
+	FL_SET(str, STR_NO_ORIG);
+    }
+    rb_ary_push(RSTRING(str)->orig, add);
+}
+
 static ID to_str;
 
 VALUE
@@ -166,57 +181,65 @@ rb_obj_as_string(obj)
     return str;
 }
 
-static VALUE
-rb_str_clone(orig)
-    VALUE orig;
-{
-    VALUE str;
-
-    if (RSTRING(orig)->orig && !FL_TEST(orig, STR_NO_ORIG))
-	str = rb_str_new3(RSTRING(orig)->orig);
-    else
-	str = rb_str_new(RSTRING(orig)->ptr, RSTRING(orig)->len);
-    if (RSTRING(orig)->orig && FL_TEST(orig, STR_NO_ORIG))
-	RSTRING(str)->orig = RSTRING(orig)->orig;
-    CLONESETUP(str, orig);
-    return str;
-}
-
 VALUE
 rb_str_dup(str)
     VALUE str;
 {
-    VALUE s;
+    VALUE str2;
+    VALUE klass;
 
     if (TYPE(str) != T_STRING) str = rb_str_to_str(str);
-    s = rb_str_new(RSTRING(str)->ptr, RSTRING(str)->len);
-    if (OBJ_TAINTED(str)) OBJ_TAINT(s);
+    klass = CLASS_OF(str);
+    while (TYPE(klass) == T_ICLASS || FL_TEST(klass, FL_SINGLETON)) {
+	klass = (VALUE)RCLASS(klass)->super;
+    }
 
-    return s;
+    if (OBJ_FROZEN(str)) str2 = rb_str_new3(str);
+    else if (FL_TEST(str, STR_NO_ORIG)) {
+	str2 = rb_str_new(RSTRING(str)->ptr, RSTRING(str)->len);
+    }
+    else if (RSTRING(str)->orig) {
+	str2 = rb_str_new3(RSTRING(str)->orig);
+    }
+    else {
+	NEWOBJ(dup, struct RString);
+	OBJSETUP(dup, klass, T_STRING);
+
+	str2 = rb_str_new4(str);
+	dup->len = RSTRING(str2)->len;
+	dup->ptr = RSTRING(str2)->ptr;
+	dup->orig = str2;
+	str2 = (VALUE)dup;
+    }
+    OBJ_INFECT(str2, str);
+    RBASIC(str2)->klass = klass;
+    return str2;
+}
+
+
+static VALUE
+rb_str_clone(str)
+    VALUE str;
+{
+    VALUE clone = rb_str_dup(str);
+    if (FL_TEST(str, STR_NO_ORIG))
+	RSTRING(clone)->orig = RSTRING(str)->orig;
+    CLONESETUP(clone, str);
+
+    return clone;
 }
 
 static VALUE
-rb_str_s_new(klass, orig)
+rb_str_s_new(argc, argv, klass)
+    int argc;
+    VALUE *argv;
     VALUE klass;
-    VALUE orig;
 {
-    NEWOBJ(str, struct RString);
+    VALUE str = rb_str_new(0, 0);
     OBJSETUP(str, klass, T_STRING);
 
-    str->orig = 0;
-    orig = rb_obj_as_string(orig);
-    str->len = RSTRING(orig)->len;
-    if (RSTRING(orig)->ptr) {
-	str->ptr = ALLOC_N(char, RSTRING(orig)->len+1);
-	memcpy(str->ptr, RSTRING(orig)->ptr, RSTRING(orig)->len);
-	str->ptr[RSTRING(orig)->len] = '\0';
-    }
-
-    if (rb_safe_level() >= 3) {
-	FL_SET(str, FL_TAINT);
-    }
-
-    return (VALUE)str;
+    rb_obj_call_init(str, argc, argv);
+    return str;
 }
 
 static VALUE
@@ -266,7 +289,7 @@ rb_str_times(str, times)
     if (len < 0) {
 	rb_raise(rb_eArgError, "negative argument");
     }
-    if (LONG_MAX / len <  RSTRING(str)->len) {
+    if (LONG_MAX/len <  RSTRING(str)->len) {
 	rb_raise(rb_eArgError, "argument too big");
     }
 
@@ -312,6 +335,7 @@ rb_str_substr(str, beg, len)
 
     if (len < 0) return Qnil;
     if (beg > RSTRING(str)->len) return Qnil;
+    if (beg == RSTRING(str)->len && len > 0) return Qnil;
     if (beg < 0) {
 	beg += RSTRING(str)->len;
 	if (beg < 0) return Qnil;
@@ -330,41 +354,39 @@ rb_str_substr(str, beg, len)
     return str2;
 }
 
+static int
+str_independent(str)
+    VALUE str;
+{
+    if (OBJ_FROZEN(str)) rb_error_frozen("string");
+    if (!OBJ_TAINTED(str) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: can't modify string");
+    if (!RSTRING(str)->orig || FL_TEST(str, STR_NO_ORIG)) return 1;
+    if (TYPE(RSTRING(str)->orig) != T_STRING) rb_bug("non string str->orig");
+    RSTRING(str)->orig = 0;
+    return 0;
+}
+
 void
 rb_str_modify(str)
     VALUE str;
 {
     char *ptr;
 
-    if (FL_TEST(str, STR_FREEZE))
-	rb_raise(rb_eTypeError, "can't modify frozen string");
-    if (!FL_TEST(str, FL_TAINT) && rb_safe_level() >= 4)
-	rb_raise(rb_eSecurityError, "Insecure: can't modify string");
-    if (!RSTRING(str)->orig || FL_TEST(str, STR_NO_ORIG)) return;
-    ptr = RSTRING(str)->ptr;
-    RSTRING(str)->ptr = ALLOC_N(char, RSTRING(str)->len+1);
+    if (str_independent(str)) return;
+    ptr = ALLOC_N(char, RSTRING(str)->len+1);
     if (RSTRING(str)->ptr) {
-	memcpy(RSTRING(str)->ptr, ptr, RSTRING(str)->len);
-	RSTRING(str)->ptr[RSTRING(str)->len] = 0;
+	memcpy(ptr, RSTRING(str)->ptr, RSTRING(str)->len);
     }
-    RSTRING(str)->orig = 0;
+    ptr[RSTRING(str)->len] = 0;
+    RSTRING(str)->ptr = ptr;
 }
 
 VALUE
 rb_str_freeze(str)
     VALUE str;
 {
-    FL_SET(str, STR_FREEZE);
-    return str;
-}
-
-static VALUE
-rb_str_frozen_p(str)
-    VALUE str;
-{
-    if (FL_TEST(str, STR_FREEZE))
-	return Qtrue;
-    return Qfalse;
+    return rb_obj_freeze(str);
 }
 
 VALUE
@@ -372,11 +394,13 @@ rb_str_dup_frozen(str)
     VALUE str;
 {
     if (RSTRING(str)->orig && !FL_TEST(str, STR_NO_ORIG)) {
-	return rb_str_freeze(RSTRING(str)->orig);
+	OBJ_FREEZE(RSTRING(str)->orig);
+	return RSTRING(str)->orig;
     }
-    if (FL_TEST(str, STR_FREEZE))
-	return str;
-    return rb_str_freeze(rb_str_dup(str));
+    if (OBJ_FROZEN(str)) return str;
+    str = rb_str_dup(str);
+    OBJ_FREEZE(str);
+    return str;
 }
 
 VALUE
@@ -403,21 +427,47 @@ rb_str_cat(str, ptr, len)
     long len;
 {
     if (len > 0) {
+	int poffset = -1;
+
 	rb_str_modify(str);
+	if (RSTRING(str)->ptr <= ptr &&
+	    ptr < RSTRING(str)->ptr + RSTRING(str)->len) {
+	    poffset = ptr - RSTRING(str)->ptr;
+	}
 	REALLOC_N(RSTRING(str)->ptr, char, RSTRING(str)->len + len + 1);
-	if (ptr)
+	if (ptr) {
+	    if (poffset >= 0) ptr = RSTRING(str)->ptr + poffset;
 	    memcpy(RSTRING(str)->ptr + RSTRING(str)->len, ptr, len);
+	}
 	RSTRING(str)->len += len;
 	RSTRING(str)->ptr[RSTRING(str)->len] = '\0'; /* sentinel */
     }
     return str;
 }
 
+VALUE
+rb_str_cat2(str, ptr)
+    VALUE str;
+    const char *ptr;
+{
+    return rb_str_cat(str, ptr, strlen(ptr));
+}
+
+VALUE
+rb_str_append(str1, str2)
+    VALUE str1, str2;
+{
+    if (TYPE(str2) != T_STRING) str2 = rb_str_to_str(str2);
+    str1 = rb_str_cat(str1, RSTRING(str2)->ptr, RSTRING(str2)->len);
+    OBJ_INFECT(str1, str2);
+
+    return str1;
+}
+
 VALUE
 rb_str_concat(str1, str2)
     VALUE str1, str2;
 {
-    rb_str_modify(str1);
     if (FIXNUM_P(str2)) {
 	int i = FIX2INT(str2);
 	if (0 <= i && i <= 0xff) { /* byte */
@@ -425,8 +475,9 @@ rb_str_concat(str1, str2)
 	    return rb_str_cat(str1, &c, 1);
 	}
     }
-    if (TYPE(str2) != T_STRING) str2 = rb_str_to_str(str2);
-    return rb_str_cat(str1, RSTRING(str2)->ptr, RSTRING(str2)->len);
+    str1 = rb_str_append(str1, str2);
+
+    return str1;
 }
 
 int
@@ -437,6 +488,29 @@ rb_str_hash(str)
     register char *p = RSTRING(str)->ptr;
     register int key = 0;
 
+#ifdef HASH_ELFHASH
+    register unsigned int g;
+
+    while (len--) {
+	key = (key << 4) + *p++;
+	if (g = key & 0xF0000000)
+	    key ^= g >> 24;
+	key &= ~g;
+    }
+#elif HASH_PERL
+    if (ruby_ignorecase) {
+	while (len--) {
+	    key = key*33 + toupper(*p);
+	    p++;
+	}
+    }
+    else {
+	while (len--) {
+	    key = key*33 + *p++;
+	}
+    }
+    key = key + (key>>5);
+#else
     if (ruby_ignorecase) {
 	while (len--) {
 	    key = key*65599 + toupper(*p);
@@ -449,11 +523,13 @@ rb_str_hash(str)
 	    p++;
 	}
     }
+    key = key + (key>>5);
+#endif
     return key;
 }
 
 static VALUE
-rb_str_hash_method(str)
+rb_str_hash_m(str)
     VALUE str;
 {
     int key = rb_str_hash(str);
@@ -469,12 +545,8 @@ rb_str_cmp(str1, str2)
     long len;
     int retval;
 
-    if (ruby_ignorecase) {
-	return rb_str_cicmp(str1, str2);
-    }
-
     len = lesser(RSTRING(str1)->len, RSTRING(str2)->len);
-    retval = memcmp(RSTRING(str1)->ptr, RSTRING(str2)->ptr, len);
+    retval = rb_memcmp(RSTRING(str1)->ptr, RSTRING(str2)->ptr, len);
     if (retval == 0) {
 	if (RSTRING(str1)->len == RSTRING(str2)->len) return 0;
 	if (RSTRING(str1)->len > RSTRING(str2)->len) return 1;
@@ -500,7 +572,7 @@ rb_str_equal(str1, str2)
 }
 
 static VALUE
-rb_str_cmp_method(str1, str2)
+rb_str_cmp_m(str1, str2)
     VALUE str1, str2;
 {
     int result;
@@ -560,7 +632,7 @@ rb_str_index(str, sub, offset)
     if (len == 0) return offset;
     e = RSTRING(str)->ptr + RSTRING(str)->len - len + 1;
     while (s < e) {
-	if (*s == *(RSTRING(sub)->ptr) && memcmp(s, p, len) == 0) {
+	if (rb_memcmp(s, p, len) == 0) {
 	    return (s-(RSTRING(str)->ptr));
 	}
 	s++;
@@ -569,7 +641,7 @@ rb_str_index(str, sub, offset)
 }
 
 static VALUE
-rb_str_index_method(argc, argv, str)
+rb_str_index_m(argc, argv, str)
     int argc;
     VALUE *argv;
     VALUE str;
@@ -591,6 +663,7 @@ rb_str_index_method(argc, argv, str)
 
     switch (TYPE(sub)) {
       case T_REGEXP:
+	pos = rb_reg_adjust_startpos(sub, str, pos, 0);
 	pos = rb_reg_search(sub, str, pos, 0);
 	break;
 
@@ -645,6 +718,7 @@ rb_str_rindex(argc, argv, str)
     switch (TYPE(sub)) {
       case T_REGEXP:
 	if (RREGEXP(sub)->len) {
+	    pos = rb_reg_adjust_startpos(sub, str, pos, 1);
 	    pos = rb_reg_search(sub, str, pos, 1);
 	}
 	if (pos >= 0) return INT2NUM(pos);
@@ -662,7 +736,7 @@ rb_str_rindex(argc, argv, str)
 	t = RSTRING(sub)->ptr;
 	if (len) {
 	    while (sbeg <= s) {
-		if (*s == *t && memcmp(s, t, len) == 0) {
+		if (rb_memcmp(s, t, len) == 0) {
 		    return INT2NUM(s - RSTRING(str)->ptr);
 		}
 		s--;
@@ -722,13 +796,13 @@ static VALUE
 rb_str_succ(orig)
     VALUE orig;
 {
-    VALUE str, str2;
+    VALUE str;
     char *sbeg, *s;
     int c = -1;
     int n = 0;
 
     str = rb_str_new(RSTRING(orig)->ptr, RSTRING(orig)->len);
-    if (OBJ_TAINTED(orig)) OBJ_TAINT(str);
+    OBJ_INFECT(str, orig);
     if (RSTRING(str)->len == 0) return str;
 
     sbeg = RSTRING(str)->ptr; s = sbeg + RSTRING(str)->len - 1;
@@ -765,7 +839,7 @@ rb_str_succ_bang(str)
     VALUE str;
 {
     rb_str_modify(str);
-    rb_str_assign(str, rb_str_succ(str));
+    rb_str_become(str, rb_str_succ(str));
 
     return str;
 }
@@ -794,7 +868,7 @@ rb_str_upto(beg, end, excl)
 }
 
 static VALUE
-rb_str_upto_method(beg, end)
+rb_str_upto_m(beg, end)
     VALUE beg, end;
 {
     return rb_str_upto(beg, end, 0);
@@ -811,6 +885,7 @@ rb_str_aref(str, indx)
       case T_FIXNUM:
 	idx = FIX2LONG(indx);
 
+      num_index:
 	if (idx < 0) {
 	    idx = RSTRING(str)->len + idx;
 	}
@@ -841,30 +916,32 @@ rb_str_aref(str, indx)
 		return rb_str_substr(str, beg, len);
 	    }
 	}
-	rb_raise(rb_eIndexError, "invalid index for string");
+	idx = NUM2LONG(indx);
+	goto num_index;
     }
     return Qnil;		/* not reached */
 }
 
 static VALUE
-rb_str_aref_method(argc, argv, str)
+rb_str_aref_m(argc, argv, str)
     int argc;
     VALUE *argv;
     VALUE str;
 {
-    VALUE arg1, arg2;
-
-    if (rb_scan_args(argc, argv, "11", &arg1, &arg2) == 2) {
-	return rb_str_substr(str, NUM2INT(arg1), NUM2INT(arg2));
+    if (argc == 2) {
+	return rb_str_substr(str, NUM2INT(argv[0]), NUM2INT(argv[1]));
     }
-    return rb_str_aref(str, arg1);
+    if (argc != 1) {
+	rb_raise(rb_eArgError, "wrong # of arguments(%d for 1)", argc);
+    }
+    return rb_str_aref(str, argv[0]);
 }
 
 static void
 rb_str_replace(str, beg, len, val)
     VALUE str, val;
-    int beg;
-    int len;
+    long beg;
+    long len;
 {
     if (RSTRING(str)->len < beg + len) {
 	len = RSTRING(str)->len - beg;
@@ -875,7 +952,7 @@ rb_str_replace(str, beg, len, val)
 	REALLOC_N(RSTRING(str)->ptr, char, RSTRING(str)->len+RSTRING(val)->len-len+1);
     }
 
-    if (len != RSTRING(val)->len) {
+    if (RSTRING(val)->len != len) {
 	memmove(RSTRING(str)->ptr + beg + RSTRING(val)->len,
 		RSTRING(str)->ptr + beg + len,
 		RSTRING(str)->len - (beg + len));
@@ -883,7 +960,9 @@ rb_str_replace(str, beg, len, val)
     if (RSTRING(str)->len < beg && len < 0) {
 	MEMZERO(RSTRING(str)->ptr + RSTRING(str)->len, char, -len);
     }
-    memmove(RSTRING(str)->ptr+beg, RSTRING(val)->ptr, RSTRING(val)->len);
+    if (RSTRING(val)->len > 0) {
+	memmove(RSTRING(str)->ptr+beg, RSTRING(val)->ptr, RSTRING(val)->len);
+    }
     RSTRING(str)->len += RSTRING(val)->len - len;
     RSTRING(str)->ptr[RSTRING(str)->len] = '\0';
 }
@@ -895,12 +974,12 @@ rb_str_aset(str, indx, val)
     VALUE str;
     VALUE indx, val;
 {
-    int idx;
-    int beg;
+    long idx, beg;
 
     switch (TYPE(indx)) {
       case T_FIXNUM:
-	beg = idx = NUM2INT(indx);
+      num_index:
+	idx = NUM2INT(indx);
 	if (idx < 0) {
 	    idx += RSTRING(str)->len;
 	}
@@ -947,26 +1026,24 @@ rb_str_aset(str, indx, val)
 		return val;
 	    }
 	}
-	rb_raise(rb_eIndexError, "invalid index for string");
+	idx = NUM2LONG(indx);
+	goto num_index;
     }
 }
 
 static VALUE
-rb_str_aset_method(argc, argv, str)
+rb_str_aset_m(argc, argv, str)
     int argc;
     VALUE *argv;
     VALUE str;
 {
-    VALUE arg1, arg2, arg3;
-
     rb_str_modify(str);
+    if (argc == 3) {
+	long beg, len;
 
-    if (rb_scan_args(argc, argv, "21", &arg1, &arg2, &arg3) == 3) {
-	int beg, len;
-
-	if (TYPE(arg3) != T_STRING) arg3 = rb_str_to_str(arg3);
-	beg = NUM2INT(arg1);
-	len = NUM2INT(arg2);
+	if (TYPE(argv[2]) != T_STRING) argv[2] = rb_str_to_str(argv[2]);
+	beg = NUM2INT(argv[0]);
+	len = NUM2INT(argv[1]);
 	if (len < 0) rb_raise(rb_eIndexError, "negative length %d", len);
 	if (beg < 0) {
 	    beg += RSTRING(str)->len;
@@ -980,10 +1057,35 @@ rb_str_aset_method(argc, argv, str)
 	if (beg + len > RSTRING(str)->len) {
 	    len = RSTRING(str)->len - beg;
 	}
-	rb_str_replace(str, beg, len, arg3);
-	return arg3;
+	rb_str_replace(str, beg, len, argv[2]);
+	return argv[2];
     }
-    return rb_str_aset(str, arg1, arg2);
+    if (argc != 2) {
+	rb_raise(rb_eArgError, "wrong # of arguments(%d for 2)", argc);
+    }
+    return rb_str_aset(str, argv[0], argv[1]);
+}
+
+static VALUE
+rb_str_slice_bang(argc, argv, str)
+    int argc;
+    VALUE *argv;
+    VALUE str;
+{
+    VALUE result;
+    VALUE buf[3];
+    int i;
+
+    if (argc < 1 || 2 < argc) {
+	rb_raise(rb_eArgError, "wrong # of arguments(%d for 1)", argc);
+    }
+    for (i=0; i<argc; i++) {
+	buf[i] = argv[i];
+    }
+    buf[i] = rb_str_new(0,0);
+    result = rb_str_aref_m(argc, buf, str);
+    rb_str_aset_m(argc+1, buf, str);
+    return result;
 }
 
 static VALUE
@@ -1014,13 +1116,15 @@ rb_str_sub_bang(argc, argv, str)
     VALUE pat, repl, match;
     struct re_registers *regs;
     int iter = 0;
-    int plen;
+    int tainted = 0;
+    long plen;
 
-    if (argc == 1 && rb_iterator_p()) {
+    if (argc == 1 && rb_block_given_p()) {
 	iter = 1;
     }
     else if (argc == 2) {
 	repl = rb_obj_as_string(argv[1]);;
+	if (OBJ_TAINTED(repl)) tainted = 1;
     }
     else {
 	rb_raise(rb_eArgError, "wrong # of arguments(%d for 2)", argc);
@@ -1033,14 +1137,14 @@ rb_str_sub_bang(argc, argv, str)
 	regs = RMATCH(match)->regs;
 
 	if (iter) {
-	    rb_match_busy(match, Qtrue);
+	    rb_match_busy(match);
 	    repl = rb_obj_as_string(rb_yield(rb_reg_nth_match(0, match)));
-	    rb_match_busy(match, Qfalse);
 	    rb_backref_set(match);
 	}
 	else {
 	    repl = rb_reg_regsub(repl, str, regs);
 	}
+	if (OBJ_TAINTED(repl)) tainted = 1;
 	plen = END(0) - BEG(0);
 	if (RSTRING(repl)->len > plen) {
 	    REALLOC_N(RSTRING(str)->ptr, char,
@@ -1055,6 +1159,8 @@ rb_str_sub_bang(argc, argv, str)
 	       RSTRING(repl)->ptr, RSTRING(repl)->len);
 	RSTRING(str)->len += RSTRING(repl)->len - plen;
 	RSTRING(str)->ptr[RSTRING(str)->len] = '\0';
+	if (tainted) OBJ_TAINT(str);
+
 	return str;
     }
     return Qnil;
@@ -1072,23 +1178,26 @@ rb_str_sub(argc, argv, str)
 }
 
 static VALUE
-rb_str_gsub_bang(argc, argv, str)
+str_gsub(argc, argv, str, bang)
     int argc;
     VALUE *argv;
     VALUE str;
+    int bang;
 {
     VALUE pat, val, repl, match;
     struct re_registers *regs;
-    int beg, n;
+    long beg, n;
+    long offset, blen, len;
     int iter = 0;
     char *buf, *bp, *cp;
-    int offset, blen, len;
+    int tainted = 0;
 
-    if (argc == 1 && rb_iterator_p()) {
+    if (argc == 1 && rb_block_given_p()) {
 	iter = 1;
     }
     else if (argc == 2) {
-	repl = rb_obj_as_string(argv[1]);;
+	repl = rb_obj_as_string(argv[1]);
+	if (OBJ_TAINTED(repl)) tainted = 1;
     }
     else {
 	rb_raise(rb_eArgError, "wrong # of arguments(%d for 2)", argc);
@@ -1097,7 +1206,10 @@ rb_str_gsub_bang(argc, argv, str)
     pat = get_pat(argv[0]);
     offset=0; n=0; 
     beg = rb_reg_search(pat, str, 0, 0);
-    if (beg < 0) return Qnil;	/* no match, no substitution */
+    if (beg < 0) {
+	if (bang) return Qnil;	/* no match, no substitution */
+	return rb_str_dup(str);
+    }
 
     blen = RSTRING(str)->len + 30; /* len + margin */
     buf = ALLOC_N(char, blen);
@@ -1109,14 +1221,14 @@ rb_str_gsub_bang(argc, argv, str)
 	match = rb_backref_get();
 	regs = RMATCH(match)->regs;
 	if (iter) {
-	    rb_match_busy(match, Qtrue);
+	    rb_match_busy(match);
 	    val = rb_obj_as_string(rb_yield(rb_reg_nth_match(0, match)));
-	    rb_match_busy(match, Qfalse);
 	    rb_backref_set(match);
 	}
 	else {
 	    val = rb_reg_regsub(repl, str, regs);
 	}
+	if (OBJ_TAINTED(val)) tainted = 1;
 	len = (bp - buf) + (beg - offset) + RSTRING(val)->len + 3;
 	if (blen < len) {
 	    while (blen < len) blen *= 2;
@@ -1157,29 +1269,47 @@ rb_str_gsub_bang(argc, argv, str)
 	memcpy(bp, cp, RSTRING(str)->len - offset);
 	bp += RSTRING(str)->len - offset;
     }
-    rb_str_modify(str);
-    free(RSTRING(str)->ptr);
+    rb_backref_set(match);
+    if (bang) {
+	if (str_independent(str)) {
+	    free(RSTRING(str)->ptr);
+	}
+    }
+    else {
+	NEWOBJ(dup, struct RString);
+	OBJSETUP(dup, rb_cString, T_STRING);
+	OBJ_INFECT(dup, str);
+	str = (VALUE)dup;
+	dup->orig = 0;
+    }
     RSTRING(str)->ptr = buf;
     RSTRING(str)->len = len = bp - buf;
     RSTRING(str)->ptr[len] = '\0';
 
+    if (tainted) OBJ_TAINT(str);
     return str;
 }
 
+static VALUE
+rb_str_gsub_bang(argc, argv, str)
+    int argc;
+    VALUE *argv;
+    VALUE str;
+{
+    return str_gsub(argc, argv, str, 1);
+}
+
 static VALUE
 rb_str_gsub(argc, argv, str)
     int argc;
     VALUE *argv;
     VALUE str;
 {
-    str = rb_str_dup(str);
-
-    rb_str_gsub_bang(argc, argv, str);
-    return str;
+    return str_gsub(argc, argv, str, 0);
 }
 
 static VALUE
-rb_str_replace_method(str, str2)
+rb_str_replace_m(str, str2)
     VALUE str, str2;
 {
     if (TYPE(str2) != T_STRING) str2 = rb_str_to_str(str2);
@@ -1219,7 +1349,8 @@ rb_f_sub(argc, argv)
 {
     VALUE str = rb_str_dup(uscore_get());
 
-    rb_str_sub_bang(argc, argv, str);
+    if (NIL_P(rb_str_sub_bang(argc, argv, str)))
+	return str;
     rb_lastline_set(str);
     return str;
 }
@@ -1239,7 +1370,8 @@ rb_f_gsub(argc, argv)
 {
     VALUE str = rb_str_dup(uscore_get());
 
-    rb_str_gsub_bang(argc, argv, str);
+    if (NIL_P(rb_str_gsub_bang(argc, argv, str)))
+	return str;
     rb_lastline_set(str);
     return str;
 }
@@ -1312,7 +1444,7 @@ static VALUE
 rb_str_to_i(str)
     VALUE str;
 {
-    return rb_str2inum(RSTRING(str)->ptr, 10);
+    return rb_str2inum(str, 10);
 }
 
 static VALUE
@@ -1339,6 +1471,7 @@ rb_str_inspect(str)
     char buf[STRMAX];
     char *p, *pend;
     char *b;
+    VALUE inspect;
 
     p = RSTRING(str)->ptr; pend = p + RSTRING(str)->len;
     b = buf;
@@ -1415,14 +1548,16 @@ rb_str_inspect(str)
 	}
     }
     *b++ = '"';
-    return rb_str_new(buf, b - buf);
+    inspect = rb_str_new(buf, b - buf);
+    OBJ_INFECT(inspect, str);
+    return inspect;
 }
 
 static VALUE
 rb_str_dump(str)
     VALUE str;
 {
-    int len;
+    long len;
     char *p, *pend;
     char *q, *qend;
     VALUE result;
@@ -1990,7 +2125,7 @@ rb_str_count(argc, argv, str)
 }
 
 static VALUE
-rb_str_split_method(argc, argv, str)
+rb_str_split_m(argc, argv, str)
     int argc;
     VALUE *argv;
     VALUE str;
@@ -1998,7 +2133,7 @@ rb_str_split_method(argc, argv, str)
     VALUE spat;
     VALUE limit;
     int char_sep = -1;
-    int beg, end, i;
+    long beg, end, i;
     int lim = 0;
     VALUE result, tmp;
 
@@ -2017,9 +2152,9 @@ rb_str_split_method(argc, argv, str)
 	char_sep = ' ';
     }
     else {
+      fs_set:
 	switch (TYPE(spat)) {
 	  case T_STRING:
-	  fs_set:
 	    if (RSTRING(spat)->len == 1) {
 		char_sep = (unsigned char)RSTRING(spat)->ptr[0];
 	    }
@@ -2038,7 +2173,7 @@ rb_str_split_method(argc, argv, str)
     beg = 0;
     if (char_sep >= 0) {
 	char *ptr = RSTRING(str)->ptr;
-	int len = RSTRING(str)->len;
+	long len = RSTRING(str)->len;
 	char *eptr = ptr + len;
 
 	if (char_sep == ' ') {	/* AWK emulation */
@@ -2080,9 +2215,9 @@ rb_str_split_method(argc, argv, str)
 	}
     }
     else {
-	int start = beg;
+	long start = beg;
+	long idx;
 	int last_null = 0;
-	int idx;
 	struct re_registers *regs;
 
 	while ((end = rb_reg_search(spat, str, start, 0)) >= 0) {
@@ -2116,7 +2251,11 @@ rb_str_split_method(argc, argv, str)
 	}
     }
     if (!NIL_P(limit) || RSTRING(str)->len > beg || lim < 0) {
-	rb_ary_push(result, rb_str_substr(str, beg, RSTRING(str)->len-beg));
+	if (RSTRING(str)->len == beg)
+	    tmp = rb_str_new(0, 0);
+	else
+	    tmp = rb_str_substr(str, beg, RSTRING(str)->len-beg);
+	rb_ary_push(result, tmp);
     }
     if (NIL_P(limit) && lim == 0) {
 	while (RARRAY(result)->len > 0 &&
@@ -2136,7 +2275,7 @@ rb_str_split(str, sep0)
 
     if (TYPE(str) != T_STRING) str = rb_str_to_str(str);
     sep = rb_str_new2(sep0);
-    return rb_str_split_method(1, &sep, str);
+    return rb_str_split_m(1, &sep, str);
 }
 
 static VALUE
@@ -2144,7 +2283,7 @@ rb_f_split(argc, argv)
     int argc;
     VALUE *argv;
 {
-    return rb_str_split_method(argc, argv, uscore_get());
+    return rb_str_split_m(argc, argv, uscore_get());
 }
 
 static VALUE
@@ -2158,7 +2297,7 @@ rb_str_each_line(argc, argv, str)
     int rslen;
     char *p = RSTRING(str)->ptr, *pend = p + RSTRING(str)->len, *s;
     char *ptr = p;
-    int len = RSTRING(str)->len;
+    long len = RSTRING(str)->len;
     VALUE line;
 
     if (rb_scan_args(argc, argv, "01", &rs) == 0) {
@@ -2188,7 +2327,7 @@ rb_str_each_line(argc, argv, str)
 	}
 	if (p[-1] == newline &&
 	    (rslen <= 1 ||
-	     memcmp(RSTRING(rs)->ptr, p-rslen, rslen) == 0)) {
+	     rb_memcmp(RSTRING(rs)->ptr, p-rslen, rslen) == 0)) {
 	    line = rb_str_new(s, p - s);
 	    rb_yield(line);
 	    if (RSTRING(str)->ptr != ptr || RSTRING(str)->len != len)
@@ -2200,6 +2339,7 @@ rb_str_each_line(argc, argv, str)
     if (s != pend) {
         if (p > pend) p = pend;
 	line = rb_str_new(s, p - s);
+	OBJ_INFECT(line, str);
 	rb_yield(line);
     }
 
@@ -2210,7 +2350,7 @@ static VALUE
 rb_str_each_byte(str)
     VALUE str;
 {
-    int i;
+    long i;
 
     for (i=0; i<RSTRING(str)->len; i++) {
 	rb_yield(INT2FIX(RSTRING(str)->ptr[i] & 0xff));
@@ -2256,10 +2396,13 @@ rb_f_chop_bang(str)
 static VALUE
 rb_f_chop()
 {
-    VALUE str = rb_str_dup(uscore_get());
+    VALUE str = uscore_get();
 
-    rb_str_chop_bang(str);
-    rb_lastline_set(str);
+    if (RSTRING(str)->len > 0) {
+	str = rb_str_dup(str);
+	rb_str_chop_bang(str);
+	rb_lastline_set(str);
+    }
     return str;
 }
 
@@ -2273,7 +2416,7 @@ rb_str_chomp_bang(argc, argv, str)
     int newline;
     int rslen;
     char *p = RSTRING(str)->ptr;
-    int len = RSTRING(str)->len;
+    long len = RSTRING(str)->len;
 
     if (rb_scan_args(argc, argv, "01", &rs) == 0) {
 	rs = rb_rs;
@@ -2299,7 +2442,7 @@ rb_str_chomp_bang(argc, argv, str)
 
     if (p[len-1] == newline &&
 	(rslen <= 1 ||
-	 memcmp(RSTRING(rs)->ptr, p+len-rslen, rslen) == 0)) {
+	 rb_memcmp(RSTRING(rs)->ptr, p+len-rslen, rslen) == 0)) {
 	rb_str_modify(str);
 	RSTRING(str)->len -= rslen;
 	RSTRING(str)->ptr[RSTRING(str)->len] = '\0';
@@ -2332,11 +2475,13 @@ rb_f_chomp(argc, argv)
     int argc;
     VALUE *argv;
 {
-    VALUE str = rb_str_dup(uscore_get());
+    VALUE str = uscore_get();
+    VALUE dup = rb_str_dup(str);
 
-    rb_str_chomp_bang(argc, argv, str);
-    rb_lastline_set(str);
-    return str;
+    if (NIL_P(rb_str_chomp_bang(argc, argv, dup)))
+	return str;
+    rb_lastline_set(dup);
+    return dup;
 }
 
 static VALUE
@@ -2424,28 +2569,42 @@ rb_str_scan(str, pat)
 {
     VALUE result;
     long start = 0;
+    VALUE match = Qnil;
 
     pat = get_pat(pat);
-    if (!rb_iterator_p()) {
+    if (!rb_block_given_p()) {
 	VALUE ary = rb_ary_new();
 
 	while (!NIL_P(result = scan_once(str, pat, &start))) {
+	    match = rb_backref_get();
 	    rb_ary_push(ary, result);
 	}
+	rb_backref_set(match);
 	return ary;
     }
-
+    
     while (!NIL_P(result = scan_once(str, pat, &start))) {
+	match = rb_backref_get();
+	rb_match_busy(match);
 	rb_yield(result);
+	rb_backref_set(match);	/* restore $~ value */
     }
+    rb_backref_set(match);
     return str;
 }
 
+static VALUE
+rb_f_scan(self, pat)
+    VALUE self, pat;
+{
+    return rb_str_scan(uscore_get(), pat);
+}
+
 static VALUE
 rb_str_hex(str)
     VALUE str;
 {
-    return rb_str2inum(RSTRING(str)->ptr, 16);
+    return rb_str2inum(str, 16);
 }
 
 static VALUE
@@ -2466,7 +2625,7 @@ rb_str_oct(str)
 	    break;
 	}
     }
-    return rb_str2inum(RSTRING(str)->ptr, base);
+    return rb_str2inum(str, base);
 }
 
 static VALUE
@@ -2490,7 +2649,7 @@ rb_str_intern(str)
     if (strlen(RSTRING(str)->ptr) != RSTRING(str)->len)
 	rb_raise(rb_eArgError, "string contains `\\0'");
     id = rb_intern(RSTRING(str)->ptr);
-    return INT2FIX(id);
+    return ID2SYM(id);
 }
 
 static VALUE
@@ -2602,27 +2761,40 @@ rb_str_center(str, w)
     return res;
 }
 
+void
+rb_str_setter(val, id, var)
+    VALUE val;
+    ID id;
+    VALUE *var;
+{
+    if (!NIL_P(val) && TYPE(val) != T_STRING) {
+	rb_raise(rb_eTypeError, "value of %s must be String", rb_id2name(id));
+    }
+    *var = val;
+}
+
 void
 Init_String()
 {
     rb_cString  = rb_define_class("String", rb_cObject);
     rb_include_module(rb_cString, rb_mComparable);
     rb_include_module(rb_cString, rb_mEnumerable);
-    rb_define_singleton_method(rb_cString, "new", rb_str_s_new, 1);
+    rb_define_singleton_method(rb_cString, "new", rb_str_s_new, -1);
+    rb_define_method(rb_cString, "initialize", rb_str_replace_m, 1);
     rb_define_method(rb_cString, "clone", rb_str_clone, 0);
     rb_define_method(rb_cString, "dup", rb_str_dup, 0);
-    rb_define_method(rb_cString, "<=>", rb_str_cmp_method, 1);
+    rb_define_method(rb_cString, "<=>", rb_str_cmp_m, 1);
     rb_define_method(rb_cString, "==", rb_str_equal, 1);
     rb_define_method(rb_cString, "===", rb_str_equal, 1);
     rb_define_method(rb_cString, "eql?", rb_str_equal, 1);
-    rb_define_method(rb_cString, "hash", rb_str_hash_method, 0);
+    rb_define_method(rb_cString, "hash", rb_str_hash_m, 0);
     rb_define_method(rb_cString, "+", rb_str_plus, 1);
     rb_define_method(rb_cString, "*", rb_str_times, 1);
     rb_define_method(rb_cString, "%", rb_str_format, 1);
-    rb_define_method(rb_cString, "[]", rb_str_aref_method, -1);
-    rb_define_method(rb_cString, "[]=", rb_str_aset_method, -1);
+    rb_define_method(rb_cString, "[]", rb_str_aref_m, -1);
+    rb_define_method(rb_cString, "[]=", rb_str_aset_m, -1);
     rb_define_method(rb_cString, "length", rb_str_length, 0);
-    rb_define_alias(rb_cString,  "size", "length");
+    rb_define_method(rb_cString, "size", rb_str_length, 0);
     rb_define_method(rb_cString, "empty?", rb_str_empty, 0);
     rb_define_method(rb_cString, "=~", rb_str_match, 1);
     rb_define_method(rb_cString, "~", rb_str_match2, 0);
@@ -2630,13 +2802,10 @@ Init_String()
     rb_define_method(rb_cString, "succ!", rb_str_succ_bang, 0);
     rb_define_method(rb_cString, "next", rb_str_succ, 0);
     rb_define_method(rb_cString, "next!", rb_str_succ_bang, 0);
-    rb_define_method(rb_cString, "upto", rb_str_upto_method, 1);
-    rb_define_method(rb_cString, "index", rb_str_index_method, -1);
+    rb_define_method(rb_cString, "upto", rb_str_upto_m, 1);
+    rb_define_method(rb_cString, "index", rb_str_index_m, -1);
     rb_define_method(rb_cString, "rindex", rb_str_rindex, -1);
-    rb_define_method(rb_cString, "replace", rb_str_replace_method, 1);
-
-    rb_define_method(rb_cString, "freeze", rb_str_freeze, 0);
-    rb_define_method(rb_cString, "frozen?", rb_str_frozen_p, 0);
+    rb_define_method(rb_cString, "replace", rb_str_replace_m, 1);
 
     rb_define_method(rb_cString, "to_i", rb_str_to_i, 0);
     rb_define_method(rb_cString, "to_f", rb_str_to_f, 0);
@@ -2657,7 +2826,7 @@ Init_String()
 
     rb_define_method(rb_cString, "hex", rb_str_hex, 0);
     rb_define_method(rb_cString, "oct", rb_str_oct, 0);
-    rb_define_method(rb_cString, "split", rb_str_split_method, -1);
+    rb_define_method(rb_cString, "split", rb_str_split_m, -1);
     rb_define_method(rb_cString, "reverse", rb_str_reverse, 0);
     rb_define_method(rb_cString, "reverse!", rb_str_reverse_bang, 0);
     rb_define_method(rb_cString, "concat", rb_str_concat, 1);
@@ -2715,6 +2884,14 @@ Init_String()
     rb_define_global_function("chomp!", rb_f_chomp_bang, -1);
 
     rb_define_global_function("split", rb_f_split, -1);
+    rb_define_global_function("scan", rb_f_scan, 1);
+
+    rb_define_method(rb_cString, "slice", rb_str_aref_m, -1);
+    rb_define_method(rb_cString, "slice!", rb_str_slice_bang, -1);
 
     to_str = rb_intern("to_s");
+
+    rb_fs = Qnil;
+    rb_define_hooked_variable("$;", &rb_fs, 0, rb_str_setter);
+    rb_define_hooked_variable("$-F", &rb_fs, 0, rb_str_setter);
 }
