@@ -1,19 +1,17 @@
-/************************************************
+/**********************************************************************
 
   struct.c -
 
   $Author: matz $
-  $Date: 1999/12/14 06:50:39 $
+  $Date: 2000/08/09 04:32:18 $
   created at: Tue Mar 22 18:44:30 JST 1995
 
-************************************************/
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
+
+**********************************************************************/
 
 #include "ruby.h"
 
-#ifdef USE_CWGUSI
-#include <stdio.h>
-#endif
-
 VALUE rb_cStruct;
 
 static VALUE struct_alloc _((int, VALUE*, VALUE));
@@ -54,12 +52,12 @@ rb_struct_s_members(obj)
 
     member = iv_get(obj, "__member__");
     if (NIL_P(member)) {
-	rb_bug("non-initialized struct");
+	rb_bug("uninitialized struct");
     }
     ary = rb_ary_new2(RARRAY(member)->len);
     p = RARRAY(member)->ptr; pend = p + RARRAY(member)->len;
     while (p < pend) {
-	rb_ary_push(ary, rb_str_new2(rb_id2name(FIX2INT(*p))));
+	rb_ary_push(ary, rb_str_new2(rb_id2name(SYM2ID(*p))));
 	p++;
     }
 
@@ -83,9 +81,9 @@ rb_struct_getmember(obj, id)
 
     member = iv_get(class_of(obj), "__member__");
     if (NIL_P(member)) {
-	rb_bug("non-initialized struct");
+	rb_bug("uninitialized struct");
     }
-    slot = INT2NUM(id);
+    slot = ID2SYM(id);
     for (i=0; i<RARRAY(member)->len; i++) {
 	if (RARRAY(member)->ptr[i] == slot) {
 	    return RSTRUCT(obj)->ptr[i];
@@ -139,7 +137,7 @@ rb_struct_set(obj, val)
     }
     for (i=0; i<RARRAY(member)->len; i++) {
 	slot = RARRAY(member)->ptr[i];
-	if (rb_id_attrset(FIX2INT(slot)) == rb_frame_last_func()) {
+	if (rb_id_attrset(SYM2ID(slot)) == rb_frame_last_func()) {
 	    return RSTRUCT(obj)->ptr[i] = val;
 	}
     }
@@ -173,7 +171,7 @@ make_struct(name, member, klass)
     rb_define_singleton_method(nstr, "[]", struct_alloc, -1);
     rb_define_singleton_method(nstr, "members", rb_struct_s_members, 0);
     for (i=0; i< RARRAY(member)->len; i++) {
-	ID id = FIX2INT(RARRAY(member)->ptr[i]);
+	ID id = SYM2ID(RARRAY(member)->ptr[i]);
 	if (i<10) {
 	    rb_define_method_id(nstr, id, ref_func[i], 0);
 	}
@@ -207,13 +205,14 @@ rb_struct_define(name, va_alist)
     VALUE nm, ary;
     char *mem;
 
-    nm = rb_str_new2(name);
+    if (!name) nm = Qnil;
+    else nm = rb_str_new2(name);
     ary = rb_ary_new();
 
     va_init_list(ar, name);
     while (mem = va_arg(ar, char*)) {
 	ID slot = rb_intern(mem);
-	rb_ary_push(ary, INT2FIX(slot));
+	rb_ary_push(ary, ID2SYM(slot));
     }
     va_end(ar);
 
@@ -229,11 +228,17 @@ rb_struct_s_def(argc, argv, klass)
     VALUE name, rest;
     long i;
     VALUE st;
+    ID id;
 
     rb_scan_args(argc, argv, "1*", &name, &rest);
     for (i=0; i<RARRAY(rest)->len; i++) {
-	ID id = rb_to_id(RARRAY(rest)->ptr[i]);
-	RARRAY(rest)->ptr[i] = INT2FIX(id);
+	id = rb_to_id(RARRAY(rest)->ptr[i]);
+	RARRAY(rest)->ptr[i] = ID2SYM(id);
+    }
+    if (!NIL_P(name) && TYPE(name) != T_STRING) {
+	id = rb_to_id(name);
+	rb_ary_unshift(rest, ID2SYM(id));
+	name = Qnil;
     }
     st = make_struct(name, rest, klass);
 
@@ -249,7 +254,7 @@ rb_struct_initialize(self, values)
     long n;
 
     size = iv_get(klass, "__size__");
-    n = FIX2INT(size);
+    n = FIX2LONG(size);
     if (n < RARRAY(values)->len) {
 	rb_raise(rb_eArgError, "struct size differs");
     }
@@ -276,7 +281,6 @@ struct_alloc(argc, argv, klass)
     size = iv_get(klass, "__size__");
     n = FIX2LONG(size);
 
-    st->len = 0;		/* avoid GC crashing  */
     st->ptr = ALLOC_N(VALUE, n);
     rb_mem_clear(st->ptr, n);
     st->len = n;
@@ -354,23 +358,24 @@ inspect_struct(s)
     }
 
     str = rb_str_new2("#<");
-    rb_str_cat(str, cname, strlen(cname));
-    rb_str_cat(str, " ", 1);
+    rb_str_cat2(str, cname);
+    rb_str_cat2(str, " ");
     for (i=0; i<RSTRUCT(s)->len; i++) {
 	VALUE str2, slot;
 	char *p;
 
 	if (i > 0) {
-	    rb_str_cat(str, ", ", 2);
+	    rb_str_cat2(str, ", ");
 	}
 	slot = RARRAY(member)->ptr[i];
-	p = rb_id2name(FIX2LONG(slot));
-	rb_str_cat(str, p, strlen(p));
-	rb_str_cat(str, "=", 1);
+	p = rb_id2name(SYM2ID(slot));
+	rb_str_cat2(str, p);
+	rb_str_cat2(str, "=");
 	str2 = rb_inspect(RSTRUCT(s)->ptr[i]);
-	rb_str_cat(str, RSTRING(str2)->ptr, RSTRING(str2)->len);
+	rb_str_append(str, str2);
     }
-    rb_str_cat(str, ">", 1);
+    rb_str_cat2(str, ">");
+    OBJ_INFECT(str, s);
 
     return str;
 }
@@ -400,14 +405,13 @@ static VALUE
 rb_struct_clone(s)
     VALUE s;
 {
-    NEWOBJ(st, struct RStruct);
-    CLONESETUP(st, s);
-    st->len = 0;		/* avoid GC crashing  */
-    st->ptr = ALLOC_N(VALUE, RSTRUCT(s)->len);
-    st->len = RSTRUCT(s)->len;
-    MEMCPY(st->ptr, RSTRUCT(s)->ptr, VALUE, st->len);
+    NEWOBJ(clone, struct RStruct);
+    CLONESETUP(clone, s);
+    clone->ptr = ALLOC_N(VALUE, RSTRUCT(s)->len);
+    clone->len = RSTRUCT(s)->len;
+    MEMCPY(clone->ptr, RSTRUCT(s)->ptr, VALUE, clone->len);
 
-    return (VALUE)st;
+    return (VALUE)clone;
 }
 
 static VALUE
@@ -425,7 +429,7 @@ rb_struct_aref_id(s, id)
 
     len = RARRAY(member)->len;
     for (i=0; i<len; i++) {
-	if (FIX2UINT(RARRAY(member)->ptr[i]) == id) {
+	if (SYM2ID(RARRAY(member)->ptr[i]) == id) {
 	    return RSTRUCT(s)->ptr[i];
 	}
     }
@@ -439,7 +443,7 @@ rb_struct_aref(s, idx)
 {
     long i;
 
-    if (TYPE(idx) == T_STRING) {
+    if (TYPE(idx) == T_STRING || TYPE(idx) == T_SYMBOL) {
 	return rb_struct_aref_id(s, rb_to_id(idx));
     }
 
@@ -467,9 +471,10 @@ rb_struct_aset_id(s, id, val)
 	rb_bug("non-initialized struct");
     }
 
+    if (OBJ_FROZEN(s)) rb_error_frozen("Struct");
     len = RARRAY(member)->len;
     for (i=0; i<len; i++) {
-	if (FIX2UINT(RARRAY(member)->ptr[i]) == id) {
+	if (SYM2ID(RARRAY(member)->ptr[i]) == id) {
 	    RSTRUCT(s)->ptr[i] = val;
 	    return val;
 	}
@@ -483,18 +488,21 @@ rb_struct_aset(s, idx, val)
 {
     long i;
 
-    if (TYPE(idx) == T_STRING) {
+    if (TYPE(idx) == T_STRING || TYPE(idx) == T_SYMBOL) {
 	return rb_struct_aset_id(s, rb_to_id(idx), val);
     }
 
     i = NUM2LONG(idx);
     if (i < 0) i = RSTRUCT(s)->len + i;
-    if (i < 0)
+    if (i < 0) {
         rb_raise(rb_eIndexError, "offset %d too small for struct(size:%d)",
 		 i, RSTRUCT(s)->len);
-    if (RSTRUCT(s)->len <= i)
+    }
+    if (RSTRUCT(s)->len <= i) {
         rb_raise(rb_eIndexError, "offset %d too large for struct(size:%d)",
 		 i, RSTRUCT(s)->len);
+    }
+    if (OBJ_FROZEN(s)) rb_error_frozen("Struct");
     return RSTRUCT(s)->ptr[i] = val;
 }
 
@@ -517,35 +525,10 @@ rb_struct_equal(s, s2)
 }
 
 static VALUE
-rb_struct_eql(s, s2)
-    VALUE s, s2;
-{
-    long i;
-
-    if (TYPE(s2) != T_STRUCT) return Qfalse;
-    if (CLASS_OF(s) != CLASS_OF(s2)) return Qfalse;
-    if (RSTRUCT(s)->len != RSTRUCT(s2)->len) {
-	rb_bug("inconsistent struct"); /* should never happen */
-    }
-
-    for (i=0; i<RSTRUCT(s)->len; i++) {
-	if (!rb_eql(RSTRUCT(s)->ptr[i], RSTRUCT(s2)->ptr[i])) return Qfalse;
-    }
-    return Qtrue;
-}
-
-static VALUE
-rb_struct_hash(s)
+rb_struct_size(s)
     VALUE s;
 {
-    long i;
-    int h;
-
-    h = CLASS_OF(s);
-    for (i=0; i<RSTRUCT(s)->len; i++) {
-	h ^= rb_hash(RSTRUCT(s)->ptr[i]);
-    }
-    return INT2FIX(h);
+    return INT2FIX(RSTRUCT(s)->len);
 }
 
 void
@@ -560,13 +543,13 @@ Init_Struct()
     rb_define_method(rb_cStruct, "clone", rb_struct_clone, 0);
 
     rb_define_method(rb_cStruct, "==", rb_struct_equal, 1);
-    rb_define_method(rb_cStruct, "eql?", rb_struct_eql, 1);
-    rb_define_method(rb_cStruct, "hash", rb_struct_hash, 0);
 
     rb_define_method(rb_cStruct, "to_s", rb_struct_to_s, 0);
     rb_define_method(rb_cStruct, "inspect", rb_struct_inspect, 0);
     rb_define_method(rb_cStruct, "to_a", rb_struct_to_a, 0);
     rb_define_method(rb_cStruct, "values", rb_struct_to_a, 0);
+    rb_define_method(rb_cStruct, "size", rb_struct_size, 0);
+    rb_define_method(rb_cStruct, "length", rb_struct_size, 0);
 
     rb_define_method(rb_cStruct, "each", rb_struct_each, 0);
     rb_define_method(rb_cStruct, "[]", rb_struct_aref, 1);
