@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   time.c -
 
   $Author: matz $
-  $Date: 2000/06/23 16:06:50 $
+  $Date: 2000/09/07 06:59:37 $
   created at: Tue Dec 28 14:31:59 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include <sys/types.h>
@@ -30,12 +30,6 @@ struct timeval {
 #include <sys/times.h>
 #endif
 
-#ifdef USE_CWGUSI
-#define time_t long
-int gettimeofday(struct timeval*, struct timezone*);
-int strcasecmp(char*, char*);
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -73,6 +67,18 @@ time_s_now(klass)
     return obj;
 }
 
+static VALUE
+time_s_new(argc, argv, klass)
+    int argc;
+    VALUE *argv;
+    VALUE klass;
+{
+    VALUE obj = time_s_now(klass);
+
+    rb_obj_call_init(obj, argc, argv);
+    return obj;
+}
+
 static VALUE
 time_new_internal(klass, sec, usec)
     VALUE klass;
@@ -81,10 +87,9 @@ time_new_internal(klass, sec, usec)
     VALUE obj;
     struct time_object *tobj;
 
-#ifndef USE_CWGUSI
     if (sec < 0 || (sec == 0 && usec < 0))
 	rb_raise(rb_eArgError, "time must be positive");
-#endif
+
     obj = Data_Make_Struct(klass, struct time_object, 0, free, tobj);
     tobj->tm_got = 0;
     tobj->tv.tv_sec = sec;
@@ -100,9 +105,10 @@ rb_time_new(sec, usec)
     return time_new_internal(rb_cTime, sec, usec);
 }
 
-struct timeval
-rb_time_interval(time)
+static struct timeval
+time_timeval(time, interval)
     VALUE time;
+    int interval;
 {
     struct timeval t;
 
@@ -129,13 +135,21 @@ rb_time_interval(time)
 	break;
 
       default:
-	rb_raise(rb_eTypeError, "can't convert %s into Time interval",
-		 rb_class2name(CLASS_OF(time)));
+	rb_raise(rb_eTypeError, "can't convert %s into Time%s",
+		 rb_class2name(CLASS_OF(time)),
+		 interval ? " interval" : "");
 	break;
     }
     return t;
 }
 
+struct timeval
+rb_time_interval(time)
+    VALUE time;
+{
+    return time_timeval(time, Qtrue);
+}
+
 struct timeval
 rb_time_timeval(time)
     VALUE time;
@@ -148,17 +162,25 @@ rb_time_timeval(time)
 	t = tobj->tv;
 	return t;
     }
-    return rb_time_interval(time);
+    return time_timeval(time, Qfalse);
 }
 
 static VALUE
-time_s_at(klass, time)
-    VALUE klass, time;
+time_s_at(argc, argv, klass)
+    int argc;
+    VALUE *argv;
+    VALUE klass;
 {
     struct timeval tv;
-    VALUE t;
+    VALUE time, t;
 
-    tv = rb_time_timeval(time);
+    if (rb_scan_args(argc, argv, "11", &time, &t) == 2) {
+	tv.tv_sec = NUM2INT(time);
+	tv.tv_usec = NUM2INT(t);
+    }
+    else {
+	tv = rb_time_timeval(time);
+    }
     t = time_new_internal(klass, tv.tv_sec, tv.tv_usec);
     if (TYPE(time) == T_DATA) {
 	struct time_object *tobj, *tobj2;
@@ -180,19 +202,20 @@ obj2long(obj)
     VALUE obj;
 {
     if (TYPE(obj) == T_STRING) {
-	obj = rb_str2inum(RSTRING(obj)->ptr, 10);
+	obj = rb_str2inum(obj, 10);
     }
 
     return NUM2LONG(obj);
 }
 
 static void
-time_arg(argc, argv, tm)
+time_arg(argc, argv, tm, usec)
     int argc;
     VALUE *argv;
     struct tm *tm;
+    time_t *usec;
 {
-    VALUE v[6];
+    VALUE v[7];
     int i;
 
     MEMZERO(tm, struct tm, 1);
@@ -203,10 +226,12 @@ time_arg(argc, argv, tm)
 	v[3] = argv[2];
 	v[4] = argv[1];
 	v[5] = argv[0];
+	*usec = 0;
 	tm->tm_isdst = RTEST(argv[9]) ? 1 : 0;
     }
     else {
-	rb_scan_args(argc, argv, "15", &v[0],&v[1],&v[2],&v[3],&v[4],&v[5]);
+	rb_scan_args(argc, argv, "16", &v[0],&v[1],&v[2],&v[3],&v[4],&v[5],&v[6]);
+	*usec = (argc == 7) ? NUM2INT(v[6]) : 0;
     }
 
     tm->tm_year = obj2long(v[0]);
@@ -357,40 +382,38 @@ make_time_t(tptr, utc_or_local)
 }
 
 static VALUE
-time_gm_or_local(argc, argv, gm_or_local, klass)
+time_utc_or_local(argc, argv, utc_p, klass)
     int argc;
     VALUE *argv;
-    int gm_or_local;
+    int utc_p;
     VALUE klass;
 {
     struct tm tm;
-    struct tm *(*fn)();
     VALUE time;
+    time_t usec;
 
-    fn = (gm_or_local) ? gmtime : localtime;
-    time_arg(argc, argv, &tm);
-
-    time = time_new_internal(klass, make_time_t(&tm, gm_or_local), 0);
-    if (gm_or_local) return time_gmtime(time);
+    time_arg(argc, argv, &tm, &usec);
+    time = time_new_internal(klass, make_time_t(&tm, utc_p), usec);
+    if (utc_p) return time_gmtime(time);
     return time_localtime(time);
 }
 
 static VALUE
-time_s_timegm(argc, argv, klass)
+time_s_mkutc(argc, argv, klass)
     int argc;
     VALUE *argv;
     VALUE klass;
 {
-    return time_gm_or_local(argc, argv, 1, klass);
+    return time_utc_or_local(argc, argv, Qtrue, klass);
 }
 
 static VALUE
-time_s_timelocal(argc, argv, klass)
+time_s_mktime(argc, argv, klass)
     int argc;
     VALUE *argv;
     VALUE klass;
 {
-    return time_gm_or_local(argc, argv, 0, klass);
+    return time_utc_or_local(argc, argv, Qfalse, klass);
 }
 
 static VALUE
@@ -410,7 +433,7 @@ time_to_f(time)
     struct time_object *tobj;
 
     GetTimeval(time, tobj);
-    return rb_float_new((double)tobj->tv.tv_sec+(double)tobj->tv.tv_usec/1000000);
+    return rb_float_new((double)tobj->tv.tv_sec+(double)tobj->tv.tv_usec/1e6);
 }
 
 static VALUE
@@ -484,7 +507,7 @@ time_eql(time1, time2)
 }
 
 static VALUE
-time_gmt_p(time)
+time_utc_p(time)
     VALUE time;
 {
     struct time_object *tobj;
@@ -521,6 +544,15 @@ time_clone(time)
     return clone;
 }
 
+static void
+time_modify(time)
+    VALUE time;
+{
+    if (OBJ_FROZEN(time)) rb_error_frozen("Time");
+    if (!OBJ_TAINTED(time) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: can't modify Time");
+}
+
 static VALUE
 time_localtime(time)
     VALUE time;
@@ -533,6 +565,7 @@ time_localtime(time)
     if (tobj->tm_got && !tobj->gmt) {
 	return time;
     }
+    time_modify(time);
     t = tobj->tv.tv_sec;
     tm_tmp = localtime(&t);
     tobj->tm = *tm_tmp;
@@ -553,6 +586,7 @@ time_gmtime(time)
     if (tobj->tm_got && tobj->gmt) {
 	return time;
     }
+    time_modify(time);
     t = tobj->tv.tv_sec;
     tm_tmp = gmtime(&t);
     tobj->tm = *tm_tmp;
@@ -599,13 +633,10 @@ time_to_s(time)
     if (tobj->tm_got == 0) {
 	time_get_tm(time, tobj->gmt);
     }
-#ifndef HAVE_TM_ZONE
     if (tobj->gmt == 1) {
-	len = strftime(buf, 128, "%a %b %d %H:%M:%S GMT %Y", &tobj->tm);
+	len = strftime(buf, 128, "%a %b %d %H:%M:%S UTC %Y", &tobj->tm);
     }
-    else
-#endif
-    {
+    else {
 	len = strftime(buf, 128, "%a %b %d %H:%M:%S %Z %Y", &tobj->tm);
     }
     return rb_str_new(buf, len);
@@ -890,7 +921,7 @@ time_strftime(time, format)
     if (tobj->tm_got == 0) {
 	time_get_tm(time, tobj->gmt);
     }
-    fmt = str2cstr(format, &len);
+    fmt = rb_str2cstr(format, &len);
     if (len == 0) {
 	rb_warning("strftime called with empty format string");
     }
@@ -921,7 +952,11 @@ time_s_times(obj)
 {
 #ifdef HAVE_TIMES
 #ifndef HZ
-#define HZ 60 /* Universal constant :-) */
+# ifdef CLK_TCK
+#   define HZ CLK_TCK
+# else
+#   define HZ 60
+# endif
 #endif /* HZ */
     struct tms buf;
 
@@ -931,22 +966,9 @@ time_s_times(obj)
 			 rb_float_new((double)buf.tms_stime / HZ),
 			 rb_float_new((double)buf.tms_cutime / HZ),
 			 rb_float_new((double)buf.tms_cstime / HZ));
-#else
-#ifdef NT
-    FILETIME create, exit, kernel, user;
-    HANDLE hProc;
-
-    hProc = GetCurrentProcess();
-    GetProcessTimes(hProc,&create, &exit, &kernel, &user);
-    return rb_struct_new(S_Tms,
-      rb_float_new((double)(kernel.dwHighDateTime*2e32+kernel.dwLowDateTime)/2e6),
-      rb_float_new((double)(user.dwHighDateTime*2e32+user.dwLowDateTime)/2e6),
-      rb_float_new((double)0),
-      rb_float_new((double)0));
 #else
     rb_notimplement();
 #endif
-#endif
 }
 
 static VALUE
@@ -1000,7 +1022,7 @@ time_load(klass, str)
     struct tm tm;
     int i;
 
-    buf = str2cstr(str, &i);
+    buf = rb_str2cstr(str, &i);
     if (i != 8) {
 	rb_raise(rb_eTypeError, "marshaled time format differ");
     }
@@ -1037,11 +1059,12 @@ Init_Time()
     rb_include_module(rb_cTime, rb_mComparable);
 
     rb_define_singleton_method(rb_cTime, "now", time_s_now, 0);
-    rb_define_singleton_method(rb_cTime, "new", time_s_now, 0);
-    rb_define_singleton_method(rb_cTime, "at", time_s_at, 1);
-    rb_define_singleton_method(rb_cTime, "gm", time_s_timegm, -1);
-    rb_define_singleton_method(rb_cTime, "local", time_s_timelocal, -1);
-    rb_define_singleton_method(rb_cTime, "mktime", time_s_timelocal, -1);
+    rb_define_singleton_method(rb_cTime, "new", time_s_new, -1);
+    rb_define_singleton_method(rb_cTime, "at", time_s_at, -1);
+    rb_define_singleton_method(rb_cTime, "utc", time_s_mkutc, -1);
+    rb_define_singleton_method(rb_cTime, "gm", time_s_mkutc, -1);
+    rb_define_singleton_method(rb_cTime, "local", time_s_mktime, -1);
+    rb_define_singleton_method(rb_cTime, "mktime", time_s_mktime, -1);
 
     rb_define_singleton_method(rb_cTime, "times", time_s_times, 0);
 
@@ -1054,6 +1077,7 @@ Init_Time()
 
     rb_define_method(rb_cTime, "localtime", time_localtime, 0);
     rb_define_method(rb_cTime, "gmtime", time_gmtime, 0);
+    rb_define_method(rb_cTime, "utc", time_gmtime, 0);
     rb_define_method(rb_cTime, "ctime", time_asctime, 0);
     rb_define_method(rb_cTime, "asctime", time_asctime, 0);
     rb_define_method(rb_cTime, "to_s", time_to_s, 0);
@@ -1076,7 +1100,8 @@ Init_Time()
     rb_define_method(rb_cTime, "isdst", time_isdst, 0);
     rb_define_method(rb_cTime, "zone", time_zone, 0);
 
-    rb_define_method(rb_cTime, "gmt?", time_gmt_p, 0);
+    rb_define_method(rb_cTime, "utc?", time_utc_p, 0);
+    rb_define_method(rb_cTime, "gmt?", time_utc_p, 0);
 
     rb_define_method(rb_cTime, "tv_sec", time_to_i, 0);
     rb_define_method(rb_cTime, "tv_usec", time_usec, 0);
