@@ -1,22 +1,22 @@
-/************************************************
+/**********************************************************************
 
   variable.c -
 
   $Author: matz $
-  $Date: 2000/03/23 04:14:22 $
+  $Date: 2000/09/15 06:00:28 $
   created at: Tue Apr 19 23:55:15 JST 1994
 
-************************************************/
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
+
+**********************************************************************/
 
 #include "ruby.h"
 #include "env.h"
 #include "node.h"
 #include "st.h"
 
-#ifndef strdup
-char *strdup();
-#endif
-
 static st_table *rb_global_tbl;
 st_table *rb_class_tbl;
 
@@ -49,8 +49,8 @@ fc_i(key, value, res)
     name = rb_id2name(key);
     if (res->path) {
 	path = rb_str_dup(res->path);
-	rb_str_cat(path, "::", 2);
-	rb_str_cat(path, name, strlen(name));
+	rb_str_cat2(path, "::");
+	rb_str_cat2(path, name);
     }
     else {
 	path = rb_str_new2(name);
@@ -129,7 +129,7 @@ classname(klass)
 	ID classid = rb_intern("__classid__");
 
 	if (st_lookup(ROBJECT(klass)->iv_tbl, classid, &path)) {
-	    path = rb_str_new2(rb_id2name(FIX2INT(path)));
+	    path = rb_str_new2(rb_id2name(SYM2ID(path)));
 	    st_insert(ROBJECT(klass)->iv_tbl, classpath, path);
 	    st_delete(RCLASS(klass)->iv_tbl, &classid, 0);
 	}
@@ -185,8 +185,8 @@ rb_set_class_path(klass, under, name)
     }
     else {
 	str = rb_str_dup(rb_class_path(under));
-	rb_str_cat(str, "::", 2);
-	rb_str_cat(str, name, strlen(name));
+	rb_str_cat2(str, "::");
+	rb_str_cat2(str, name);
     }
     rb_iv_set(klass, "__classpath__", str);
 }
@@ -216,7 +216,7 @@ rb_name_class(klass, id)
     VALUE klass;
     ID id;
 {
-    rb_iv_set(klass, "__classid__", INT2FIX(id));
+    rb_iv_set(klass, "__classid__", ID2SYM(id));
 }
 
 static st_table *autoload_tbl = 0;
@@ -720,7 +720,6 @@ rb_generic_ivar_table(obj)
     VALUE obj;
 {
     st_table *tbl;
-    VALUE val;
 
     if (!generic_iv_tbl) return 0;
     if (!st_lookup(generic_iv_tbl, obj, &tbl)) return 0;
@@ -871,7 +870,7 @@ rb_ivar_get(obj, id)
 {
     VALUE val;
 
-    if (!FL_TEST(obj, FL_TAINT) && rb_safe_level() >= 4)
+    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't access instance variable");
     switch (TYPE(obj)) {
       case T_OBJECT:
@@ -886,7 +885,7 @@ rb_ivar_get(obj, id)
 	break;
     }
     if (ruby_verbose) {
-	rb_warning("instance var %s not initialized", rb_id2name(id));
+	rb_warning("instance variable %s not initialized", rb_id2name(id));
     }
     return Qnil;
 }
@@ -897,8 +896,9 @@ rb_ivar_set(obj, id, val)
     ID id;
     VALUE val;
 {
-    if (!FL_TEST(obj, FL_TAINT) && rb_safe_level() >= 4)
+    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify instance variable");
+    if (OBJ_FROZEN(obj)) rb_error_frozen("object");
     switch (TYPE(obj)) {
       case T_OBJECT:
       case T_CLASS:
@@ -949,10 +949,11 @@ VALUE
 rb_obj_instance_variables(obj)
     VALUE obj;
 {
-    VALUE ary = rb_ary_new();
+    VALUE ary;
 
-    if (!FL_TEST(obj, FL_TAINT) && rb_safe_level() >= 4)
+    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't get metainfo");
+    ary = rb_ary_new();
     switch (TYPE(obj)) {
       case T_OBJECT:
       case T_CLASS:
@@ -982,8 +983,9 @@ rb_obj_remove_instance_variable(obj, name)
     VALUE val = Qnil;
     ID id = rb_to_id(name);
 
-    if (!FL_TEST(obj, FL_TAINT) && rb_safe_level() >= 4)
+    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify instance variable");
+    if (OBJ_FROZEN(obj)) rb_error_frozen("object");
     if (!rb_is_instance_id(id)) {
 	rb_raise(rb_eNameError, "`%s' is not an instance variable",
 		 rb_id2name(id));
@@ -1005,6 +1007,25 @@ rb_obj_remove_instance_variable(obj, name)
     return val;
 }
 
+static int
+top_const_get(id, klassp)
+    ID id;
+    VALUE *klassp;
+{
+    VALUE value;
+
+    /* pre-defined class */
+    if (st_lookup(rb_class_tbl, id, klassp)) return Qtrue;
+
+    /* autoload */
+    if (autoload_tbl && st_lookup(autoload_tbl, id, 0)) {
+	rb_autoload_load(id);
+	*klassp = rb_const_get(rb_cObject, id);
+	return Qtrue;
+    }
+    return Qfalse;
+}
+
 VALUE
 rb_const_get_at(klass, id)
     VALUE klass;
@@ -1015,8 +1036,8 @@ rb_const_get_at(klass, id)
     if (RCLASS(klass)->iv_tbl && st_lookup(RCLASS(klass)->iv_tbl, id, &value)) {
 	return value;
     }
-    if (klass == rb_cObject) {
-	return rb_const_get(klass, id);
+    if (klass == rb_cObject && top_const_get(id, &value)) {
+	return value;
     }
     rb_raise(rb_eNameError, "uninitialized constant %s::%s",
 	     RSTRING(rb_class_path(klass))->ptr,
@@ -1024,6 +1045,19 @@ rb_const_get_at(klass, id)
     return Qnil;		/* not reached */
 }
 
+void
+rb_autoload_load(id)
+    ID id;
+{
+    char *modname;
+    VALUE module;
+
+    st_delete(autoload_tbl, &id, &modname);
+    module = rb_str_new2(modname);
+    free(modname);
+    FL_UNSET(module, FL_TAINT);
+    rb_f_require(Qnil, module);
+}
 
 VALUE
 rb_const_get(klass, id)
@@ -1038,28 +1072,13 @@ rb_const_get(klass, id)
 	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,&value)) {
 	    return value;
 	}
+	if (tmp == rb_cObject && top_const_get(id, &value)) return value;
 	tmp = RCLASS(tmp)->super;
     }
     if (BUILTIN_TYPE(klass) == T_MODULE) {
 	return rb_const_get(rb_cObject, id);
     }
 
-    /* pre-defined class */
-    if (st_lookup(rb_class_tbl, id, &value)) return value;
-
-    /* autoload */
-    if (autoload_tbl && st_lookup(autoload_tbl, id, 0)) {
-	char *modname;
-	VALUE module;
-
-	st_delete(autoload_tbl, &id, &modname);
-	module = rb_str_new2(modname);
-	FL_UNSET(module, FL_TAINT);
-	free(modname);
-	rb_f_require(Qnil, module);
-	return rb_const_get(klass, id);
-    }
-
     /* Uninitialized constant */
     if (klass && klass != rb_cObject)
 	rb_raise(rb_eNameError, "uninitialized constant %s::%s",
@@ -1072,7 +1091,7 @@ rb_const_get(klass, id)
 }
 
 static int
-const_i(key, value, ary)
+sv_i(key, value, ary)
     ID key;
     VALUE value;
     VALUE ary;
@@ -1096,6 +1115,9 @@ rb_mod_remove_const(mod, name)
     if (!rb_is_const_id(id)) {
 	rb_raise(rb_eNameError, "`%s' is not constant", rb_id2name(id));
     }
+    if (!OBJ_TAINTED(mod) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: can't remove constant");
+    if (OBJ_FROZEN(mod)) rb_error_frozen("class/module");
 
     if (RCLASS(mod)->iv_tbl && st_delete(ROBJECT(mod)->iv_tbl, &id, &val)) {
 	return val;
@@ -1126,13 +1148,13 @@ VALUE
 rb_mod_const_at(mod, ary)
     VALUE mod, ary;
 {
-    if (!FL_TEST(mod, FL_TAINT) && rb_safe_level() >= 4)
+    if (!OBJ_TAINTED(mod) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't get metainfo");
     if (RCLASS(mod)->iv_tbl) {
-	st_foreach(RCLASS(mod)->iv_tbl, const_i, ary);
+	st_foreach(RCLASS(mod)->iv_tbl, sv_i, ary);
     }
     if ((VALUE)mod == rb_cObject) {
-	st_foreach(rb_class_tbl, const_i, ary);
+	st_foreach(rb_class_tbl, sv_i, ary);
 	if (autoload_tbl) {
 	    st_foreach(autoload_tbl, autoload_i, ary);
 	}
@@ -1140,27 +1162,26 @@ rb_mod_const_at(mod, ary)
     return ary;
 }
 
-VALUE
-rb_mod_constants(mod)
-    VALUE mod;
-{
-    return rb_mod_const_at(mod, rb_ary_new());
-}
-
 VALUE
 rb_mod_const_of(mod, ary)
     VALUE mod;
     VALUE ary;
 {
-    rb_mod_const_at(mod, ary);
     for (;;) {
+	rb_mod_const_at(mod, ary);
 	mod = RCLASS(mod)->super;
 	if (!mod) break;
-	rb_mod_const_at(mod, ary);
     }
     return ary;
 }
 
+VALUE
+rb_mod_constants(mod)
+    VALUE mod;
+{
+    return rb_mod_const_of(mod, rb_ary_new());
+}
+
 int
 rb_const_defined_at(klass, id)
     VALUE klass;
@@ -1205,22 +1226,76 @@ rb_const_defined(klass, id)
     return rb_autoload_defined(id);
 }
 
+static void
+mod_av_set(klass, id, val, dest, once)
+    VALUE klass;
+    ID id;
+    VALUE val;
+    char *dest;
+    int once;
+{
+    if (!OBJ_TAINTED(klass) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: can't set %s", dest);
+    if (OBJ_FROZEN(klass)) rb_error_frozen("class/module");
+    if (!RCLASS(klass)->iv_tbl) {
+	RCLASS(klass)->iv_tbl = st_init_numtable();
+    }
+    else if (once && st_lookup(RCLASS(klass)->iv_tbl, id, 0)) {
+	rb_warn("already initialized %s %s", dest, rb_id2name(id));
+    }
+
+    st_insert(RCLASS(klass)->iv_tbl, id, val);
+}
+    
 void
 rb_const_set(klass, id, val)
     VALUE klass;
     ID id;
     VALUE val;
 {
-    if (!FL_TEST(klass, FL_TAINT) && rb_safe_level() >= 4)
-	rb_raise(rb_eSecurityError, "Insecure: can't set constant");
-    if (!RCLASS(klass)->iv_tbl) {
-	RCLASS(klass)->iv_tbl = st_init_numtable();
+    mod_av_set(klass, id, val, "constant", Qtrue);
+}
+
+void
+rb_const_assign(klass, id, val)
+    VALUE klass;
+    ID id;
+    VALUE val;
+{
+    VALUE tmp = klass;
+    
+    while (tmp) {
+	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,0)) {
+	    st_insert(RCLASS(tmp)->iv_tbl, id, val);
+	    return;
+	}
+	tmp = RCLASS(tmp)->super;
     }
-    else if (st_lookup(RCLASS(klass)->iv_tbl, id, 0)) {
-	rb_warn("already initialized constant %s", rb_id2name(id));
+    /* pre-defined class */
+    if (st_lookup(rb_class_tbl, id, 0)) {
+	st_delete(rb_class_tbl, id, 0);
+	st_insert(RCLASS(rb_cObject)->iv_tbl, id, val);
+	return;
     }
 
-    st_insert(RCLASS(klass)->iv_tbl, id, val);
+    /* autoload */
+    if (autoload_tbl && st_lookup(autoload_tbl, id, 0)) {
+	char *modname;
+
+	st_delete(autoload_tbl, &id, &modname);
+	free(modname);
+	st_insert(RCLASS(rb_cObject)->iv_tbl, id, val);
+	return;
+    }
+
+    /* Uninitialized constant */
+    if (klass && klass != rb_cObject)
+	rb_raise(rb_eNameError, "uninitialized constant %s::%s",
+		 RSTRING(rb_class_path(klass))->ptr,
+		 rb_id2name(id));
+    else {
+	rb_raise(rb_eNameError, "uninitialized constant %s",rb_id2name(id));
+    }
 }
 
 void
@@ -1248,6 +1323,204 @@ rb_define_global_const(name, val)
     rb_define_const(rb_cObject, name, val);
 }
 
+void
+rb_cvar_set(klass, id, val)
+    VALUE klass;
+    ID id;
+    VALUE val;
+{
+    VALUE tmp;
+
+    tmp = klass;
+    while (tmp) {
+	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,0)) {
+	    if (!OBJ_TAINTED(tmp) && rb_safe_level() >= 4)
+		rb_raise(rb_eSecurityError, "Insecure: can't modify class variable");
+	    st_insert(RCLASS(tmp)->iv_tbl,id,val);
+	    return;
+	}
+	tmp = RCLASS(tmp)->super;
+    }
+
+    rb_raise(rb_eNameError,"uninitialized class variable %s in %s",
+	     rb_id2name(id), rb_class2name(klass));
+}
+
+void
+rb_cvar_declare(klass, id, val)
+    VALUE klass;
+    ID id;
+    VALUE val;
+{
+    VALUE tmp;
+
+    tmp = klass;
+    while (tmp) {
+	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,0)) {
+	    if (!OBJ_TAINTED(tmp) && rb_safe_level() >= 4)
+		rb_raise(rb_eSecurityError, "Insecure: can't modify class variable");
+	    st_insert(RCLASS(tmp)->iv_tbl,id,val);
+	    return;
+	}
+	tmp = RCLASS(tmp)->super;
+    }
+
+    mod_av_set(klass, id, val, "class variable", Qfalse);
+}
+
+VALUE
+rb_cvar_get(klass, id)
+    VALUE klass;
+    ID id;
+{
+    VALUE value;
+    VALUE tmp;
+
+    tmp = klass;
+    while (tmp) {
+	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,&value)) {
+	    return value;
+	}
+	tmp = RCLASS(tmp)->super;
+    }
+
+    rb_raise(rb_eNameError,"uninitialized class variable %s in %s",
+	     rb_id2name(id), rb_class2name(klass));
+    return Qnil;		/* not reached */
+}
+
+int
+rb_cvar_defined(klass, id)
+    VALUE klass;
+    ID id;
+{
+    VALUE tmp;
+
+    tmp = klass;
+    while (tmp) {
+	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,0)) {
+	    return Qtrue;
+	}
+	tmp = RCLASS(tmp)->super;
+    }
+
+    return Qfalse;
+}
+
+int
+rb_cvar_defined_singleton(obj, id)
+    VALUE obj;
+    ID id;
+{
+    switch (TYPE(obj)) {
+      case T_MODULE:
+      case T_CLASS:
+	break;
+      default:
+	obj = CLASS_OF(obj);
+	break;
+    }
+    return rb_cvar_defined(obj, id);
+}
+
+void
+rb_cvar_set_singleton(obj, id, value)
+    VALUE obj;
+    ID id;
+    VALUE value;
+{
+    switch (TYPE(obj)) {
+      case T_MODULE:
+      case T_CLASS:
+	break;
+      default:
+	obj = CLASS_OF(obj);
+	break;
+    }
+    rb_cvar_set(obj, id, value);
+}
+
+VALUE
+rb_cvar_get_singleton(obj, id)
+    VALUE obj;
+    ID id;
+{
+    switch (TYPE(obj)) {
+      case T_MODULE:
+      case T_CLASS:
+	break;
+      default:
+	obj = CLASS_OF(obj);
+	break;
+    }
+    return rb_cvar_get(obj, id);
+}
+
+void
+rb_cv_set(klass, name, val)
+    VALUE klass;
+    const char *name;
+    VALUE val;
+{
+    rb_cvar_set(klass, rb_intern(name), val);
+}
+
+VALUE
+rb_cv_get(klass, name)
+    VALUE klass;
+    const char *name;
+{
+    return rb_cvar_get(klass, rb_intern(name));
+}
+
+void
+rb_define_class_variable(klass, name, val)
+    VALUE klass;
+    const char *name;
+    VALUE val;
+{
+    ID id = rb_intern(name);
+
+    if (!rb_is_class_id(id)) {
+	rb_raise(rb_eNameError, "wrong class variable name %s", name);
+    }
+    rb_cvar_declare(klass, id, val);
+}
+
+static int
+cv_i(key, value, ary)
+    ID key;
+    VALUE value;
+    VALUE ary;
+{
+    if (rb_is_class_id(key)) {
+	VALUE kval = rb_str_new2(rb_id2name(key));
+	if (!rb_ary_includes(ary, kval)) {
+	    rb_ary_push(ary, kval);
+	}
+    }
+    return ST_CONTINUE;
+}
+
+VALUE
+rb_mod_class_variables(obj)
+    VALUE obj;
+{
+    VALUE ary = rb_ary_new();
+
+    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: can't get metainfo");
+
+    for (;;) {
+	if (RCLASS(obj)->iv_tbl) {
+	    st_foreach(RCLASS(obj)->iv_tbl, cv_i, ary);
+	}
+	obj = RCLASS(obj)->super;
+	if (!obj) break;
+    }
+    return ary;
+}
+
 VALUE
 rb_iv_get(obj, name)
     VALUE obj;
