@@ -155,17 +155,6 @@ flock(int fd, int oper)
 //#undef const
 //FILE *fdopen(int, const char *);
 
-#if 0
-void
-sleep(unsigned int len)
-{
-	time_t end;
-
-	end = time((time_t *)0) + len;
-	while (time((time_t *)0) < end)
-		;
-}
-#endif
 
 //
 // Initialization stuff
@@ -195,7 +184,7 @@ NtInitialize(int *argc, char ***argv) {
 char *getlogin()
 {
     char buffer[200];
-    int len = 200;
+    DWORD len = 200;
     extern char *NTLoginName;
 
     if (NTLoginName == NULL) {
@@ -588,7 +577,7 @@ int
 mypclose(FILE *fp)
 {
     int i;
-    int exitcode;
+    DWORD exitcode;
 
     Sleep(100);
     for (i = 0; i < MYPOPENSIZE; i++) {
@@ -772,6 +761,7 @@ NtFreeCmdLine(void)
 // any existing files, just leave it in the list.
 //
 
+#if 0
 void
 NtCmdGlob (NtCmdLineElement *patt)
 {
@@ -845,6 +835,69 @@ NtCmdGlob (NtCmdLineElement *patt)
 	free(patt->str);
     // free(patt);  //TODO:  memory leak occures here. we have to fix it.
 }
+#else
+typedef struct {
+    NtCmdLineElement *head;
+    NtCmdLineElement *tail;
+} ListInfo;
+
+static void
+insert(char *path, ListInfo *listinfo)
+{
+    NtCmdLineElement *tmpcurr;
+
+    tmpcurr = ALLOC(NtCmdLineElement);
+    MEMZERO(tmpcurr, NtCmdLineElement, 1);
+    tmpcurr->len = strlen(path);
+    tmpcurr->str = ALLOC_N(char, tmpcurr->len + 1);
+    tmpcurr->flags |= NTMALLOC;
+    strcpy(tmpcurr->str, path);
+    if (listinfo->tail) {
+	listinfo->tail->next = tmpcurr;
+	tmpcurr->prev = listinfo->tail;
+	listinfo->tail = tmpcurr;
+    }
+    else {
+	listinfo->tail = listinfo->head = tmpcurr;
+    }
+}
+
+#ifdef HAVE_SYS_PARAM_H
+# include <sys/param.h>
+#else
+# define MAXPATHLEN 512
+#endif
+
+void
+NtCmdGlob (NtCmdLineElement *patt)
+{
+    ListInfo listinfo;
+    char buffer[MAXPATHLEN], *buf = buffer;
+
+    listinfo.head = listinfo.tail = 0;
+
+    if (patt->len >= MAXPATHLEN)
+	buf = ruby_xmalloc(patt->len + 1);
+
+    strncpy(buf, patt->str, patt->len);
+    buf[patt->len] = 0;
+    rb_glob(buf, insert, (VALUE)&listinfo);
+    if (buf != buffer)
+	free(buf);
+
+    if (listinfo.head && listinfo.tail) {
+	listinfo.head->prev = patt->prev;
+	listinfo.tail->next = patt->next;
+	if (listinfo.head->prev)
+	    listinfo.head->prev->next = listinfo.head;
+	if (listinfo.tail->next)
+	    listinfo.tail->next->prev = listinfo.tail;
+    }
+    if (patt->flags & NTMALLOC)
+	free(patt->str);
+    // free(patt);  //TODO:  memory leak occures here. we have to fix it.
+}
+#endif
 
 // 
 // Check a command string to determine if it has I/O redirection
@@ -914,6 +967,8 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
 	return 0;
     }
 
+    cmdline = strdup(cmdline);
+
     //
     // strip trailing white space
     //
@@ -923,36 +978,6 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
         --ptr;
     *++ptr = '\0';
 
-    //
-    // check for newlines and formfeeds. If we find any, make a new
-    // command string that replaces them with escaped sequences (\n or \f)
-    //
-
-    for (ptr = cmdline, newline = 0; *ptr; ptr++) {
-	if (*ptr == '\n' || *ptr == '\f')
-	    newline++;
-    }
-
-    if (newline) {
-	base = ALLOC_N(char, strlen(cmdline) + 1 + newline + slashes);
-	if (base == NULL) {
-	    fprintf(stderr, "malloc failed!\n");
-	    return 0;
-	}
-	for (i = 0, ptr = base; (unsigned) i < strlen(cmdline); i++) {
-	    switch (cmdline[i]) {
-	      case '\n':
-		*ptr++ = '\\';
-		*ptr++ = 'n';
-		break;
-	      default:
-		*ptr++ = cmdline[i];
-	    }
-	}
-	*ptr = '\0';
-	cmdline = base;
-	need_free++;
-    }
 
     //
     // Ok, parse the command line, building a list of CmdLineElements.
@@ -983,6 +1008,9 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
 	    //
 
 	    switch (*ptr) {
+	      case '\\':
+	        if (ptr[1] == '"') ptr++;
+	        break;
 	      case ' ':
 	      case '\t':
 #if 0
@@ -1067,12 +1095,6 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
 	//
 
 	curr = ALLOC(NtCmdLineElement);
-	if (curr == NULL) {
-	    NtFreeCmdLine();
-	    fprintf(stderr, "Out of memory!!\n");
-	    *vec = NULL;
-	    return 0;
-	}
 	memset (curr, 0, sizeof(*curr));
 
 	len = ptr - base;
@@ -1082,9 +1104,19 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
 	// we can remove them.
 	//
 
-	if (InputCmd &&
-	    ((base[0] == '\"' && base[len-1] == '\"') ||
-	     (base[0] == '\'' && base[len-1] == '\''))) {
+	if (InputCmd && (base[0] == '\"' && base[len-1] == '\"')) {
+	    char *p;
+	    base++;
+	    len -= 2;
+	    base[len] = 0;
+	    for (p = base; p < base + len; p++) {
+		if ((p[0] == '\\' || p[0] == '\"') && p[1] == '"') {
+		    strcpy(p, p + 1);
+		    len--;
+		}
+	    }
+	}
+	else if (InputCmd && (base[0] == '\'' && base[len-1] == '\'')) {
 	    base++;
 	    len -= 2;
 	}
@@ -1133,12 +1165,6 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
 
     len = (elements+1)*sizeof(char *) + strsz;
     buffer = ALLOC_N(char, len);
-    if (buffer == NULL) {
-	fprintf(stderr, "Out of memory!!\n");
-	NtFreeCmdLine();
-	*vec = NULL;
-	return 0;
-    }
     
     memset (buffer, 0, len);
 
@@ -1166,11 +1192,12 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
     }
     NtFreeCmdLine();
     *vec = (char **) buffer;
+    free(cmdline);
     return elements;
 }
 
 
-#if 1
+#if !defined __MINGW32__
 //
 // UNIX compatible directory access functions for NT
 //
@@ -1184,7 +1211,7 @@ NtMakeCmdVector (char *cmdline, char ***vec, int InputCmd)
 //
 
 DIR *
-opendir(char *filename)
+opendir(const char *filename)
 {
     DIR            *p;
     long            len;
@@ -1525,28 +1552,39 @@ valid_filename(char *s)
 // if we can prevent perl from trying to do stdio on sockets.
 //
 
-EXTERN_C int __cdecl _alloc_osfhnd(void);
-EXTERN_C int __cdecl _set_osfhnd(int fh, long value);
+//EXTERN_C int __cdecl _alloc_osfhnd(void);
+//EXTERN_C int __cdecl _set_osfhnd(int fh, long value);
 EXTERN_C void __cdecl _lock_fhandle(int);
 EXTERN_C void __cdecl _unlock_fhandle(int);
 EXTERN_C void __cdecl _unlock(int);
 
+#if defined _MT || defined __MSVCRT__
+#define MSVCRT_THREADS
+#endif
+
 typedef struct	{
     long osfhnd;    /* underlying OS file HANDLE */
     char osfile;    /* attributes of file (e.g., open in text mode?) */
     char pipech;    /* one char buffer for handles opened on pipes */
-#if defined (_MT) && !defined (DLL_FOR_WIN32S)
+#ifdef MSVCRT_THREADS
     int lockinitflag;
     CRITICAL_SECTION lock;
-#endif  /* defined (_MT) && !defined (DLL_FOR_WIN32S) */
+#endif
 }	ioinfo;
 
+#if !defined _CRTIMP
+#define _CRTIMP __declspec(dllimport)
+#endif
+
 EXTERN_C _CRTIMP ioinfo * __pioinfo[];
 
 #define IOINFO_L2E			5
 #define IOINFO_ARRAY_ELTS	(1 << IOINFO_L2E)
 #define _pioinfo(i)	(__pioinfo[i >> IOINFO_L2E] + (i & (IOINFO_ARRAY_ELTS - 1)))
-#define _osfile(i)	(_pioinfo(i)->osfile)
+
+#define _osfhnd(i)  (_pioinfo(i)->osfhnd)
+#define _osfile(i)  (_pioinfo(i)->osfile)
+#define _pipech(i)  (_pioinfo(i)->pipech)
 
 #define FOPEN			0x01	/* file handle open */
 #define FNOINHERIT		0x10	/* file handle opened O_NOINHERIT */
@@ -1554,6 +1592,22 @@ EXTERN_C _CRTIMP ioinfo * __pioinfo[];
 #define FDEV			0x40	/* file handle refers to device */
 #define FTEXT			0x80	/* file handle is in text mode */
 
+#define _set_osfhnd(fh, osfh) (void)(_osfhnd(fh) = osfh)
+
+static int
+_alloc_osfhnd(void)
+{
+    HANDLE hF = CreateFile("NUL", 0, 0, NULL, OPEN_ALWAYS, 0, NULL);
+    int fh = _open_osfhandle((long)hF, 0);
+    CloseHandle(hF);
+    if (fh == -1)
+        return fh;
+#ifdef MSVCRT_THREADS
+    EnterCriticalSection(&(_pioinfo(fh)->lock));
+#endif
+    return fh;
+}
+
 static int
 my_open_osfhandle(long osfhandle, int flags)
 {
@@ -1585,7 +1639,9 @@ my_open_osfhandle(long osfhandle, int flags)
     fileflags |= FOPEN;		/* mark as open */
 
     _osfile(fh) = fileflags;	/* set osfile entry */
-//    _unlock_fhandle(fh);
+#ifdef MSVCRT_THREADS
+    LeaveCriticalSection(&_pioinfo(fh)->lock);
+#endif
 
     return fh;			/* return handle */
 }
@@ -1632,7 +1688,9 @@ myfdopen (int fd, const char *mode)
 void
 myfdclose(FILE *fp)
 {
+#if !defined __MINGW32__
     _free_osfhnd(fileno(fp));
+#endif
     fclose(fp);
 }
 
@@ -1649,7 +1707,9 @@ char *
 mystrerror(int e) 
 {
     static char buffer[512];
+#if !defined __MINGW32__
     extern int sys_nerr;
+#endif
     DWORD source = 0;
 
     if (e < 0 || e > sys_nerr) {
@@ -2419,3 +2479,33 @@ done:
 
     return res;
 }
+
+static long
+filetime_to_clock(FILETIME *ft)
+{
+    __int64 qw = ft->dwHighDateTime;
+    qw <<= 32;
+    qw |= ft->dwLowDateTime;
+    qw /= 10000;  /* File time ticks at 0.1uS, clock at 1mS */
+    return (long) qw;
+}
+
+int
+mytimes(struct tms *tmbuf)
+{
+    FILETIME create, exit, kernel, user;
+
+    if (GetProcessTimes(GetCurrentProcess(),&create, &exit, &kernel, &user)) {
+	tmbuf->tms_utime = filetime_to_clock(&user);
+	tmbuf->tms_stime = filetime_to_clock(&kernel);
+	tmbuf->tms_cutime = 0;
+	tmbuf->tms_cstime = 0;
+    }
+    else {
+	tmbuf->tms_utime = clock();
+	tmbuf->tms_stime = 0;
+	tmbuf->tms_cutime = 0;
+	tmbuf->tms_cstime = 0;
+    }
+    return 0;
+}
