@@ -9,6 +9,7 @@
  *
  */
 
+#undef EXTERN
 #if defined(IMPORT)
 #define EXTERN extern __declspec(dllimport)
 #elif defined(EXPORT)
@@ -163,6 +164,12 @@ extern "C++" {
 #define strncasecmp _strnicmp
 /* these are defined in nt.c */
 
+#ifdef __MINGW32__
+struct timezone {
+  int tz_minuteswest;
+  int tz_dsttime;
+};
+#endif
 extern int NtMakeCmdVector(char *, char ***, int);
 extern void NtInitialize(int *, char ***);
 extern char *NtGetLib(void);
@@ -407,4 +414,16 @@ extern char *mystrerror(int);
 #endif
 #define rename myrename
 
+struct tms {
+	long	tms_utime;
+	long	tms_stime;
+	long	tms_cutime;
+	long	tms_cstime;
+};
+
+#ifdef times
+#undef times
+#endif
+#define times mytimes
+
 #endif
